'''
Creating a new HDF5 dataset for MODIS MOD16, MOD17 calibration (in the VIIRS
era), based on the SMAP L4C calibration dataset (356 eddy covariance sites
from the FLUXNET2015/ La Thuille collection).

The output HDF5 field structure looks like (names with a "/" are groups, all
others are data fields):

    coordinates/
    FLUXNET/
      site_id
      GPP
      NEE
      RECO
    MOD15A2H/
      fPAR
    time
'''

import h5py
import numpy as np

FLUXNET_HDF = '/anx_lagr3/arthur.endsley/SMAP_L4C/calibration/Fluxnet2015_LaThuile_tower_data_for_356_sites.h5'
L4C_HDF = '/anx_lagr3/arthur.endsley/SMAP_L4C/calibration/v6_Y2021/L4_C_tower_site_drivers_NRv9-1_for_356_sites.h5'
OUTPUT_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v1.h5'

def main():
    'Creates a copy of the FLUXNET2015/ La Thuille calibration data'
    target_hdf = h5py.File(OUTPUT_HDF, 'a')
    for group in (('FLUXNET', 'MOD15A2H', 'coordinates')):
        if group not in target_hdf.keys():
            target_hdf.create_group(group)
    # Copy the FLUXNET data over
    with h5py.File(FLUXNET_HDF, 'r') as hdf:
        for field in ('GPP', 'NEE', 'RECO'):
            if f'FLUXNET/{field}' in target_hdf.keys():
                continue # Skip fields already copied
            dataset = target_hdf.create_dataset(
                f'FLUXNET/{field}', hdf[field].shape, np.float32, hdf[field][:])
            dataset.attrs['description'] = f'Observed {field} flux at FLUXNET eddy covariance tower sites'
            dataset.attrs['units'] = 'g C m-2 day-1'
    # Copy the site ancillary data over
    with h5py.File(L4C_HDF, 'r') as hdf:
        # Write the site IDs...this, unfortunately, has to be verbose
        if 'FLUXNET/site_id' not in target_hdf.keys():
            dt = h5py.string_dtype(encoding = 'utf-8')
            sites = hdf['FLUXNET/site_id'][:].tolist()
            dataset = target_hdf.create_dataset('site_id', (len(sites),), dt)
            for i, site_id in enumerate(sites):
                dataset[i] = site_id
        if 'time' not in target_hdf.keys():
            time = hdf['time'][:,0:3] # Copy just Year, Month, Day
            dataset = target_hdf.create_dataset(
                'time', time.shape, time.dtype, time)
            dataset.attrs['description'] = 'Date: Year, Month, Day'
        if 'coordinates/lng_lat' not in hdf.keys():
            coords = hdf['coords/lng_lat'][:]
            dataset = target_hdf.create_dataset(
                'coordinates/lng_lat', coords.shape, coords.dtype, coords)
            dataset.attrs['description'] = 'Longitude, Latitude coordinates of FLUXNET eddy covariance tower site'


if __name__ == '__main__':
    main()
