'''
Extracts MERRA-2 surface meteorology for the 356 eddy covariance sites from
the FLUXNET2015/ La Thuille collection.

The output HDF5 field structure looks like (names with a "/" are groups, all
others are data fields):

    coordinates/
    MERRA2/
      Tmin
      QV10M
      QV10M_daytime
      QV10M_nighttime
      SWGDN
      ...
    FLUXNET/
      site_id
      GPP
      NEE
      RECO
    time

NOTE: This version uses sunrise and sunset times to compute the separate
sun-up and sun-down data fields.
'''

import datetime
import glob
import os
import re
import warnings
import netCDF4
import h5py
import numpy as np
import xarray as xr
from typing import Sequence
from functools import partial
from tqdm import tqdm
from suntransit import sunrise_sunset
from mod17.science import daynight_partition

EXPECTED_NUM_FILES = 6575 # Daily files from 2000-01-01 through 2017-12-31 (6575 days)
OUTPUT_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v1.h5'
MERRA2_DIR = '/anx_lagr4/MODIS_VIIRS/calibration/inputs/MERRA-2/'
MERRA2_DATASETS = [ # (dataset, filename template) pairs
    ('tavg1_2d_slv_Nx*', 'MERRA2_???.tavg1_2d_slv_Nx.*.SUB.nc'),
    ('tavg1_2d_slv_Nx',  'MERRA2_???.tavg1_2d_slv_Nx.*.SUB.nc'),
    ('tavg1_2d_rad_Nx',  'MERRA2_???.tavg1_2d_rad_Nx.*.SUB.nc'),
]
MERRA2_FIELDS = {
    'tavg1_2d_slv_Nx*': ('Tmin',),
    'tavg1_2d_slv_Nx':  ('T10M', 'QV10M', 'PS'),
    'tavg1_2d_rad_Nx':  ('SWGDN',)
}
MERRA2_DATE_RX = re.compile(r'^MERRA2\_\d{3}\.tavg1\_2d\_(slv|rad)\_Nx\.(?P<date>\d{8})\.SUB\.nc$')

# Flip data up/down (ud) or left/right?
#   Argument lats/ lons should be a 1D numpy.ndarray
flip_ud = lambda lats: np.argmax(lats) > np.argmin(lats)
flip_lr = lambda lons: np.argmax(lons) < np.argmin(lons)


def main():
    'Populates the calibration driver dataset with data from MERRA2'
    target_hdf = h5py.File(OUTPUT_HDF, 'a')
    if 'MERRA2' not in target_hdf.keys():
        target_hdf.create_group('MERRA2')
    shp = (EXPECTED_NUM_FILES, target_hdf['FLUXNET/site_id'].size)
    coords = target_hdf['coordinates/lng_lat'][:]
    coord_arrays = (
        xr.DataArray(coords[:,0], dims = 'points'),
        xr.DataArray(coords[:,1], dims = 'points'),
    )
    for dataset, template in MERRA2_DATASETS:
        print(f'Working on "{dataset}" dataset...')
        file_list = glob.glob(f'{MERRA2_DIR}/{dataset}/{template}')
            assert len(file_list) == EXPECTED_NUM_FILES,\
        f'Did not find {EXPECTED_NUM_FILES} files!'
        file_list.sort() # Get chronological order
        # Create a dataset for each field, including "_daytime" and
        #   "_nightttime" datasets
        nc0 = netCDF4.Dataset(file_list[0])
        for field in MERRA2_FIELDS[dataset]:
            for suffix in ('', '_daytime', '_nighttime'):
                if field == 'Tmin' and suffix != '':
                    break # No daytime or nighttime averages for Tmin
                if f'MERRA2/{field}{suffix}' not in target_hdf.keys():
                    dataset = target_hdf.create_dataset(
                        f'MERRA2/{field}{suffix}', shp, np.float32)
                    dataset.attrs['description'] = nc0.variables[field].long_name
                    dataset.attrs['units'] = nc0.variables[field].units
        nc0.close()
        for i, filename in enumerate(tqdm(file_list)):
            nc = netCDF4.Dataset(filename)
            # Get sunrise, sunset times for slicing indices on hourly
            #   MERRA-2 arrays
            updown = updown_array(nc, coords.swapaxes(0, 1)[::-1])
            with warnings.catch_warnings():
                warnings.simplefilter('ignore')
                data = extract(
                    filename, updown, coord_arrays, MERRA2_FIELDS[dataset])
            for field in data.keys():
                target_hdf[f'MERRA2/{field}'][i] = data[field][0]
                if field == 'Tmin':
                    break # No daytime/ nighttime averages
                target_hdf[f'MERRA2/{field}_daytime'][i] = data[field][1]
                target_hdf[f'MERRA2/{field}_nighttime'][i] = data[field][2]


def extract(
        filename: str, updown: np.ndarray, coords: tuple, fields: Sequence) -> list:
    '''
    Extracts data from a MERRA-2 NetCDF4 granule, based on the provided field.

    Parameters
    ----------
    filename : str
    updown : numpy.ndarray
        A (2 x ...) array with the hour of sunrise, sunset
    coords : tuple
        A tuple of two xarray.DataArray: (longitude, latitude)
    fields : Sequence
        Sequence of the field names to extract

    Returns
    -------
    list
        Sequence of numpy.ndarray instances; if the field has daytime or
        nighttime averages, the returned list might look like:
        [24-hour average, daytime average, nighttime average]
    '''
    nc = xr.open_dataset(filename)
    assert nc['time'].size == 24, 'Unexpected time axis length!'
    assert updown.shape[0] == 2, 'Unexpected shape of "updown" array argument'
    # Nearest-neighbour lookup for each site
    x, y = coords
    subset = nc.sel(lon = x, lat = y, method = 'nearest')
    results = []
    for field_name in fields:
        if field_name == 'Tmin':
            tmin = subset['T10M'].values.min(axis = 0)
            results.append((field_name, (tmin, None, None)))
        else:
            arr = subset[field_name].values
            avg = arr.mean(axis = 0) # 24-hour average
            # Calculate daytime and nighttime averages; if sun never rises or never
            #   sets within a 24-hour period, fill in NaN
            arr = daynight_partition(arr, updown, missing = [np.nan] * 2)
            results.append((field_name, (avg, arr[0], arr[1])))
    return dict(results)


def updown_array(nc: netCDF4.Dataset, coords: np.ndarray) -> np.ndarray:
    '''
    Returns
    -------
    numpy.ndarray
        A (2 x M x N) array, where the first axis enumerates the
        (sunrise, sunset) hour and M and N index rows, columns of the array
        spatial domain
    '''
    def sunrise_sunset2(*args, **kwargs):
        # Round down to the nearest hour
        return np.floor(np.array(sunrise_sunset(*args, **kwargs)))

    # Get the date from the filename...
    match = MERRA2_DATE_RX.match(os.path.basename(nc.filepath()))
    dt = datetime.datetime.strptime(match.groupdict()['date'], '%Y%m%d')
    # Calculate sunrise, sunset for each pixel (lat-lng pair)
    return np.apply_along_axis(partial(sunrise_sunset2, dt = dt), 0, coords)


if __name__ == '__main__':
    main()
