'''
DEPRECATED. It takes WAY too long to run already before we spend extra time
checking the radiation field for day length.

Extracts MERRA-2 surface meteorology for the 356 eddy covariance sites from
the FLUXNET2015/ La Thuille collection.

The output HDF5 field structure looks like (names with a "/" are groups, all
others are data fields):

    coordinates/
    MERRA2/
      Tmin
      QV10M
      QV10M_daytime
      QV10M_nighttime
      SWGDN
      ...
    FLUXNET/
      site_id
      GPP
      NEE
      RECO
    time

NOTE: This version uses the net down-welling short-wave radiation as an
indicator of whether the sun is above the horizon, for computing separate
daytime ("sun-up") and night-time quantities.
'''

import datetime
import glob
import os
import re
import warnings
import netCDF4
import h5py
import numpy as np
import xarray as xr
from typing import Sequence
from tqdm import tqdm
from mod17.science import daynight_partition

EXPECTED_NUM_FILES = 6575 # Daily files from 2000-01-01 through 2017-12-31 (6575 days)
OUTPUT_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v1.h5'
MERRA2_DIR = '/anx_lagr4/MODIS_VIIRS/calibration/inputs/MERRA2'
MERRA2_RAD_DATASET = ('tavg1_2d_rad_Nx', 'MERRA2_???.tavg1_2d_rad_Nx.*.SUB.nc')
MERRA2_DATASETS = [ # (dataset, filename template) pairs
    ('tavg1_2d_slv_Nx*', 'MERRA2_???.tavg1_2d_slv_Nx.*.SUB.nc'),
    ('tavg1_2d_slv_Nx',  'MERRA2_???.tavg1_2d_slv_Nx.*.SUB.nc'),
    MERRA2_RAD_DATASET,
]
MERRA2_FIELDS = {
    'tavg1_2d_rad_Nx':  ('SWGDN', 'SWGNT'),
    'tavg1_2d_slv_Nx':  ('T10M', 'QV10M', 'PS'),
    'tavg1_2d_slv_Nx*': ('Tmin',),
}
MERRA2_DATE_RX = re.compile(r'^MERRA2\_\d{3}\.tavg1\_2d\_(slv|rad)\_Nx\.(?P<date>\d{8})\.SUB\.nc$')

# Flip data up/down (ud) or left/right?
#   Argument lats/ lons should be a 1D numpy.ndarray
flip_ud = lambda lats: np.argmax(lats) > np.argmin(lats)
flip_lr = lambda lons: np.argmax(lons) < np.argmin(lons)


def main():
    'Populates the calibration driver dataset with data from MERRA2'
    target_hdf = h5py.File(OUTPUT_HDF, 'a')
    if 'MERRA2' not in target_hdf.keys():
        target_hdf.create_group('MERRA2')
    shp = (EXPECTED_NUM_FILES, target_hdf['FLUXNET/site_id'].size)
    coords = target_hdf['coordinates/lng_lat'][:]
    coord_arrays = (
        xr.DataArray(coords[:,0], dims = 'points'),
        xr.DataArray(coords[:,1], dims = 'points'),
    )
    for dataset, template in MERRA2_DATASETS:
        print(f'Working on "{dataset}" dataset...')
        # File list for the current data fields of interest
        file_list = glob.glob(f'{MERRA2_DIR}/{dataset}/{template}')
        # In order to compute separate daytime, night-time quanitities, we
        #   need the GMAO SWGNT data
        _dataset, _template = MERRA2_RAD_DATASET
        rad_file_list = glob.glob(f'{MERRA2_DIR}/{_dataset}/{_template}')
        assert len(file_list) == EXPECTED_NUM_FILES\
            and len(rad_file_list) == EXPECTED_NUM_FILES,\
            f'Did not find {EXPECTED_NUM_FILES} files!'
        file_list.sort() # Get chronological order
        rad_file_list.sort()
        # Create a dataset for each field, including "_daytime" and
        #   "_nightttime" datasets
        nc0 = netCDF4.Dataset(file_list[0])
        for field in MERRA2_FIELDS[dataset]:
            for suffix in ('', '_daytime', '_nighttime'):
                if field == 'Tmin' and suffix != '':
                    break # No daytime or nighttime averages for Tmin
                if f'MERRA2/{field}{suffix}' not in target_hdf.keys():
                    dataset = target_hdf.create_dataset(
                        f'MERRA2/{field}{suffix}', shp, np.float32)
                    dataset.attrs['description'] = nc0.variables[field].long_name
                    dataset.attrs['units'] = nc0.variables[field].units
        nc0.close()
        for i, filename in enumerate(tqdm(file_list)):
            nc = netCDF4.Dataset(filename)
            # Figure out which file has the SWGNT field
            rad_filename = None
            if dataset != 'tavg1_2d_rad_Nx':
                rad_filename = rad_file_list[i]
            with warnings.catch_warnings():
                warnings.simplefilter('ignore')
                data = extract(
                    filename, rad_filename, coord_arrays, MERRA2_FIELDS[dataset])
            for field in data.keys():
                target_hdf[f'MERRA2/{field}'][i] = data[field][0]
                if field == 'Tmin':
                    break # No daytime/ nighttime averages
                target_hdf[f'MERRA2/{field}_daytime'][i] = data[field][1]
                target_hdf[f'MERRA2/{field}_nighttime'][i] = data[field][2]


def extract(
        filename: str, rad_filename: str, coords: tuple, fields: Sequence,
    ) -> list:
    '''
    Extracts data from a MERRA-2 NetCDF4 granule, based on the provided field.

    Parameters
    ----------
    filename : str
    rad_filename : str
    coords : tuple
        A tuple of two xarray.DataArray: (longitude, latitude)
    fields : Sequence
        Sequence of the field names to extract

    Returns
    -------
    list
        Sequence of numpy.ndarray instances; if the field has daytime or
        nighttime averages, the returned list might look like:
        [24-hour average, daytime average, nighttime average]
    '''
    nc = xr.open_dataset(filename)
    nc_rad = nc
    if rad_filename is not None:
        nc_rad = xr.open_dataset(rad_filename)
    assert nc['time'].size == 24, 'Unexpected time axis length!'
    # Nearest-neighbour lookup for each site
    x, y = coords
    subset = nc.sel(lon = x, lat = y, method = 'nearest')
    results = []
    for field_name in fields:
        if field_name == 'Tmin':
            tmin = subset['T10M'].values.min(axis = 0)
            results.append((field_name, (tmin, None, None)))
        else:
            # First, generate the 24-hour average
            arr = subset[field_name].values
            avg = arr.mean(axis = 0) # 24-hour average
            # Then, generate the nighttime, daytime averages
            subset_rad = nc_rad.sel(lon = x, lat = y, method = 'nearest')
            # Get a mask where net down-welling short-wave radiation is above zero
            sun_up = subset_rad['SWGNT'].values > 0
            day = np.nanmean(np.where(sun_up, arr, np.nan), axis = 0)
            # If the sun never rises, use the 24-hour average instead
            day = np.where(sun_up.sum(axis = 0) == 0, avg, day)
            night = np.nanmean(np.where(sun_up, np.nan, arr), axis = 0)
            results.append((field_name, (avg, day, night)))
    return dict(results)



if __name__ == '__main__':
    main()
