'''
Extracts the MODIS MCD12Q1 land-cover classification (Plant Functional Type,
PFT) for the 356 eddy covariance sites from the FLUXNET2015/ La Thuille
collection.

The output HDF5 field structure looks like (names with a "/" are groups, all
others are data fields):

    state/
        PFT
'''

import glob
import os
import re
import h5py
import numpy as np
from osgeo import gdal
from tqdm import tqdm

MODIS_LC_DIR = '/anx_lagr4/MODIS_VIIRS/MCD12Q1'
MODIS_LC_RX = re.compile(r'^MCD12Q1\.A2015001\.h(?P<hh>\d{2})v(?P<vv>\d{2})\.006.*\.hdf$')
OUTPUT_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v1.h5'
PIXEL_RADIUS = 1

def main():
    'Populates the calibration driver dataset with the PFT'
    target_hdf = h5py.File(OUTPUT_HDF, 'a')
    if 'state' not in target_hdf.keys():
        target_hdf.create_group('state')
    n = 1 + (PIXEL_RADIUS * 2)
    row, col = target_hdf[f'coordinates/sinusoid_{n}x{n}_subgrid_row_col'][:]
    tile_h, tile_v = target_hdf[f'coordinates/sinusoid_{n}x{n}_subgrid_tile_h_v'][:]
    file_list = glob.glob(os.path.join(MODIS_LC_DIR, '*.hdf'))
    assert len(file_list) == 315
    # Get a container for the PFT
    results = np.full(tile_h.shape, np.nan)
    for filename in tqdm(file_list):
        hh, vv = map(int, MODIS_LC_RX.match(os.path.basename(filename)).groups())
        tile_mask = np.logical_and(tile_h == hh, tile_v == vv)
        if not tile_mask.any():
            continue # Skip tiles that contain no FLUXNET sites
        ds = gdal.Open(f'HDF4_EOS:EOS_GRID:"{filename}":MCD12Q1:LC_Type2')
        arr = ds.ReadAsArray()
        midx = np.ravel_multi_index(
            (row[tile_mask].ravel(), col[tile_mask].ravel()), arr.shape)
        results[tile_mask] = arr.ravel()[midx]
    if 'state/PFT' in target_hdf.keys():
        del target_hdf['state/PFT']
    dataset = target_hdf.create_dataset(
        'state/PFT', results.shape, np.int16, results)
    dataset.attrs['source'] = 'MODIS MCD12Q1.A2015001 LC_Type2'


if __name__ == '__main__':
    main()
