'''
Formatting and cleaning NPP data for calibrating MOD17. Prior to this step, I
first manually re-formatted each NPP CSV file so that its columns are:

    UID, site_name, year, PFT, longitude, latitude, ANPP, BNPP, TNPP

Where `*NPP` is one or more columns of NPP data in [g C m-2 year-1] units.
The processing steps here include:

1. Combining the NPP datasets and registering the longitude, latitude of each
    measurement in the `OUTPUT_HDF`.
2. Identifying the MODIS MCD12Q1 Type-2 land cover associated with each
    measurement location.
3. Recording the NPP data in the `OUTPUT_HDF`.

Measurement locations might not line up with valid MCD12Q1 PFTs, so we use a
progressively wider search radius to find the most common valid PFT among
neighboring pixels.

Extra data for PFT6, which is woefully under-represented in the calibraiton
data so far, was generated from the validation dataset by:

    cat ~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/NPP_validation/NPP_Gridded_2335_unique_cells_R3.csv | awk -F "," '{ if ( $1 == "NPP_MultiBiome_614_512" || $1 == "NPP_MultiBiome_614_553" || $1 == "NPP_MultiBiome_614_560" || $1 == "NPP_MultiBiome_614_575" || $1 == "NPP_MultiBiome_614_601" || $1 == "NPP_MultiBiome_614_618" || $1 == "NPP_MultiBiome_614_619" || $1 == "NPP_MultiBiome_614_620" || $1 == "NPP_MultiBiome_614_621" || $1 == "NPP_MultiBiome_614_621" || $1 == "NPP_MultiBiome_614_636" || $1 == "NPP_MultiBiome_614_636" || $1 == "NPP_MultiBiome_614_643" || $1 == "NPP_MultiBiome_614_665" || $1 == "NPP_MultiBiome_614_705" || $1 == "NPP_MultiBiome_614_721" || $1 == "NPP_MultiBiome_614_803" || $1 == "NPP_MultiBiome_614_804" || $1 == "NPP_MultiBiome_614_804" || $1 == "NPP_MultiBiome_614_805" || $1 == "NPP_MultiBiome_614_818" || $1 == "NPP_MultiBiome_614_818" || $1 == "NPP_MultiBiome_614_835" || $1 == "NPP_MultiBiome_614_836" || $1 == "NPP_MultiBiome_614_842" || $1 == "NPP_MultiBiome_614_1217" || $1 == "NPP_MultiBiome_614_1217" || $1 == "NPP_MultiBiome_614_1217" || $1 == "NPP_MultiBiome_614_1217" || $1 == "NPP_MultiBiome_614_1259") print }' >> ~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/NPP_calibration/NPP_Gridded_2335_unique_cells_R3_PFT6_subset.csv

The 30 sites were randomly chosen.

This version includes additional sites from the ORNL database.
'''

import csv
import glob
import os
import re
import h5py
import numpy as np
from itertools import combinations
from collections import Counter
from osgeo import gdal
from tqdm import tqdm
from mod17.utils import haversine
from mod17.srs import *

NPP_DIR = '/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/NPP_calibration/'
MODIS_LC_DIR = '/anx_lagr4/MODIS_VIIRS/MCD12Q1/'
MODIS_LC_RX = re.compile(r'^MCD12Q1\.A2015001\.h(?P<hh>\d{2})v(?P<vv>\d{2})\.006.*\.hdf$')
OUTPUT_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v8.h5'
HEADER = ('UID', 'site_name', 'year', 'PFT', 'longitude', 'latitude')
VALID_PFT = (1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12)
# 30 random extra PFT6 sites taken from the validation dataset; this is just
#   an FYI, these names are not used in processing
PFT6_EXTRAS = [
    'NPP_MultiBiome_614_512', 'NPP_MultiBiome_614_553', 'NPP_MultiBiome_614_560', 'NPP_MultiBiome_614_575', 'NPP_MultiBiome_614_601', 'NPP_MultiBiome_614_618', 'NPP_MultiBiome_614_619', 'NPP_MultiBiome_614_620', 'NPP_MultiBiome_614_621', 'NPP_MultiBiome_614_621', 'NPP_MultiBiome_614_636', 'NPP_MultiBiome_614_636', 'NPP_MultiBiome_614_643', 'NPP_MultiBiome_614_665', 'NPP_MultiBiome_614_705', 'NPP_MultiBiome_614_721', 'NPP_MultiBiome_614_803', 'NPP_MultiBiome_614_804', 'NPP_MultiBiome_614_804', 'NPP_MultiBiome_614_805', 'NPP_MultiBiome_614_818', 'NPP_MultiBiome_614_818', 'NPP_MultiBiome_614_835', 'NPP_MultiBiome_614_836', 'NPP_MultiBiome_614_842', 'NPP_MultiBiome_614_1217', 'NPP_MultiBiome_614_1217', 'NPP_MultiBiome_614_1217', 'NPP_MultiBiome_614_1217', 'NPP_MultiBiome_614_1259'
]

def main():
    'Populates the calibration driver dataset with NPP data'
    target_hdf = h5py.File(OUTPUT_HDF, 'a')
    if 'NPP' not in target_hdf.keys():
        target_hdf.create_group('NPP')
    # Read in the NPP CSV files
    csv_file_list = glob.glob(os.path.join(NPP_DIR, '*.csv'))
    assert len(csv_file_list) == 13
    loc_names = []
    coords = []
    npp_above_list = []
    npp_below_list = []
    npp_total_list = []
    pft_list = []
    year_list = []
    for filename in csv_file_list:
        with open(filename, 'r') as file:
            reader = csv.reader(file)
            for line in reader:
                if reader.line_num == 1:
                    assert all([i == j for i, j in zip(line[:6], HEADER)])
                    continue # Skip to next line
                if len(line) > 8:
                    lng, lat, npp_a, npp_b, npp = map(
                        lambda x: float(x) if x != '' else np.nan, line[4:9])
                else:
                    # Total NPP is not provided
                    npp = np.nan
                    lng, lat, npp_a, npp_b = map(
                        lambda x: float(x) if x != '' else np.nan, line[4:8])
                if lng < -900 or lat < -900:
                    continue # Don't include sites without coordinates
                if (npp_a >= 0) or (npp >= 0):
                    # EITHER Total NPP, Below-ground, or Above-ground NPP required
                    npp_a = npp_a if npp_a > -900 else np.nan
                    npp_b = npp_b if npp_b > -900 else np.nan
                    npp = npp if npp > -900 else np.nan
                    loc_names.append(line[0])
                    coords.append((lng, lat))
                    npp_above_list.append(npp_a)
                    npp_below_list.append(npp_b)
                    npp_total_list.append(npp)
                    # If a PFT code was provided, include it
                    if line[3] != '' and line[3] != 'NA':
                        pft_list.append(int(line[3]))
                    else:
                        pft_list.append(-9)
                    # If a year was provided, include it
                    try:
                        year = int(line[2])
                    except ValueError:
                        year = -9999
                    year_list.append(year)
    assert len(loc_names) == len(coords)
    assert len(coords) == len(npp_above_list)
    assert len(year_list) == len(loc_names)
    # Convert to arrays
    npp_above = np.array(npp_above_list)
    npp_below = np.array(npp_below_list)
    npp_total = np.array(npp_total_list)
    # And total ANPP and BNPP where they weren't already totaled...
    npp_total = np.where(np.isnan(npp_total), npp_above + npp_below, npp_total)

    # Duplication detection
    combos = combinations(coords, 2)
    name_combos = list(combinations(loc_names, 2))
    n_combos = len(name_combos)
    results = []
    for i, pair in tqdm(enumerate(combos), total = n_combos):
        h = haversine(pair[0], pair[1])
        if h > 0 and h < 10:
            results.append((h, *name_combos[i]))
    names1 = [r[1] for r in results]
    names2 = [r[2] for r in results]
    idx1 = [loc_names.index(n) for n in names1]
    idx2 = [loc_names.index(n) for n in names2]
    npp_above_dedupe = []
    npp_below_dedupe = []
    npp_total_dedupe = []
    loc_names_dedupe = []
    coords_dedupe = []
    # First, deal with the non-duplicates
    for i, name in enumerate(loc_names):
        if name not in names1 and name not in names2:
            coords_dedupe.append(coords[i])
            loc_names_dedupe.append(name)
            npp_above_dedupe.append(npp_above[i])
            npp_below_dedupe.append(npp_below[i])
            npp_total_dedupe.append(npp_total[i])
    # Then, merge the data of the duplicates; arbitrarily prefer the second
    #   group of data ("names2", "idx2"), which comes from Luo et al.
    assert names2[0][0:3] == 'Luo'
    assert names2[-1][0:3] == 'Luo'
    npp_tot = np.where(
        np.logical_or(np.isnan(npp_total[idx1]), np.isnan(npp_total[idx2])),
        np.nanmedian(np.vstack((npp_total[idx1], npp_total[idx2])),
            axis = 0), npp_total[idx2])
    npp_a = np.where(
        np.logical_or(np.isnan(npp_above[idx1]), np.isnan(npp_above[idx2])),
        np.nanmedian(np.vstack((npp_above[idx1], npp_above[idx2])),
            axis = 0), npp_above[idx2])
    npp_b = np.where(
        np.logical_or(np.isnan(npp_below[idx1]), np.isnan(npp_below[idx2])),
        np.nanmedian(np.vstack((npp_below[idx1], npp_below[idx2])),
            axis = 0), npp_below[idx2])
    coords_dedupe.extend(np.stack(coords)[idx2].tolist())
    npp_above_dedupe.extend(npp_a)
    npp_below_dedupe.extend(npp_b)
    npp_total_dedupe.extend(npp_tot)
    loc_names_dedupe.extend(names2)

    # Write the site IDs...this, unfortunately, has to be verbose
    dt = h5py.string_dtype(encoding = 'utf-8')
    if 'NPP/site_id' not in target_hdf.keys():
        dataset = target_hdf.create_dataset(
            'NPP/site_id', (len(loc_names),), dt)
        for i, name in enumerate(loc_names):
            dataset[i] = name
    if 'NPP/lng_lat' not in target_hdf.keys():
        target_hdf.create_dataset(
            'NPP/lng_lat', np.stack(coords).shape, np.float32, np.stack(coords))
    if 'NPP/year' not in target_hdf.keys():
        target_hdf.create_dataset(
            'NPP/year', np.stack(year_list).shape, np.int16, np.stack(year_list))
    if 'NPP/NPP_above' not in target_hdf.keys():
        dataset = target_hdf.create_dataset(
            'NPP/NPP_above', (len(npp_above),), np.float32, np.stack(npp_above))
        dataset.attrs['units'] = 'g C m-2 year-1'
    if 'NPP/NPP_below' not in target_hdf.keys():
        dataset = target_hdf.create_dataset(
            'NPP/NPP_below', (len(npp_below),), np.float32, np.stack(npp_below))
        dataset.attrs['units'] = 'g C m-2 year-1'
    if 'NPP/NPP_total' not in target_hdf.keys():
        dataset = target_hdf.create_dataset(
            'NPP/NPP_total', (len(npp_below),), np.float32, np.stack(npp_total))
        dataset.attrs['units'] = 'g C m-2 year-1'
    # Now, for the land-cover type
    file_list = glob.glob(os.path.join(MODIS_LC_DIR, '*.hdf'))
    assert len(file_list) == 315
    # Convert the new coordinates to MODIS Sinusoidal coordinates and tile
    #   numbers
    s_coords = np.stack(list(map(modis_row_col_from_wgs84, coords)))
    row, col = (s_coords[:,0].astype(np.int32), s_coords[:,1].astype(np.int32))
    tile = np.stack(list(map(modis_tile_from_wgs84, coords)))
    # Get a container for the PFT
    results = np.full((len(npp_above_list),), np.nan)
    for filename in tqdm(file_list):
        hh, vv = map(int, MODIS_LC_RX.match(os.path.basename(filename)).groups())
        tile_mask = np.logical_and(tile[:,0] == hh, tile[:,1] == vv)
        if not tile_mask.any():
            continue # Skip tiles that contain no FLUXNET sites
        ds = gdal.Open(f'HDF4_EOS:EOS_GRID:"{filename}":MCD12Q1:LC_Type2')
        arr = ds.ReadAsArray()
        midx = np.ravel_multi_index(
            (row[tile_mask].ravel(), col[tile_mask].ravel()), arr.shape)
        subset = arr.ravel()[midx]
        # If there are any locations that have an invalid PFT...
        for i, pft in enumerate(subset.tolist()):
            if pft not in VALID_PFT:
                for radius in (5, 10, 20):
                    # Search in an area around this row/ column
                    r0, c0 = map(int, s_coords[tile_mask][i] - radius)
                    r1, c1 = map(int, s_coords[tile_mask][i] + radius)
                    alt = filter(lambda p: p in VALID_PFT, arr[r0:r1,c0:c1].ravel().tolist())
                    try:
                        subset[i] = Counter(alt).most_common()[0][0]
                    except:
                        continue # Try the next size!
        results[tile_mask] = subset
    if 'NPP/PFT' in target_hdf.keys():
        del target_hdf['NPP/PFT']
    # Replace the MCD12Q1 results with user-specified PFTs
    results = np.where(np.array(pft_list) > 0, np.array(pft_list), results)
    print(Counter(results.tolist()))
    dataset = target_hdf.create_dataset(
        'NPP/PFT', results.shape, np.int16, results)
    dataset.attrs['source'] = 'MODIS MCD12Q1.A2015001 LC_Type2'


if __name__ == '__main__':
    main()
