'''
Extracts MERRA-2 surface meteorology at the locations of the NPP measurements
for the period 1980-2000. Produces only a single 365-day year for comparison
to one-time ORNL NPP estimates during calibration. As such, if the year of the
ORNL measurement is not known, a random year between 1980 and 2000, inclusive,
is chosen.

The output HDF5 field structure looks like (names with a "/" are groups, all
others are data fields):

    NPP/
      surface_met_MERRA2/
        Tmin
        QV10M
        QV10M_daytime
        QV10M_nighttime
        SWGDN
        ...
        SWGNT
        ...
        LWGNT
        ...
'''

import datetime
import glob
import os
import re
import warnings
import netCDF4
import h5py
import numpy as np
import xarray as xr
from typing import Sequence
from functools import partial
from tqdm import tqdm
from suntransit import sunrise_sunset
from mod17.science import daynight_partition

EXPECTED_NUM_FILES = 7671 # Daily files from 1980-01-01 through 2000-12-31 (7671 days)
OUTPUT_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v8.h5'
MERRA2_DIR = '/anx_lagr4/MODIS_VIIRS/calibration/inputs/MERRA-2_1980-2000/'
MERRA2_DATASETS = [ # (dataset, filename template) pairs
    ('tavg1_2d_slv_Nx*', 'MERRA2_???.tavg1_2d_slv_Nx.*.SUB.nc'),
    ('tavg1_2d_slv_Nx',  'MERRA2_???.tavg1_2d_slv_Nx.*.SUB.nc'),
    ('tavg1_2d_rad_Nx',  'MERRA2_???.tavg1_2d_rad_Nx.*.SUB.nc'),
]
MERRA2_FIELDS = {
    'tavg1_2d_slv_Nx*': ('Tmin',),
    'tavg1_2d_slv_Nx':  ('T10M', 'QV10M', 'PS'),
    'tavg1_2d_rad_Nx':  ('SWGDN', 'SWGNT', 'LWGNT')
}
MERRA2_DATE_RX = re.compile(r'^MERRA2\_\d{3}\.tavg1\_2d\_(slv|rad)\_Nx\.(?P<date>\d{8})\.SUB\.nc$')

# Flip data up/down (ud) or left/right?
#   Argument lats/ lons should be a 1D numpy.ndarray
flip_ud = lambda lats: np.argmax(lats) > np.argmin(lats)
flip_lr = lambda lons: np.argmax(lons) < np.argmin(lons)


def main():
    'Populates the calibration driver dataset with data from MERRA2'
    target_hdf = h5py.File(OUTPUT_HDF, 'a')
    if 'NPP/surface_met_MERRA2' not in target_hdf.keys():
        target_hdf.create_group('NPP/surface_met_MERRA2')
    else:
        del target_hdf['NPP/surface_met_MERRA2']
    # We're creating a 365-day year, corresponding either to a real year (as
    #   indicated from ORNL dataset) or a synthetic year
    shp = (365, target_hdf['NPP/site_id'].size)
    coords = target_hdf['NPP/lng_lat'][:]
    coord_arrays = (
        xr.DataArray(coords[:,0], dims = 'points'),
        xr.DataArray(coords[:,1], dims = 'points'),
    )
    # Get a Day-by-Year array; if the year is provided from the ORNL dataset
    #   and it falls in [1980,2000], then every day is that year; otherwise,
    #   it is a random year from [1980,2000]
    all_years = target_hdf['NPP/year'][:]
    all_years = all_years[np.newaxis,:].repeat(365, axis = 0)
    np.random.seed(17)
    all_years = np.where(np.logical_or(
        np.logical_or(all_years < 1980, all_years > 2000), all_years == -9999),
        np.random.randint(1980, 2000, all_years.shape), all_years)
    for dataset, template in MERRA2_DATASETS:
        print(f'Working on "{dataset}" dataset...')
        file_list = glob.glob(f'{MERRA2_DIR}/{dataset}/{template}')
        assert len(file_list) == EXPECTED_NUM_FILES,\
            f'Did not find {EXPECTED_NUM_FILES} files!'
        file_list.sort() # Get chronological order
        # Create a dataset for each field, including "_daytime" and
        #   "_nightttime" datasets
        nc0 = netCDF4.Dataset(file_list[0])
        for field in MERRA2_FIELDS[dataset]:
            for suffix in ('', '_daytime', '_nighttime'):
                if field == 'Tmin' and suffix != '':
                    break # No daytime or nighttime averages for Tmin
                if f'NPP/surface_met_MERRA2/{field}{suffix}' not in target_hdf.keys():
                    dset = target_hdf.create_dataset(
                        f'NPP/surface_met_MERRA2/{field}{suffix}', shp, np.float32)
                    _field = field
                    if field == 'Tmin':
                        _field = 'T10M'
                    dset.attrs['description'] = nc0.variables[_field].long_name
                    dset.attrs['units'] = nc0.variables[_field].units
        nc0.close()
        for i, filename in enumerate(tqdm(file_list)):
            # Get the date from the filename...
            match = MERRA2_DATE_RX.match(os.path.basename(filename))
            dt = datetime.datetime.strptime(match.groupdict()['date'], '%Y%m%d')
            jday = int(dt.strftime('%j')) - 1 # Subtract 1 so Day 1 is index 0
            year = dt.year
            if year % 4 == 0:
                if jday == 59:
                    continue # Skip leap days (February 29, minus 1)
                if jday > 59:
                    jday -= 1 # Other days in a leap year should decrement 1
            if not (all_years[jday,:] == year).any():
                continue # Skip files that are not needed (low chance)
            idx = np.argwhere(all_years[jday,:] == year).ravel()
            nc = netCDF4.Dataset(filename)
            # Get sunrise, sunset times for slicing indices on hourly
            #   MERRA-2 arrays
            updown = updown_array(nc, coords.swapaxes(0, 1)[::-1], dt)
            with warnings.catch_warnings():
                warnings.simplefilter('ignore')
                data = extract(
                    filename, updown, coord_arrays, MERRA2_FIELDS[dataset])
            for field in data.keys():
                target_hdf[f'NPP/surface_met_MERRA2/{field}'][jday,idx]\
                    = data[field][0][idx]
                if field == 'Tmin':
                    break # No daytime/ nighttime averages
                target_hdf[f'NPP/surface_met_MERRA2/{field}_daytime']\
                    [jday,idx] = data[field][1][idx]
                target_hdf[f'NPP/surface_met_MERRA2/{field}_nighttime']\
                    [jday,idx] = data[field][2][idx]


def extract(
        filename: str, updown: np.ndarray, coords: tuple, fields: Sequence) -> list:
    '''
    Extracts data from a MERRA-2 NetCDF4 granule, based on the provided field.

    Parameters
    ----------
    nc : str
    updown : numpy.ndarray
        A (2 x ...) array with the hour of sunrise, sunset
    coords : tuple
        A tuple of two xarray.DataArray: (longitude, latitude)
    feilds : Sequence
        Sequence of the field names to extract

    Returns
    -------
    list
        Sequence of numpy.ndarray instances; if the field has daytime or
        nighttime averages, the returned list might look like:
        [24-hour average, daytime average, nighttime average]
    '''
    nc = xr.open_dataset(filename)
    assert nc['time'].size == 24, 'Unexpected time axis length!'
    assert updown.shape[0] == 2, 'Unexpected shape of "updown" array argument'
    # Nearest-neighbour lookup for each site
    x, y = coords
    subset = nc.sel(lon = x, lat = y, method = 'nearest')
    results = []
    for field_name in fields:
        if field_name == 'Tmin':
            tmin = subset['T10M'].values.min(axis = 0)
            results.append((field_name, (tmin, None, None)))
        else:
            arr = subset[field_name].values
            avg = arr.mean(axis = 0) # 24-hour average
            # Calculate daytime and nighttime averages; if sun never rises or never
            #   sets within a 24-hour period, fill in NaN
            arr = daynight_partition(arr, updown, missing = [np.nan] * 2)
            results.append((field_name, (avg, arr[0], arr[1])))
    return dict(results)


def updown_array(
        nc: netCDF4.Dataset, coords: np.ndarray,
        dt: datetime.datetime) -> np.ndarray:
    '''
    Returns
    -------
    numpy.ndarray
        A (2 x M x N) array, where the first axis enumerates the
        (sunrise, sunset) hour and M and N index rows, columns of the array
        spatial domain
    '''
    def sunrise_sunset2(*args, **kwargs):
        # Round down to the nearest hour
        return np.floor(np.array(sunrise_sunset(*args, **kwargs)))

    # Calculate sunrise, sunset for each pixel (lat-lng pair)
    return np.apply_along_axis(partial(sunrise_sunset2, dt = dt), 0, coords)


if __name__ == '__main__':
    main()
