'''
DEPRECATED. We realized that using a climatology will reduce the coefficient
of variation in predicted NPP.

Computes a climatology for MERRA-2. The output HDF5 field structure looks like
(names with a "/" are groups, all others are data fields):

    /
        Tmin
        QV10M
        QV10M_daytime
        QV10M_nighttime
        SWGDN
        ...
        SWGNT
        ...
        LWGNT
        ...
'''

import datetime
import glob
import os
import re
import warnings
import netCDF4
import h5py
import numpy as np
import xarray as xr
from typing import Sequence
from functools import partial
from tqdm import tqdm
from suntransit import sunrise_sunset
from mod17.science import daynight_partition

EXPECTED_NUM_FILES = 7671 # Daily files from 1980-01-01 through 2000-12-31 (7671 days)
MOD17_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v3.h5'
OUTPUT_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/MERRA2_climatology_1980-2000.h5'
MERRA2_DIR = '/anx_lagr4/MODIS_VIIRS/calibration/inputs/MERRA-2_1980-2000/'
MERRA2_DATASETS = [ # (dataset, filename template) pairs
    ('tavg1_2d_slv_Nx*', 'MERRA2_???.tavg1_2d_slv_Nx.*.SUB.nc'),
    ('tavg1_2d_slv_Nx',  'MERRA2_???.tavg1_2d_slv_Nx.*.SUB.nc'),
    ('tavg1_2d_rad_Nx',  'MERRA2_???.tavg1_2d_rad_Nx.*.SUB.nc'),
]
MERRA2_FIELDS = {
    'tavg1_2d_slv_Nx*': ('Tmin',),
    'tavg1_2d_slv_Nx':  ('T10M', 'QV10M', 'PS'),
    'tavg1_2d_rad_Nx':  ('SWGDN', 'SWGNT', 'LWGNT')
}
MERRA2_DATE_RX = re.compile(r'^MERRA2\_\d{3}\.tavg1\_2d\_(slv|rad)\_Nx\.(?P<date>\d{8})\.SUB\.nc$')

# Flip data up/down (ud) or left/right?
#   Argument lats/ lons should be a 1D numpy.ndarray
flip_ud = lambda lats: np.argmax(lats) > np.argmin(lats)
flip_lr = lambda lons: np.argmax(lons) < np.argmin(lons)


def main():
    ''
    target_hdf = h5py.File(OUTPUT_HDF, 'a')
    for dataset, template in MERRA2_DATASETS:
        print(f'Working on "{dataset}" dataset...')
        file_list = glob.glob(f'{MERRA2_DIR}/{dataset}/{template}')
        assert len(file_list) == EXPECTED_NUM_FILES,\
            f'Did not find {EXPECTED_NUM_FILES} files!'
        file_list.sort() # Get chronological order
        # Create a dataset for each field, including "_daytime" and
        #   "_nightttime" datasets
        nc0 = netCDF4.Dataset(file_list[0])
        # Get a global array of sunrise, sunset times for this day
        x = nc0.variables['lon'][:]
        y = nc0.variables['lat'][:]
        # I think I already know how the NetCDF4 arrays need to be flipped
        assert flip_ud(y)
        assert not flip_lr(x)
        xc, yc = np.meshgrid(x, y)
        coords = np.array(np.stack((yc.ravel(), xc.ravel()), axis = 0))
        shp = (365, y.size, x.size)
        # Pre-allocate arrays
        for field in MERRA2_FIELDS[dataset]:
            for suffix in ('', '_daytime', '_nighttime'):
                if field == 'Tmin' and suffix != '':
                    break # No daytime or nighttime averages for Tmin
                if f'{field}{suffix}' not in target_hdf.keys():
                    dset = target_hdf.create_dataset(
                        f'{field}{suffix}', shp, np.float32)
                    _field = field
                    if field == 'Tmin':
                        _field = 'T10M'
                    dset.attrs['description'] = nc0.variables[_field].long_name
                    dset.attrs['units'] = nc0.variables[_field].units
        nc0.close()
        for day in tqdm(range(1, 366)):
            results = dict([(k, []) for k in MERRA2_FIELDS[dataset]])
            # Re-create the file list, searching for matching days instead
            tpl = template\
                .replace('*', datetime.datetime.strptime(str(day), '%j')\
                .strftime('*%m%d'))
            file_list = glob.glob(f'{MERRA2_DIR}/{dataset}/{tpl}')
            assert len(file_list) == 21, 'Did not find 21 years of files!'
            # It doesn't matter much what year it is, only time of year
            func = partial(
                sunrise_sunset, dt = datetime.datetime.strptime(str(day), '%j'))
            # Get sunrise, sunset times for slicing indices on hourly
            #   MERRA-2 arrays
            updown = np.apply_along_axis(
                func, 0, coords).reshape((2, *xc.shape))
            for filename in file_list:
                with warnings.catch_warnings():
                    warnings.simplefilter('ignore')
                    data = extract(filename, updown, MERRA2_FIELDS[dataset])
                for field in data.keys():
                    results[field].append(data[field])
            # Compute mean for this day across the years
            for field in data.keys():
                arr_mean = np.stack(
                    [r[0] for r in results[field]]).mean(axis = 0)
                # NOTE: Flipping up-down because of how NetCDF4 stores arrays
                target_hdf[f'{field}'][day-1] = np.flipud(arr_mean)
                if field == 'Tmin':
                    continue # No daytime/ nighttime averages
                arr_day = np.stack(
                    [r[1] for r in results[field]]).mean(axis = 0)
                arr_night = np.stack(
                    [r[2] for r in results[field]]).mean(axis = 0)
                target_hdf[f'{field}_daytime'][day-1] = np.flipud(arr_day)
                target_hdf[f'{field}_nighttime'][day-1] = np.flipud(arr_night)


def extract(
        filename: str, updown: np.ndarray, fields: Sequence) -> list:
    '''
    Extracts data from a MERRA-2 NetCDF4 granule, based on the provided field.

    Parameters
    ----------
    nc : str
    updown : numpy.ndarray
        A (2 x ...) array with the hour of sunrise, sunset
    feilds : Sequence
        Sequence of the field names to extract

    Returns
    -------
    list
        Sequence of numpy.ndarray instances; if the field has daytime or
        nighttime averages, the returned list might look like:
        [24-hour average, daytime average, nighttime average]
    '''
    nc = xr.open_dataset(filename)
    assert nc['time'].size == 24, 'Unexpected time axis length!'
    assert updown.shape[0] == 2, 'Unexpected shape of "updown" array argument'
    results = []
    for field_name in fields:
        if field_name == 'Tmin':
            tmin = nc['T10M'].values.min(axis = 0)
            results.append((field_name, (tmin, None, None)))
        else:
            arr = nc[field_name].values
            avg = arr.mean(axis = 0) # 24-hour average
            # Calculate daytime and nighttime averages; if sun never rises or never
            #   sets within a 24-hour period, fill in NaN
            arr = daynight_partition(arr, updown, missing = [np.nan] * 2)
            results.append((field_name, (avg, arr[0], arr[1])))
    return dict(results)


if __name__ == '__main__':
    main()
