'''
Calculates mean annual NPP for 1980-2000 based on the FluxCom NEE data
and the RH data from Tang et al. (2020):

    NEE = RH - NPP
    NPP = RH - NEE

References:

    Tang, X., S. Fan, M. Du, W. Zhang, S. Gao, S. Liu, G. Chen, Z. Yu,
    and W. Yang. 2020. Spatial and temporal patterns of global soil
    heterotrophic respiration in terrestrial ecosystems.
    Earth System Science Data 12 (2):1037–1051.
'''

import calendar
import os
import numpy as np
import netCDF4
from osgeo import gdalconst
from pyl4c.lib.netcdf import netcdf_array
from pyl4c.spatial import array_to_raster, dump_raster, intersect_rasters

OUTPUT_NPP = '/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/20211015_mean_annual_NPP_1980-2000_gCm-2yr-1_FLUXCOM+Tang-et-al2020.tiff'
FLUXCOM_DIR = '/home/arthur/Downloads/FluxCom'
TANG_NETCDF = '/anx_lagr3/arthur.endsley/SMAP_L4C/third-party_data/Tang_et_al_2020_global_RH/RH.RF.720.360.1980.2016-Yearly.nc'
YEARS = range(1980, 2001)

def main():
    nc_tang = netCDF4.Dataset(TANG_NETCDF)
    for i, year in enumerate(YEARS):
        # Get this year's RH
        rh0, gt, wkt = netcdf_array(
            nc_tang, ('RH', 'longitude', 'latitude'), (0.5, 0.5), time_idx = i)
        rh0[rh0 == -9999] = np.nan
        # Get this year's NEE
        nc = netCDF4.Dataset(os.path.join(
            FLUXCOM_DIR,
            f'NEE.RS_METEO.FP-NONE.MLM-ALL.METEO-ERA5.720_360.monthly.{year}.nc'))
        nee0 = np.array(nc.variables['NEE'][:])
        nee0[nee0 < -9000] = np.nan
        # Convert mean monthly NEE (g C m-2 day-1) to monthly NEE (g C m-2 month-1)
        nee0 = np.nansum(np.stack([
            nee0[j-1] * calendar.mdays[j] for j in range(1, 13)
        ]), axis = 0)
        if year == 1980:
            rh = rh0
            nee = nee0
            # Get Tang et al. (2020) SRS
            gt_tang, wkt_tang = gt, wkt
            # Get FluxCom SRS
            _, gt, wkt = netcdf_array(nc, ('NEE', 'lon', 'lat'), (0.5, 0.5))
        else:
            rh = np.add(rh, rh0, out = rh)
            nee = np.add(nee, nee0, out = nee)
    rh = rh / len(YEARS)
    nee = nee / len(YEARS)
    # Project FluxCom onto the same lng-lat grid as Tang et al.
    rh_rast = array_to_raster(rh, gt_tang, wkt_tang)
    nee_rast = array_to_raster(nee, gt, wkt)
    nee_reproj = intersect_rasters(
        rh_rast, nee_rast, method = gdalconst.GRA_NearestNeighbour)
    rh_arr = rh_rast.ReadAsArray()
    nee_arr = nee_reproj.ReadAsArray()
    # Use Tang et al. (2020) as a data mask
    nee_arr[np.isnan(rh_arr)] = np.nan
    # NEE = RH - NPP  -->  NPP = RH - NEE
    npp = rh_arr - nee_arr
    npp[npp < 0] = np.nan
    dump_raster(
        array_to_raster(npp, gt_tang, wkt_tang), OUTPUT_NPP)


if __name__ == '__main__':
    main()
