# Plots a map of the composite mean annual NPP product

library(sp)
library(raster)
suppressPackageStartupMessages(library(sf))
library(dplyr, warn.conflicts = F)
library(tidyr, warn.conflicts = F)
library(stringr)
library(ggplot2)
library(latex2exp)
library(hdf5r)
options(stringsAsFactors = F)

gg.outline <- st_read('/home/arthur/Workspace/NTSG/SMAP_L4C/data/geo/ne_50m_coastline_epsg4326.shp',
  quiet = TRUE)
my.theme <- theme_linedraw() +
  theme(legend.position = 'top', legend.margin = margin(0, 0, -0.2, 0, 'cm'),
    plot.margin = margin(0.1, 0.5, 0.1, 0.1, 'cm'), legend.key.width = unit(2, 'cm'),
    panel.background = element_rect(fill = 'white', color = 'transparent'),
    plot.background = element_rect(fill = 'white', color = 'transparent'))

MOD17.HDF5 <- '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v9.h5'
PFT.LOOKUP <- data.frame(code = c(1:10, 12), 
  PFT = c('ENF', 'EBF', 'DNF', 'DBF', 'MF', 'CSH', 'OSH', 'WSV', 'SAV', 'GRS', 'CRO'))

# Load mean annual NNP image
img <- raster::stack(
  '/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/20211015_mean_annual_NPP_1980-2000_gCm-2yr-1_FLUXCOM+Tang-et-al2020.tiff')
gg.img <- as.data.frame(img, xy = TRUE)
names(gg.img) <- c('x', 'y', 'value')

h5.file <- hdf5r::h5file(MOD17.HDF5, mode = 'r')
missing <- is.na(h5.file[['NPP/NPP_total_filled']][1:1212])
x <- h5.file[['NPP/lng_lat']][1,1:1212]
x <- x[!missing]
y <- h5.file[['NPP/lng_lat']][2,1:1212]
y <- y[!missing]
pft <- h5.file[['NPP/PFT']][1:1212]
pft <- pft[!missing]
h5.file$close()
stopifnot(sum(missing) == 64)

geo.l4c <- data.frame(x = x, y = y, code = pft) %>%
  left_join(PFT.LOOKUP, by = 'code')
coordinates(geo.l4c) <- ~ x + y
geo.l4c <- st_set_crs(st_as_sf(geo.l4c), 4326)


# Plotting ORNL sites ##########################################################

ggplot() +
  geom_raster(data = gg.img, aes(x = x, y = y, fill = value)) +
  geom_sf(data = gg.outline, size = 0.2, color = 'black') +
  geom_sf(data = geo.l4c, shape = 6, color = 'red') +
  coord_sf(expand = FALSE, ylim = c(-62, 85)) +
  scale_fill_fermenter(
    palette = 'YlGnBu', type = 'div', oob = scales::squish,
      na.value = 'lightgray', expand = c(0, 0), breaks = seq(0, 2e3, 250),
      guide = guide_colorbar(barheight = unit(0.35, 'cm'), 
        barwidth = unit(9, 'cm'), ticks.colour = 'transparent', frame.colour = 'black')) +
  labs(x = NULL, y = NULL, fill = TeX('Mean Annual NPP $(g\\,C\\,m^{-2}\\,yr^{-1})\\,$')) +
  my.theme 
ggsave(width = 1200/172, height = 590/172, dpi = 172,
  file = '/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202110/20211015_mean_annual_NPP_1980-2000_with_ORNL_sites.png')


# Plotting FLUXNET sites #######################################################

require(reticulate)
if (Sys.info()['nodename'] == 'Gullveig') {
  use_python('/usr/local/python-env/mod17/bin/python3.8')
} else {
  use_python('/usr/local/python-env/mod17/bin/python3.10', required = TRUE)
}
mod17 <- import('mod17')

h5.file <- hdf5r::h5file(MOD17.HDF5, mode = 'r')
fluxnet.x <- h5.file[['coordinates/lng_lat']][1,1:356]
fluxnet.y <- h5.file[['coordinates/lng_lat']][2,1:356]
site.list <- h5.file[['FLUXNET/site_id']][1:356]
fluxnet.pft <- h5.file[['state/PFT']][1:9,1:356]
fluxnet.pft <- mod17$utils$pft_dominant(t(fluxnet.pft), site_list = site.list)
h5.file$close()

geo.fluxnet <- data.frame(x = fluxnet.x, y = fluxnet.y, code = fluxnet.pft) %>%
  left_join(PFT.LOOKUP, by = 'code')
coordinates(geo.fluxnet) <- ~ x + y
geo.fluxnet <- st_set_crs(st_as_sf(geo.fluxnet), 4326)

ggplot() +
  geom_raster(data = gg.img, aes(x = x, y = y, fill = value)) +
  geom_sf(data = gg.outline, size = 0.2, color = 'black') +
  geom_sf(data = filter(geo.fluxnet, PFT %in% c('SAV', 'WSV')),
    shape = 6, color = 'red') +
  coord_sf(expand = FALSE, ylim = c(-62, 85)) +
  facet_wrap(~ PFT) +
  scale_fill_fermenter(
    palette = 'YlGnBu', type = 'div', oob = scales::squish,
      na.value = 'lightgray', expand = c(0, 0), breaks = seq(0, 2e3, 250),
      guide = guide_colorbar(barheight = unit(0.35, 'cm'), 
        barwidth = unit(9, 'cm'), ticks.colour = 'transparent', frame.colour = 'black')) +
  labs(x = NULL, y = NULL, fill = TeX('Mean Annual NPP $(g\\,C\\,m^{-2}\\,yr^{-1})\\,$')) +
  my.theme 


# StackOverflow debugging #######################

# library("ggplot2")
# theme_set(theme_bw())
# library("sf")
# library("rnaturalearth")
# library("rnaturalearthdata")
# library(rgeos)
# world <- ne_countries(scale = "medium", returnclass = "sf")
# ggplot(data = world) +
#   geom_sf() +
#   coord_sf(expand = FALSE, ylim = c(-60, 85)) +
#   scale_y_continuous(limits = c(-79, 79))
# 
# ggplot(data = world) +
#   geom_sf() +
#   coord_sf(expand = FALSE) +
#   scale_y_continuous(limits = c(-60, 85))

# By PFT ########################################

ggplot() +
  geom_raster(data = gg.img, aes(x = x, y = y, fill = value)) +
  geom_sf(data = gg.outline, size = 0.2, color = 'black') +
  geom_sf(data = filter(geo.l4c, PFT %in% c('CSH', 'OSH', 'SAV', 'WSV')),
    shape = 6, color = 'red') +
  coord_sf(expand = FALSE, ylim = c(-62, 85)) +
  facet_wrap(~ PFT) +
  scale_fill_fermenter(
    palette = 'YlGnBu', type = 'div', oob = scales::squish,
      na.value = 'lightgray', expand = c(0, 0), breaks = seq(0, 2e3, 250),
      guide = guide_colorbar(barheight = unit(0.35, 'cm'), 
        barwidth = unit(9, 'cm'), ticks.colour = 'transparent', frame.colour = 'black')) +
  labs(x = NULL, y = NULL, fill = TeX('Mean Annual NPP $(g\\,C\\,m^{-2}\\,yr^{-1})\\,$')) +
  my.theme +
  theme(strip.text = element_text(color = 'black', face = 'bold'),
    strip.background = element_blank())
ggsave(width = 8.5, height = 5, dpi = 172,
  file = '/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202110/20211015_mean_annual_NPP_1980-2000_with_ORNL_sites_by_PFT.png')

ggplot() +
  geom_raster(data = gg.img, aes(x = x, y = y, fill = value)) +
  geom_sf(data = gg.outline, size = 0.2, color = 'black') +
  geom_sf(data = filter(geo.l4c, PFT %in% c('ENF', 'EBF', 'DNF', 'DBF')),
    shape = 6, color = 'red') +
  coord_sf(expand = FALSE, ylim = c(-62, 85)) +
  facet_wrap(~ PFT) +
  scale_fill_fermenter(
    palette = 'YlGnBu', type = 'div', oob = scales::squish,
      na.value = 'lightgray', expand = c(0, 0), breaks = seq(0, 2e3, 250),
      guide = guide_colorbar(barheight = unit(0.35, 'cm'), 
        barwidth = unit(9, 'cm'), ticks.colour = 'transparent', frame.colour = 'black')) +
  labs(x = NULL, y = NULL, fill = TeX('Mean Annual NPP $(g\\,C\\,m^{-2}\\,yr^{-1})\\,$')) +
  my.theme +
  theme(strip.text = element_text(color = 'black', face = 'bold'),
    strip.background = element_blank())
ggsave(width = 8.5, height = 5, dpi = 172,
  file = '/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202110/20211015_mean_annual_NPP_1980-2000_with_ORNL_sites_by_PFT_again.png')
