'''
Generates coordinates from which to extract data on a MODIS Sinusoidal grid
(e.g., MODIS fPAR, LAI, and land cover)...

      sinusoid_NxN_subgrid_row_col
      sinusoid_NxN_subgrid_tile_h_v
'''

import csv
import h5py
import numpy as np
from functools import partial
from mod17.srs import TILE_LINES, modis_tile_from_wgs84, modis_row_col_from_wgs84, modis_row_col_to_wgs84
from mod17.utils import mod15a2h_qc_fail

PIXEL_RADIUS = 1 # Radius, in number of pixels, around each tower site
MOD17_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v8.h5'

def main():
    'Generates output CSV that knows the lng/lat coordinates of subgrid'
    with h5py.File(MOD17_HDF, 'r') as hdf:
        coords = hdf['NPP/lng_lat'][:].swapaxes(0, 1)
    grid = modis_row_col_from_wgs84(coords)
    row, col = grid
    row_col = np.stack((
        np.apply_along_axis(
            lambda r: r + np.arange(-PIXEL_RADIUS, PIXEL_RADIUS + 1), 0, row[None,:]),
        np.apply_along_axis(
            lambda c: c + np.arange(-PIXEL_RADIUS, PIXEL_RADIUS + 1), 0, col[None,:])))
    row_col_expanded = np.stack([
        np.stack(list(map(
            lambda rc: rc.ravel(), np.meshgrid(*row_col[...,i], indexing = 'ij'))))
        for i in range(0, row_col.shape[-1])
    ], axis = 1)
    # Get a matrix of (H, V) or tile number elements with the same shape
    tile = modis_tile_from_wgs84(coords)[...,None]\
        .repeat((1 + 2*PIXEL_RADIUS)**2, axis = -1)
    # Col-wise, none of the expanded grid cells should lie in a tile to the right
    ncol = TILE_LINES[500]
    assert np.argwhere(row_col_expanded[1] >= ncol).size == 0,\
        "It's expected that none of the expanded column indices are too large"
    # ...But they might lie in a different tile column, so we first:
    # Get the next tile column over, next tile row down (where applicable);
    #   we have to flip the first axis (row-column) because the tile dimensions
    #   (H and V) have reverse meanings (i.e., H indexes along column direction,
    #   V indexes along the row direction).
    tile = np.where(row_col_expanded[::-1] < 0, tile - 1, tile)
    tile = np.where(row_col_expanded[::-1] >= ncol, tile + 1, tile)
    # Then, count backwards from the total number of columns (i.e., we're
    #   indexing into the next tile to the left from right to left)
    _row_col_expanded = row_col_expanded.copy()
    row_col_expanded[row_col_expanded < 0] += ncol
    row_col_expanded[row_col_expanded >= ncol] -= ncol # ncol == number of rows, too
    coords_out = modis_row_col_to_wgs84(
        row_col_expanded, tile[0], tile[1]).round(6)
    n = 1 + (PIXEL_RADIUS * 2)
    with h5py.File(MOD17_HDF, 'a') as hdf:
        if f'NPP/sinusoid_{n}x{n}_subgrid_row_col' in hdf.keys():
            del hdf[f'NPP/sinusoid_{n}x{n}_subgrid_row_col']
            del hdf[f'NPP/sinusoid_{n}x{n}_subgrid_tile_h_v']
        hdf.create_dataset(f'NPP/sinusoid_{n}x{n}_subgrid_row_col',
            row_col_expanded.shape, np.int16, row_col_expanded)
        hdf.create_dataset(f'NPP/sinusoid_{n}x{n}_subgrid_tile_h_v',
            tile.shape, np.int16, tile)


if __name__ == '__main__':
    main()
