'''
Calculates the mean total NPP at ORNL inventory sites, using an independent
estimate of total NPP (based on FluxCom and Tang et al. 2018) to fill in
either missing above-ground or below-ground NPP.
'''

import os
import numpy as np
import h5py
import xarray as xr
from collections import Counter
from matplotlib import pyplot

NPP = '/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/20211015_mean_annual_NPP_1980-2000_gCm-2yr-1_FLUXCOM+Tang-et-al2020.tiff'
MOD17_HDF5 = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v8.h5'

def main():
    with h5py.File(MOD17_HDF5, 'r') as hdf:
        npp_above = hdf['NPP/NPP_above'][:]
        npp_below = hdf['NPP/NPP_below'][:]
        npp_total = hdf['NPP/NPP_total'][:]
        coords = hdf['NPP/lng_lat'][:]
        pft_map = hdf['NPP/PFT'][:]
    x = xr.DataArray(coords[:,0], dims = 'points')
    y = xr.DataArray(coords[:,1], dims = 'points')
    npp = xr.open_rasterio(NPP)
    subset = npp.sel(x = x, y = y, method = 'nearest')
    npp_tang = subset.values.ravel()
    assert npp_tang.size == npp_total.size
    for i in range(0, npp_total.size):
        if not np.isnan(npp_total[i]):
            continue # Already have total NPP
        if np.isnan(npp_above[i]):
            npp_above[i] = npp_tang[i] - npp_below[i]
        if np.isnan(npp_below[i]):
            npp_below[i] = npp_tang[i] - npp_above[i]
    npp_above[npp_above < 0] = np.nan
    npp_below[npp_below < 0] = np.nan
    # Calculate total NPP
    total = np.where(np.isnan(npp_total), npp_above + npp_below, npp_total)
    # Tabulate sites by PFT
    pft_map[total >= 0]
    count = Counter(pft_map[total >= 0])
    count = list(dict(count).items())
    count.sort(key = lambda x: x[0])
    with h5py.File(MOD17_HDF5, 'a') as hdf:
        ds = hdf.create_dataset('NPP/NPP_total_filled', total.shape, total.dtype, total)
        ds.attrs['units'] = 'g C m-2 year-1'
        ds.attrs['description'] = 'Calculated as Total = Above + Below; if Total was missing, either Above or Below were filled based on independent known Total'


if __name__ == '__main__':
    main()
