'''
DEPRECATED. See 0701.

Extracts the MODIS MOD15A2H fPAR and LAI data for the ORNL sites with NPP
validaton data.

The output HDF5 field structure looks like (names with a "/" are groups, all
others are data fields):

    NPP/
        MOD15A2H_fPAR
        MOD15A2H_LAI
'''

import datetime
import glob
import os
import re
import h5py
import numpy as np
from osgeo import gdal
from tqdm import tqdm
from mod17.srs import modis_tile_from_wgs84, modis_row_col_from_wgs84
from mod17.utils import mod15a2h_qc_fail

MODIS_MOD15_DIR = '/anx_lagr4/MODIS_VIIRS/MOD15A2H'
MODIS_MOD15_RX = re.compile(r'^MOD15A2H\.A(?P<date>\d{7})\.h(?P<hh>\d{2})v(?P<vv>\d{2})\.061.*\.hdf$')
MOD17_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v4.h5'
DATE_START = datetime.date(2000, 1, 1)
DATE_END = datetime.date(2004, 12, 31)
N_DAYS = 1827 # Num. days between START and END

def main():
    'Populates the calibration driver dataset with fPAR, LAI data'
    date_list = [DATE_START]
    while date_list[-1] < DATE_END:
        date_list.append(date_list[-1] + datetime.timedelta(days = 1))
    # Get coordinate arrays for each model
    with h5py.File(MOD17_HDF, 'r') as hdf:
        coords = hdf['NPP/lng_lat'][:].swapaxes(0, 1)
    # Get a matrix of (H, V) or tile number elements with the same shape
    tiles = modis_tile_from_wgs84(coords)
    tile_h, tile_v = tiles
    grid = modis_row_col_from_wgs84(coords) # Coords within a tile
    rows, cols = grid.astype(np.int32)
    # Create output arrays
    all_fpar = np.full((N_DAYS, *rows.shape), -9999, dtype = np.int16)
    all_lai = np.full((N_DAYS, *rows.shape), -9999, dtype = np.int16)
    for d, date in enumerate(tqdm(date_list)):
        file_glob = 'MOD15A2H.A%s*' % date.strftime('%Y%j')
        file_list = glob.glob(os.path.join(MODIS_MOD15_DIR, file_glob))
        if len(file_list) == 0:
            continue # Skip dates with no data
        # Should be approximately 290 granules for a given date
        assert len(file_list) < 300
        for filename in file_list:
            _, hh, vv = map(
                int, MODIS_MOD15_RX.match(os.path.basename(filename)).groups())
            tile_mask = np.logical_and(tile_h == hh, tile_v == vv)
            if not tile_mask.any():
                continue # Skip tiles that contain no FLUXNET sites
            fpar = gdal.Open(
                f'HDF4_EOS:EOS_GRID:"{filename}":MOD_Grid_MOD15A2H:Fpar_500m')\
                    .ReadAsArray()
            lai = gdal.Open(
                f'HDF4_EOS:EOS_GRID:"{filename}":MOD_Grid_MOD15A2H:Lai_500m')\
                    .ReadAsArray()
            qc = gdal.Open(
                f'HDF4_EOS:EOS_GRID:"{filename}":MOD_Grid_MOD15A2H:FparLai_QC')\
                    .ReadAsArray()
            # Mask out low-quality retrievals
            fail = mod15a2h_qc_fail(qc)
            fpar[fail] = 255
            lai[fail] = 255
            midx = np.ravel_multi_index(
                (rows[tile_mask].ravel(), cols[tile_mask].ravel()), fpar.shape)
            all_fpar[d,tile_mask] = fpar.ravel()[midx]
            all_lai[d,tile_mask] = lai.ravel()[midx]
    with h5py.File(MOD17_HDF, 'a') as hdf:
        dataset = hdf.create_dataset(
            'NPP/MOD15A2H_fPAR', all_fpar.shape, np.int16, all_fpar)
        dataset.attrs['source'] = 'MODIS MOD15A2H "Fpar_500m" Collection 6.1'
        dataset = hdf.create_dataset(
            'NPP/MOD15A2H_LAI', all_lai.shape, np.int16, all_lai)
        dataset.attrs['source'] = 'MODIS MOD15A2H "Lai_500m" Collection 6.1'


if __name__ == '__main__':
    main()
