'''
DEPRECATED. See 0802.

Interpolates the 8-day MODIS MOD15A2H fPAR and LAI data so as to obtain daily
values--a forward- and backward-filling approach, no linear interpolation.

This instance calculates a climatology (and fills with climatology) *before*
interpolation.
'''

import datetime
import os
import h5py
import numpy as np
from scipy.ndimage import generic_filter
from pyl4c import suppress_warnings
from pyl4c.science import climatology365, ordinals365

MOD17_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v5.h5'
DATE_START = datetime.date(2000, 1, 1)
DATE_END = datetime.date(2004, 12, 31)

@suppress_warnings
def main():
    'Interpolates fPAR, LAI data'
    dates = [DATE_START]
    while dates[-1] < DATE_END:
        dates.append(dates[-1] + datetime.timedelta(days = 1))
    with h5py.File(MOD17_HDF, 'r') as hdf:
        fpar0 = hdf['NPP/MOD15A2H_fPAR'][:].astype(np.float32)
        lai0 = hdf['NPP/MOD15A2H_LAI'][:].astype(np.float32)
    # NOTE: valid range for both fPAR and LAI tops out at 100
    fpar_masked = np.where(
        np.logical_or(fpar0 < 0, fpar0 > 100), np.nan, fpar0)
    lai_masked = np.where(
        np.logical_or(lai0 < 0, lai0 > 20), np.nan, lai0)
    assert not np.apply_along_axis(
        lambda x: np.isnan(x).all(), 0, np.nanmean(fpar0, axis = -1)).any()
    assert not np.apply_along_axis(
        lambda x: np.isnan(x).all(), 0, np.nanmean(lai0, axis = -1)).any()
    # Compute fPAR and LAI climatologies
    print('Computing climatologies...')
    fpar_clim = climatology365(fpar_masked, dates, ignore_leap = False)
    lai_clim = climatology365(lai_masked, dates, ignore_leap = False)
    doy = np.array(ordinals365(dates))
    # Calculate truly missing values (as opposed to unobserved values within
    #   an 8-day interval) based on an 8-day moving window
    print('Applying moving window...')
    # Assumed to be the same for fPAR and LAI
    filt = np.stack([ # Assumed to be the same for fPAR and LAI
        generic_filter( # size = 9 refers to our "8-day" window (needs odd num.)
            fpar_masked[:,i], lambda x: np.isnan(x).sum(), size = 9)
        for i in range(0, fpar_masked.shape[1])
    ], axis = 1)
    # Fill in climatology...
    print('Filling in from climatology...')
    fpar = np.where(np.logical_or(filt > 8, fpar0 > 100),
        fpar_clim[doy-1], fpar0)
    lai = np.where(np.logical_or(filt > 8, lai0 > 20),
        lai_clim[doy-1], lai0)
    # Then, replace -9999 with NaN
    fpar = np.where(fpar < 0, np.nan, fpar)
    lai = np.where(lai < 0, np.nan, lai)
    # Forward, then backward-fill, separately for each subgrid pixel
    #   (for compatibility with non-vectorized functions)
    print('Forward-backward filling...')
    fpar_final = np.stack([
        fill_forward(fill_forward(fpar[...,i])[::-1])[::-1]
        for i in range(0, fpar.shape[-1])
    ], axis = 1)
    lai_final = np.stack([
        fill_forward(fill_forward(lai[...,i])[::-1])[::-1]
        for i in range(0, lai.shape[-1])
    ], axis = 1)
    print('Writing to file...')
    with h5py.File(MOD17_HDF, 'a') as hdf:
        shp = fpar_final.shape
        dset = hdf.create_dataset(
            'NPP/MOD15A2H_fPAR_interp', shp, fpar_final.dtype, fpar_final)
        dset.attrs['source'] = 'MODIS MOD15A2H "Fpar_500m" Collection 6.1'
        dset.attrs['description'] = 'Filled with climatology, then forward/backward interpolated (nearest neighbor) to daily time steps'
        dset = hdf.create_dataset(
            'NPP/MOD15A2H_LAI_interp', shp, lai_final.dtype, lai_final)
        dset.attrs['source'] = 'MODIS MOD15A2H "Lai_500m" Collection 6.1'
        dset.attrs['description'] = 'Filled with climatology, then forward/backward interpolated (nearest neighbor) to daily time steps'


def fill_forward(arr):
    'Forward filling, along axis 0, of NaNs with earlier non-NaN entires'
    mask = np.isnan(arr)
    # Get the numeric index along axis 0 where non-NaN, zero otherwise
    idx = np.where(~mask, np.arange(mask.shape[0]), 0)
    # Fill-forward with the last non-NaN numeric index
    np.maximum.accumulate(idx, axis = 0, out = idx)
    return arr[idx]


if __name__ == '__main__':
    main()
