'''
There are some spurious peaks in the MOD15A2H fPAR and LAI climatologies
that should be removed...

For fPAR, see examples at indices: 240, 260

For LAI, see examples at indices: 11, 62, 82, 89, 184
'''

import datetime
import os
import h5py
import numpy as np
from scipy.ndimage import generic_filter
from tqdm import tqdm
from pyl4c import suppress_warnings

MOD17_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v8.h5'
SUBGRID_SIZE = 9 # Number of 500-m pixels per tower site (NOT the radius)
LAI_CUTOFF = 30 # Maximum diff. in (raw) LAI units allowed between consecutive 8-day periods

@suppress_warnings
def main():
    'Filters the fPAR data'
    with h5py.File(MOD17_HDF, 'r') as hdf:
        lai = hdf['NPP/MOD15A2H_LAI_clim'][:].astype(np.float32)

    # Compute difference between 2 consecutive 8-day periods
    delta = np.concatenate([
        np.zeros((16, *lai.shape[1:])),
        np.apply_along_axis(lambda x: x[16:] - x[:-16], 0, lai)
    ], axis = 0)
    delta = np.where(delta < 0, 0, delta)
    # Calculate the minimum LAI in a 4-day moving window (accounting for
    #   8-day redundancy: 8 + 4 = 12)
    min_filt = []
    for i in tqdm(range(lai.shape[1])):
        min_filt.append(np.apply_along_axis(
            lambda x: generic_filter(x, np.nanmin, size = 12), 1, lai[:,i,:]))
    min_filt = np.stack(min_filt).swapaxes(0, 1)
    lai_filt = np.where(delta > LAI_CUTOFF, min_filt, lai)
    with h5py.File(MOD17_HDF, 'a') as hdf:
        dset = hdf.create_dataset('NPP/MOD15A2H_LAI_clim_filt', lai_filt.shape, np.float32, lai_filt)
        dset.attrs['source'] = 'MODIS MOD15A2H "Lai_500m" Collection 6.1'
        dset.attrs['description'] = 'Filled with climatology, then forward/backward interpolated (nearest neighbor) to daily time steps, computed as a climatology, then filtered'


if __name__ == '__main__':
    main()
