'''
Extracts the coordinates of all flux tower sites and NPP sample sites used
in MOD16 and MOD17 calibration, as a CSV file.

[x] MOD16 tower sites
[x] MOD17 tower sites
[x] NPP sample locations
'''

import csv
import datetime
import h5py

MOD16_HDF5 = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_tower_site_latent_heat_and_drivers_v3.h5'
MOD17_HDF5 = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v1.h5'
OUTPUT_CSV_TPL = '/home/arthur/Downloads/20220128_%s_site_coordinates_WGS84.csv'

def main():
    with h5py.File(MOD16_HDF5, 'r') as hdf:
        sites  = hdf['FLUXNET/site_id'][:]
        coords = hdf['coordinates/lng_lat'][:].tolist()
    write_csv('MOD16', sites, coords, 'MxD16 calibration and validation tower site locations')
    with h5py.File(MOD17_HDF5, 'r') as hdf:
        npp_sites = hdf['NPP/site_id'][:]
        npp_coords = hdf['NPP/lng_lat'][:].tolist()
        sites  = hdf['FLUXNET/site_id'][:]
        coords = hdf['coordinates/lng_lat'][:].tolist()
    write_csv('MOD17', sites, coords, 'MxD17 calibration and validation tower site locations')
    write_csv('MOD17_NPP', npp_sites, npp_coords, 'MxD17 Annual NPP validation sample locations')


def write_csv(name, sites, coords, description):
    'Writes a series of site IDs and coordinates to a CSV file'
    if hasattr(sites[0], 'decode'):
        sites = list(map(lambda x: x.decode('utf-8'), sites))
    with open(OUTPUT_CSV_TPL % name, 'w') as file:
        writer = csv.writer(file)
        today = datetime.date.today().strftime('%Y-%m-%d')
        writer.writerow(('# %s' % description,))
        writer.writerow(('# Date: %s' % today,))
        writer.writerow(('# Author: K. Arthur Endsley (arthur.endsley@ntsg.umt.edu)',))
        writer.writerow(('group', 'site_id', 'longitude', 'latitude'))
        for site_id, coord_pair in zip(sites, coords):
            lng, lat = coord_pair
            writer.writerow((name, site_id, round(lng, 5), round(lat, 5)))


if __name__ == '__main__':
    main()
