'''
For MOD16 re-calibration, re-formatting the MOD15A2HGF point samples that
Maosheng prepared, based on the FLUXNET sites and the 3-by-3 subgrid.
I used the upstream script to generate the coordinates to send to Maosheng:

    20220128_0101_Cal-Val_sample_site_coordinates_to_CSV.py

This version is for getting the data to drive MxD16.
'''

import csv
import datetime
import glob
import os
import h5py
import numpy as np
from tqdm import tqdm

# Extracted from Maosheng's archive "MOD16_sites_MOD15A2HGF_C61_cutout_2000_2021.tar.gz"
MAOSHENG_CSV_DIR = '/home/arthur/Downloads/Maosheng/MOD16_sites_MOD15A2HGF_C61_cutout_2000_2021'
CALVAL_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_tower_site_latent_heat_and_drivers_v4.h5'
N_SUBGRID = 9 # i.e., 3-by-3 subgrid has 9 pixels

def main():
    # Read in the FLUXNET site list
    with h5py.File(CALVAL_HDF, 'r') as hdf:
        site_list = hdf['FLUXNET/site_id'][:].tolist()
        date_list = [ # Compile list of YYYYJJJ dates, where JJJ is DOY
            datetime.date(*d).strftime('%Y%j')
            for d in hdf['time'][:].tolist()
        ]
        n_sites = len(site_list)
    if hasattr(site_list[0], 'decode'):
        # Convert site names from UTF-8 if necessary
        site_list = list(map(lambda x: x.decode('utf-8'), site_list))
    # Open the output file for writing, copy site names over
    target_hdf = h5py.File(CALVAL_HDF, 'a')
    if 'MODIS/MOD15A2HGF_fPAR' in target_hdf.keys():
        del target_hdf['MODIS/MOD15A2HGF_fPAR']
        del target_hdf['MODIS/MOD15A2HGF_LAI']
    shp = (len(date_list), n_sites, N_SUBGRID)
    empty = np.full(shp, -9999, dtype = np.int16)
    dataset = target_hdf.create_dataset('MODIS/MOD15A2HGF_fPAR', shp, np.int16, empty)
    dataset.attrs['source'] = 'From Maosheng Zhao, MODIS MOD15A2HGF "Fpar_500m" Collection 6.1'
    dataset = target_hdf.create_dataset('MODIS/MOD15A2HGF_LAI', shp, np.int16, empty)
    dataset.attrs['source'] = 'From Maosheng Zhao, MODIS MOD15A2HGF "Lai_500m" Collection 6.1'
    for field, src_field in zip(('fPAR', 'LAI'), ('Fpar_500m', 'Lai_500m')):
        for i, site_id in tqdm(enumerate(site_list)):
            file_list = glob.glob(f'{MAOSHENG_CSV_DIR}/MOD15A2HGF.{src_field}.{site_id}.20??.csv')
            file_list.sort()
            for j, filename in enumerate(file_list):
                if int(filename[-8:-4]) > 2019:
                    break # Skip files after 2019
                with open(filename, 'r') as file:
                    reader = csv.reader(file)
                    for line in reader:
                        date = line[0]
                        # Figure out which date this composite is associated with
                        t = date_list.index(date)
                        # Mask out fill values
                        data = map(lambda x: -9999 if x > 100 else x, map(int, line[1:]))
                        target_hdf[f'MODIS/MOD15A2HGF_{field}'][t,i,:] = list(data)


if __name__ == '__main__':
    main()
