'''
Interpolates the 8-day MODIS MOD15A2HGF fPAR and LAI data so as to obtain
daily values--a forward- and backward-filling approach, no linear
interpolation.

This instance calculates a climatology (and fills with climatology) *before*
interpolation.
'''

import datetime
import os
import h5py
import numpy as np
from scipy.ndimage import generic_filter
from pyl4c import suppress_warnings
from pyl4c.science import climatology365, ordinals365

MOD16_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_tower_site_latent_heat_and_drivers_v4.h5'
MOD17_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v2.h5'
SUBGRID_SIZE = 9 # Number of 500-m pixels per tower site (NOT the radius)

@suppress_warnings
def main(hdf_filename):
    'Populates the calibration driver dataset with fPAR, LAI data'
    with h5py.File(hdf_filename, 'a') as hdf:
        dates = [
            datetime.date(*d) for d in hdf['time'][:].tolist()
        ]
        fpar0 = hdf['MODIS/MOD15A2HGF_fPAR'][:].astype(np.float32)
        lai0 = hdf['MODIS/MOD15A2HGF_LAI'][:].astype(np.float32)
    # NOTE: valid range for both fPAR and LAI tops out at 100
    fpar_masked = np.where(
        np.logical_or(fpar0 < 0, fpar0 > 100), np.nan, fpar0)
    lai_masked = np.where(
        np.logical_or(lai0 < 0, lai0 > 100), np.nan, lai0)
    assert not np.apply_along_axis(
        lambda x: np.isnan(x).all(), 0, np.nanmean(fpar0, axis = -1)).any()
    assert not np.apply_along_axis(
        lambda x: np.isnan(x).all(), 0, np.nanmean(lai0, axis = -1)).any()
    # Compute fPAR and LAI climatologies
    fpar_clim = climatology365(fpar_masked, dates, ignore_leap = False)
    lai_clim = climatology365(lai_masked, dates, ignore_leap = False)
    doy = np.array(ordinals365(dates))
    # Calculate truly missing values (as opposed to unobserved values within
    #   an 8-day interval) based on an 8-day moving window
    fpar_avg = np.nanmean(fpar_masked, axis = -1)
    filt = np.stack([ # Assumed to be the same for fPAR and LAI
        generic_filter(fpar_avg[:,i], lambda x: np.isnan(x).sum(), size = 9)
        for i in range(0, fpar_avg.shape[1])
    ], axis = 1)[...,np.newaxis].repeat(SUBGRID_SIZE, axis = -1)
    # Fill in climatology...
    fpar = np.where(np.logical_or(filt > 8, fpar0 > 100),
        fpar_clim[doy-1], fpar0)
    lai = np.where(np.logical_or(filt > 8, lai0 > 100),
        lai_clim[doy-1], lai0)
    # Then, replace -9999 with NaN
    fpar = np.where(fpar < 0, np.nan, fpar)
    lai = np.where(lai < 0, np.nan, lai)
    # Forward, then backward-fill, separately for each subgrid pixel
    #   (for compatibility with non-vectorized functions)
    fpar_final = np.stack([
        fill_forward(fill_forward(fpar[...,i])[::-1])[::-1]
        for i in range(0, fpar.shape[-1])
    ], axis = 2)
    lai_final = np.stack([
        fill_forward(fill_forward(lai[...,i])[::-1])[::-1]
        for i in range(0, lai.shape[-1])
    ], axis = 2)
    # There can be NaNs in the subgrid, but dropping NaNs, averaging over the
    #   subgrid should produce valid values everywhere
    if hdf_filename == MOD16_HDF:
        assert not np.isnan(np.nanmean(fpar_final, axis = -1)).any()
        assert not np.isnan(np.nanmean(lai_final, axis = -1)).any()
    with h5py.File(hdf_filename, 'a') as hdf:
        shp = fpar_final.shape
        dset = hdf.create_dataset(
            'MODIS/MOD15A2HGF_fPAR_interp', shp, fpar_final.dtype, fpar_final)
        dset.attrs['source'] = 'From Maosheng, MODIS MOD15A2HGF "Fpar_500m" Collection 6.1'
        dset.attrs['description'] = 'Filled with climatology, then forward/backward interpolated (nearest neighbor) to daily time steps'
        dset = hdf.create_dataset(
            'MODIS/MOD15A2HGF_LAI_interp', shp, lai_final.dtype, lai_final)
        dset.attrs['source'] = 'From Maosheng, MODIS MOD15A2HGF "Lai_500m" Collection 6.1'
        dset.attrs['description'] = 'Filled with climatology, then forward/backward interpolated (nearest neighbor) to daily time steps'


def fill_forward(arr):
    'Forward filling, along axis 0, of NaNs with earlier non-NaN entires'
    mask = np.isnan(arr)
    # Get the numeric index along axis 0 where non-NaN, zero otherwise
    idx = np.where(
        ~mask, np.arange(mask.shape[0])[:,np.newaxis]\
            .repeat(arr.shape[1], axis = 1), 0)
    # Fill-forward with the last non-NaN numeric index
    np.maximum.accumulate(idx, axis = 0, out = idx)
    return arr[idx, np.arange(idx.shape[1])[None,:]]


if __name__ == '__main__':
    main(MOD16_HDF)
    # main(MOD17_HDF)
