'''
Re-formatting the MCD43GF point samples that Maosheng prepared, based on
the FLUXNET sites and the 3-by-3 subgrid. Requires the upstream script:

    20220128_0101_Cal-Val_sample_site_coordinates_to_CSV.py

This version is for getting the data to drive MxD16.

NOTE: We copy the data from year 2017 to years 2018 and 2019 because, as
Maosheng wrote:

    Even repeating 2017 snow-free gap-filled albedo is much better than using
    the default value of 40% reflectance when MCD43 data is bad from 2018
    onwards, and that is how the official C61 MxD16A2/MxD16A2GF are processed.
    Please note that gap-filled albedo will be used only when MCD43A3 is bad
    (>1000) and snow/ice free (a bit for snow/ice can be derived from
    FparExtra_QC for an 8-day). There is no gap-filled MCD43 after 2017 and
    this is because science team stopped to create the data after 2017.
'''

import csv
import datetime
import glob
import os
import h5py
import numpy as np
from tqdm import tqdm

# Extracted from Maosheng's archive "MOD17_sites_MOD15A2HGF_C61_cutout_2000_2021.tar.gz"
MAOSHENG_CSV_DIR = '/home/arthur/Downloads/Maosheng/MOD16_sites_MCD43GFbsasw_C6_cutout_2000_2017'
CALVAL_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_tower_site_latent_heat_and_drivers_v4.h5'
N_SUBGRID = 9 # i.e., 3-by-3 subgrid has 9 pixels
SCALE_FACTOR = 0.001

def main():
    # Read in the FLUXNET site list
    with h5py.File(CALVAL_HDF, 'r') as hdf:
        site_list = hdf['FLUXNET/site_id'][:].tolist()
        date_list = [ # Compile list of YYYYJJJ dates, where JJJ is DOY
            datetime.date(*d).strftime('%Y%j')
            for d in hdf['time'][:].tolist()
        ]
        n_sites = len(site_list)
    if hasattr(site_list[0], 'decode'):
        # Convert site names from UTF-8 if necessary
        site_list = list(map(lambda x: x.decode('utf-8'), site_list))
    # Open the output file for writing, copy site names over
    target_hdf = h5py.File(CALVAL_HDF, 'a')
    if 'MODIS/MCD43GF_black_sky_sw_albedo' not in target_hdf.keys():
        shp = (len(date_list), n_sites, N_SUBGRID)
        empty = np.full(shp, np.nan, dtype = np.float32)
        dataset = target_hdf.create_dataset(
            'MODIS/MCD43GF_black_sky_sw_albedo', shp, np.float32, empty)
        dataset.attrs['source'] = 'From Maosheng Zhao, MODIS MCD43GF Collection 6.1'
    for i, site_id in tqdm(enumerate(site_list)):
        file_list = glob.glob(
            f'{MAOSHENG_CSV_DIR}/MCD43GFbsasw.Albedo_Map.{site_id}.20??.csv')
        file_list.sort()
        if len(file_list) == 0:
            import ipdb
            ipdb.set_trace()#FIXME
        for j, filename in enumerate(file_list):
            with open(filename, 'r') as file:
                reader = csv.reader(file)
                for line in reader:
                    date = line[0]
                    # Figure out which date this composite is associated with
                    t = date_list.index(date)
                    # Mask out fill values, re-scale valid values
                    data = map(
                        lambda x: np.nan if x > 1000 else x * SCALE_FACTOR,
                        map(int, line[1:]))
                    target_hdf['MODIS/MCD43GF_black_sky_sw_albedo'][t,i,:] = list(data)
    # Copy 2017 data to 2018 and 2019; see note above
    i = date_list.index('2017001')
    j = date_list.index('2017365')
    target_hdf['MODIS/MCD43GF_black_sky_sw_albedo'][(j+1):] =\
        np.tile(target_hdf['MODIS/MCD43GF_black_sky_sw_albedo'][i:(j+1)], (2,1,1))


if __name__ == '__main__':
    main()
