'''
Interpolating the MCD43GF point samples that Maosheng prepared, based on
the FLUXNET sites and the 3-by-3 subgrid. Requires the upstream script:

    20220128_0101_Cal-Val_sample_site_coordinates_to_CSV.py

This version is for getting the data to drive MxD16.

NOTE: We copy the data from year 2017 to years 2018 and 2019 because, as
Maosheng wrote:

    Even repeating 2017 snow-free gap-filled albedo is much better than using
    the default value of 40% reflectance when MCD43 data is bad from 2018
    onwards, and that is how the official C61 MxD16A2/MxD16A2GF are processed.
    Please note that gap-filled albedo will be used only when MCD43A3 is bad
    (>1000) and snow/ice free (a bit for snow/ice can be derived from
    FparExtra_QC for an 8-day). There is no gap-filled MCD43 after 2017 and
    this is because science team stopped to create the data after 2017.
'''

import csv
import datetime
import glob
import os
import h5py
import numpy as np
from pyl4c.science import climatology365, ordinals365

CALVAL_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_tower_site_latent_heat_and_drivers_v4.h5'

def main():
    # Read in the FLUXNET site list
    with h5py.File(CALVAL_HDF, 'r') as hdf:
        site_list = hdf['FLUXNET/site_id'][:].tolist()
        date_list = [
            datetime.date(*d) for d in hdf['time'][:].tolist()
        ]
        n_sites = len(site_list)
    if hasattr(site_list[0], 'decode'):
        # Convert site names from UTF-8 if necessary
        site_list = list(map(lambda x: x.decode('utf-8'), site_list))
    # Open the output file for writing, copy site names over
    target_hdf = h5py.File(CALVAL_HDF, 'a')
    albedo0 = target_hdf['MODIS/MCD43GF_black_sky_sw_albedo'][:]
    # Fill missing dates with climatology
    albedo_clim = climatology365(albedo0, date_list, ignore_leap = False)
    doy = np.array(ordinals365(date_list))
    albedo = np.where(np.isnan(albedo0), albedo_clim[doy-1], albedo0)
    # Then, use forward-backward filling for remaining missing dates
    albedo_fill = np.stack([
        fill_forward(fill_forward(albedo[...,i])[::-1])[::-1]
        for i in range(0, albedo.shape[-1])
    ], axis = 2)
    target_hdf['MODIS/MCD43GF_black_sky_sw_albedo'][:] = albedo_fill
    target_hdf.flush()
    target_hdf.close()


def fill_forward(arr):
    'Forward filling, along axis 0, of NaNs with earlier non-NaN entires'
    mask = np.isnan(arr)
    # Get the numeric index along axis 0 where non-NaN, zero otherwise
    idx = np.where(
        ~mask, np.arange(mask.shape[0])[:,np.newaxis]\
            .repeat(arr.shape[1], axis = 1), 0)
    # Fill-forward with the last non-NaN numeric index
    np.maximum.accumulate(idx, axis = 0, out = idx)
    return arr[idx, np.arange(idx.shape[1])[None,:]]


if __name__ == '__main__':
    main()
