# Plotting of posterior distributions for MOD17 GPP parameters
# 
#   install.packages("remotes")
#   remotes::install_github("echasnovski/ggplot2", ref="geom_density-bounds")

library(dplyr, warn.conflicts = F)
library(tidyr, warn.conflicts = F)
library(stringr)
library(hdf5r)
library(ggplot2)
library(RColorBrewer)
library(reticulate)

PFT.TABLE <- data.frame(code = c(1:10, 12),
  abbrv = c('ENF', 'EBF', 'DNF', 'DBF', 'MF', 'CSH', 'OSH', 'WSV', 'SAV', 'GRS', 'CRO'))
PFT.TABLE$abbrv <- with(PFT.TABLE, ordered(abbrv, levels = abbrv))

# Plotting LUE_max posterior ##############################

h5.post <- H5File$new('/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/20220503_MOD17_GPP_recal_posterior_LUE_max.h5', 'r')
posterior <- h5.post[['LUE_max_posterior']][1:8229,1:11]
h5.post$close()
df.posterior <- as.data.frame(posterior)
names(df.posterior) <- PFT.TABLE$abbrv
df.lue <- df.posterior %>%
  gather(key = PFT, value = value) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv))

ggplot(df.lue, mapping = aes(x = PFT, y = value * 1e3)) +
  geom_boxplot(fill = 'lightgray', color = '#555555', 
    outlier.shape = 1, outlier.color = 'lightgray') +
  geom_point(aes(y = y, color = 'prev'), shape = 19, size = 2,
    data = data.frame(y = 1e3 * c(0.001211, 0.001405, 0.001227, 0.001526, 0.001226, 
                            0.001495, 0.001027, 0.001498, 0.001454, 0.001215, 0.0013),
                      PFT = PFT.TABLE$abbrv)) +
  # Can look at the Madani et al. (2017) best-fit means
  geom_point(aes(y = y, color = 'prior'), shape = 19, size = 2,
    data = data.frame(y = c(0.98, 1.40, 1.23, 1.68, 1.43, 0.80, 0.74, 0.93, 0.93, 1.19, 1.94),
                      PFT = PFT.TABLE$abbrv)) +
  scale_y_continuous(expand = c(0, 5e-2)) +
  scale_color_manual(name = NULL, values = c('prior' = 'blue', 'prev' = 'red'), 
                     labels = c('prior' = 'Prior (Madani et al.)', 'prev' = 'Collection 6.1')) +
  labs(x = 'Plant Functional Type (PFT)', y = latex2exp::TeX('MOD17 LUE_max $(g\\,C\\,MJ^{-1})$')) +
  theme_minimal() +
  theme(legend.position = c(0.2,0.9), 
        legend.background = element_rect(color = 'darkgray', fill = 'white'),
        legend.margin = margin(-0.1, 0.2, 0.1, 0.1, 'cm'),
        plot.background = element_rect(fill = 'white', color = 'transparent'))
ggsave(width = 5, height = 3.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220503_MOD17_LUE_max_posterior.png')


# Plotting VPD posterior(s) ##############################

h5.post <- H5File$new('/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/20220503_MOD17_GPP_recal_posterior_vpd1.h5', 'r')
posterior1 <- h5.post[['vpd1_posterior']][1:8229,1:11]
h5.post$close()

df.vpd1 <- as.data.frame(posterior1)
names(df.vpd1) <- PFT.TABLE$abbrv

vpd.picks1 <- df.vpd1 %>%
  gather(key = PFT, value = value) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv)) %>%
  group_by(PFT) %>%
  filter(!is.na(value)) %>%
  summarize(Mean.Posterior = mean(value, na.rm = T),
    Max.Posterior = with(density(value, adjust = 2, kernel = 'triangular'), x[which(y == max(y))]))

df.vpd1 %>%
  gather(key = PFT, value = value) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv)) %>%
ggplot(mapping = aes(x = value)) +
  geom_density(aes(group = PFT, color = PFT), size = 0.8, key_glyph = 'smooth', 
    kernel = 'triangular', adjust = 2, bounds = c(0, 8000)) +
  scale_x_continuous(limits = c(0, 8000), expand = c(0, 0), breaks = seq(0, 8000, 2000)) +
  scale_y_continuous(expand = c(0, 0.000001)) +
  labs(x = 'MOD17 vpd1 (Pa)', y = 'Density') +
  theme_minimal() +
  theme(axis.text.y = element_text(angle = 90, hjust = 0.5),
        strip.text = element_text(face = 'bold'))
ggsave(width = 4.5, height = 4, dpi = 172,
  '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220503_vpd1_posterior_parameter_plot.png')

# Plotting Tmin posterior(s) ##############################

h5.post <- H5File$new('/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/20220503_MOD17_GPP_recal_posterior_tmin1.h5', 'r')
posterior1 <- h5.post[['tmin1_posterior']][1:8229,1:11]
h5.post$close()

df.tmin1 <- as.data.frame(posterior1)
names(df.tmin1) <- PFT.TABLE$abbrv

# Get proposed best-fit parameters
tmin.picks1 <- df.tmin1 %>%
  gather(key = PFT, value = value) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv)) %>%
  group_by(PFT) %>%
  filter(!is.na(value)) %>%
  summarize(Mean.Posterior = mean(value, na.rm = T),
    Max.Posterior = with(density(value, adjust = 1.5, kernel = 'triangular'), x[which(y == max(y))]))

df.tmin1 %>%
  gather(key = PFT, value = value) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv)) %>%
ggplot(mapping = aes(x = value)) +
  geom_density(aes(group = PFT, color = PFT), size = 0.8, key_glyph = 'smooth', 
    kernel = 'triangular', adjust = 2, bounds = c(0, 25)) +
  scale_x_continuous(limits = c(0, 25), expand = c(0, 0), breaks = seq(0, 25, 5)) +
  scale_y_continuous(expand = c(0, 0.0001)) +
  labs(x = 'MOD17 tmin1 (deg C)', y = 'Density') +
  theme_minimal() +
  theme(axis.text.y = element_text(angle = 90, hjust = 0.5),
        strip.text = element_text(face = 'bold'))
ggsave(width = 4.5, height = 4, dpi = 172,
  '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220503_tmin1_posterior_parameter_plot.png')


# Exporting BPLUT ##############################################

select(vpd.picks1, PFT, vpd1 = Max.Posterior) %>%
  mutate_at(.vars = vars(vpd1), .funs = function (x) { round(x, 0) }) %>%
  left_join(select(tmin.picks1, PFT, tmin1 = Max.Posterior), by = 'PFT') %>%
  mutate(vpd0 = c(650,1000,650,650,650,650,650,650,650,650,650)) %>%
  mutate(tmin0 = c(-8,-8,-8,-6,-7,-8,-8,-8,-8,-8,-8)) %>%
  mutate_at(.vars = vars(tmin0, tmin1), .funs = function (x) { round(x, 0) }) %>%
  select(PFT, tmin0, tmin1, vpd0, vpd1) %>%
  mutate(vpd1 = round(vpd1 / 50, 0) * 50) %>%
  gather(key = parameter, value = value, -PFT) %>%
  pivot_wider(names_from = PFT, id_cols = parameter, values_from = value) %>%
  knitr::kable(format = 'markdown')

require(reticulate)
if (Sys.info()['nodename'] == 'Gullveig') {
  use_python('/usr/local/python-env/mod17/bin/python3.8')
} else {
  use_python('/usr/local/python-env/mod17/bin/python3.10', required = TRUE)
}
mod17 <- import('mod17')

bplut <- mod17$utils$restore_bplut('/usr/local/dev/mod17/lib/python/mod17/data/MOD17_BPLUT_C5.1_MERRA_NASA.csv')

out.lue <- with(df.lue %>%
  group_by(PFT) %>%
  summarize(LUE_max = mean(value)), LUE_max)

bplut[['LUE_max']] = c(NA)
for (i in 1:11) {
  if (i == 11) {
    bplut[['LUE_max']][13] = round(out.lue[i], 6)
    bplut[['tmin1']][13] = with(tmin.picks1, round(Max.Posterior, 1))[i]
    bplut[['vpd1']][13] = with(vpd.picks1, round(Max.Posterior, 0))[i]
  } else {
    bplut[['LUE_max']][i+1] = out.lue[i]
    # Collection 6.1 values
    bplut[['tmin1']][i+1] = with(tmin.picks1, round(Max.Posterior, 1))[i]
    bplut[['vpd1']][i+1] = with(vpd.picks1, round(Max.Posterior, 0))[i]
  }
}
bplut

mod17$utils$write_bplut(dict(bplut), '/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/20220503_MOD17_GPP_recal_BPLUT.csv')


# Plotting tmin boxplots #############################

df.tmin1 %>%
  gather(key = PFT, value = value) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv)) %>%
ggplot(aes(x = value)) +
  geom_density(size = 0.8, key_glyph = 'smooth', kernel = 'triangular',
    adjust = 2, bounds = c(0, 25), fill = 'lightgray', alpha = 0.6) +
  geom_vline(aes(xintercept = y, color = 'prev', linetype = 'prev'), size = 0.9,
    data = data.frame(y = c(8.31, 9.09, 10.44, 9.94, 9.5, 8.61, 8.8, 11.39, 11.39, 12.02, 12.02),
      PFT = PFT.TABLE$abbrv), alpha = 0.6) +
  geom_vline(aes(xintercept = y, color = 'pick', linetype = 'pick'), size = 0.9,
    data = data.frame(y = tmin.picks1$Max.Posterior,
      PFT = PFT.TABLE$abbrv), alpha = 0.6) +
  scale_color_manual(name = NULL, values = c('prev' = 'red', 'pick' = 'blue', 'zhang' = 'black'), 
    labels = c('prev' = 'Collection 6.1', 'pick' = 'Posterior Maximum', 'zhang' = 'Zhang et al.')) +
  scale_linetype_manual(name = NULL, values = c('prev' = 'solid', 'pick' = 'solid', 'zhang' = 'dotted'), 
    labels = c('prev' = 'Collection 6.1', 'pick' = 'Posterior Maximum', 'zhang' = 'Zhang et al.')) +
  facet_wrap(~ PFT, ncol = 4) +
  coord_cartesian(ylim = c(0, 0.06)) +
  scale_x_continuous(expand = c(0, 0.01), breaks = seq(5, 25, 5)) +
  scale_y_continuous(expand = c(0, 0), breaks = c(0.02, 0.04)) +
  labs(x = latex2exp::TeX('MOD17 tmin1 (deg C)'), y = 'Density') +
  theme_linedraw() +
  theme(
    legend.position = 'top',
    legend.margin = margin(0, 0, -0.2, 0, 'cm'),
    axis.text.y = element_text(angle = 90, hjust = 0.5),
    strip.background = element_blank(), 
    strip.text = element_text(color = 'black', face = 'bold'))
ggsave(width = 8, height = 5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220503_MOD17_tmin1_posterior.png')


# Plotting vpd boxplots ##############################

df.vpd1 %>%
  gather(key = PFT, value = value) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv)) %>%
ggplot(aes(x = value)) +
  geom_density(size = 0.8, key_glyph = 'smooth', kernel = 'triangular',
    adjust = 2, fill = 'lightgray', alpha = 0.6, bounds = c(1000, 8000)) +
  geom_vline(aes(xintercept = y, color = 'prev', linetype = 'prev'), size = 0.9, 
    data = data.frame(y = c(3000,4000,3500,2900,2900,4300,4400,3500,3600,4200,4500),
      PFT = PFT.TABLE$abbrv), alpha = 0.6) +
  geom_vline(aes(xintercept = y, color = 'pick', linetype = 'pick'), size = 0.9, 
    data = data.frame(y = vpd.picks1$Max.Posterior,
      PFT = PFT.TABLE$abbrv), alpha = 0.6) +
  geom_vline(aes(xintercept = y, color = 'zhang', linetype = 'zhang'), size = 0.9,
    data = data.frame(y = 1000 * c(5.1, 4.8, 5.4, 6.0, 5.2, 4.0, 5.7, 5.6, 5.6, 5.8, 5.2),
      PFT = PFT.TABLE$abbrv)) +
  scale_color_manual(name = NULL, values = c('prev' = 'red', 'pick' = 'blue', 'zhang' = 'black'), 
    labels = c('prev' = 'Collection 6.1', 'pick' = 'Posterior Maximum', 'zhang' = 'Zhang et al.')) +
  scale_linetype_manual(name = NULL, values = c('prev' = 'solid', 'pick' = 'solid', 'zhang' = 'dotted'), 
    labels = c('prev' = 'Collection 6.1', 'pick' = 'Posterior Maximum', 'zhang' = 'Zhang et al.')) +
  facet_wrap(~ PFT, ncol = 4) +
  scale_x_continuous(expand = c(0, 0.01), limits = c(1000, 8000),
    breaks = seq(0, 6e3, 2e3)) +
  scale_y_continuous(expand = c(0, 0), limits = c(0, 9e-4),
    breaks = seq(3e-4, 9e-4, 3e-4)) +
  labs(x = latex2exp::TeX('MOD17 vpd1 (Pa)'), y = 'Density') +
  theme_linedraw() +
  theme(
    legend.position = 'top',
    legend.margin = margin(0, 0, -0.2, 0, 'cm'),
    strip.background = element_blank(), 
    strip.text = element_text(color = 'black', face = 'bold'),
    axis.text.y = element_text(angle = 90, hjust = 0.5))
ggsave(width = 8, height = 5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220503_MOD17_vpd1_posterior.png')
