'''
Runs MOD17 using the re-calibrated BPLUT for different parameterizations:

- Run MOD17 with fixed (at Collection 6.1): `tmin0`, `tmin1`
- Run MOD17 with fixed (at Collection 6.1): `tmin0`, `vpd0`
- Run MOD17 with fixed (at Collection 6.1): `tmin0`, `tmin1`, `vpd0`

'''

import os
import numpy as np
import h5py
import mod17
from mod17 import MOD17, PFT_VALID
from mod17.utils import restore_bplut, pft_remap

OLD_BPLUT = os.path.join(os.path.dirname(mod17.__file__), 'data/MOD17_BPLUT_C5.1_MERRA_NASA.csv')
BPLUT = '/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/20220421_MOD17_GPP_recal_BPLUT.csv'
MOD17_HDF5 = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v2.h5'
OUTPUT_HDF5 = [
    (('tmin0', 'tmin1'),
        '/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/20220421_MOD17_GPP_recal_tmin0-tmin1-fixed_run.h5'),
    (('tmin0', 'vpd0'),
        '/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/20220421_MOD17_GPP_recal_tmin0-vpd0-fixed_run.h5'),
    (('tmin0', 'tmin1', 'vpd0'),
        '/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/20220421_MOD17_GPP_recal_tmin0-tmin1-vpd0-fixed_run.h5')
]
N_DAYS = 6575
N_SITES = 356
N_SUBGRID = 9

def main(fixed, output_path):
    old_bplut = restore_bplut(OLD_BPLUT)
    params_dict = restore_bplut(BPLUT)
    for key in fixed:
        params_dict[key] = old_bplut[key]
    # NOTE: In this early draft, we conduct model calibration at site scale,
    #   ignoring subgrid heterogeneity
    with h5py.File(MOD17_HDF5, 'r') as hdf:
        sites = hdf['FLUXNET/site_id'][:]
        if hasattr(sites[0], 'decode'):
            sites = np.array(list(map(lambda x: x.decode('utf-8'), sites)))
        # Create a map of PFTs, including the site subgrid
        pft_map = pft_remap(
            hdf['state/PFT'][:].astype(np.int16), sites.tolist())
        # Invalid PFTs and blacklisted sites will be NaN
        # pft_mask = np.zeros(pft_map.shape, dtype = np.uint8)
        # for pft in PFT_VALID:
        #     pft_mask = np.where(pft_map == pft, 1, pft_mask)
        # pft_map[pft_mask == 0] = np.nan
        # NOTE: Converting from Kelvin to Celsius
        tday = hdf['MERRA2/T10M_daytime'][:][...,None] - 273.15
        qv10m = hdf['MERRA2/QV10M_daytime'][:][...,None]
        ps = hdf['MERRA2/PS_daytime'][:][...,None]
        drivers = [ # fPAR, Tmin, VPD, PAR
            hdf['MODIS/MOD15A2HGF_fPAR_interp'][:],
            hdf['MERRA2/Tmin'][:][...,None]  - 273.15,
            MOD17.vpd(qv10m, ps, tday),
            MOD17.par(hdf['MERRA2/SWGDN'][:][...,None]),
        ]
        # Convert fPAR from (%) to [0,1]
        drivers[0] /= 100
    # Create a vectorized BPLUT; there are 5 GPP parameters
    params_vector = params_dict.copy()
    for p_name in MOD17.required_parameters[0:5]:
        params_vector[p_name] = params_dict[p_name][pft_map]\
            .reshape((1,N_SITES,N_SUBGRID))
    model = MOD17(params_vector)
    gpp0 = model.daily_gpp(*drivers)
    gpp = np.nanmean(gpp0, axis = -1)
    with h5py.File(output_path, 'a') as hdf:
        if 'MOD17_GPP' in hdf.keys():
            del hdf['MOD17_GPP']
        dataset = hdf.create_dataset('MOD17_GPP', gpp.shape, np.float32, gpp)
        dataset.attrs['units'] = 'g C m-2 d-1'
        dataset.attrs['description'] = 'MOD17 GPP predictions based on BPLUT GPP parameters updated 2022-04-21; tmin0 held fixed'


if __name__ == '__main__':
    for fixed, output_path in OUTPUT_HDF5:
        main(fixed, output_path)
