'''
Runs MOD17 using the Collection 6.1 BPLUT, based on a quick optimization to
FLUXNET GPP data and the posterior mean from the stochastic MCMC sampler.

Here, using 45% of Net short-wave downward radiation (SWGNT) instead of 45% of
Incoming short-wave radiation (SWGDN).
'''

import os
import numpy as np
import h5py
import mod17
from mod17 import MOD17, PFT_VALID
from mod17.utils import restore_bplut, pft_remap

BPLUT = '/usr/local/dev/mod17/lib/python/mod17/data/MOD17_BPLUT_C5.1_MERRA_NASA.csv'
MOD17_HDF5 = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v9.h5'
OUTPUT_HDF5 = '/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/20220920_MOD17_GPP_C6-1_predictions_at_tower_sites_SWGNT.h5'
N_DAYS = 6575
N_SITES = 356
N_SUBGRID = 9

def main():
    params_dict = restore_bplut(BPLUT)
    # NOTE: In this early draft, we conduct model calibration at site scale,
    #   ignoring subgrid heterogeneity
    with h5py.File(MOD17_HDF5, 'r') as hdf:
        sites = hdf['FLUXNET/site_id'][:]
        if hasattr(sites[0], 'decode'):
            sites = np.array(list(map(lambda x: x.decode('utf-8'), sites)))
        # Create a map of PFTs, including the site subgrid
        pft_map = pft_remap(
            hdf['state/PFT'][:].astype(np.int16), sites.tolist())
        # NOTE: Converting from Kelvin to Celsius
        tday = hdf['MERRA2/T10M_daytime'][:][...,None] - 273.15
        qv10m = hdf['MERRA2/QV10M_daytime'][:][...,None]
        ps = hdf['MERRA2/PS_daytime'][:][...,None]
        drivers = [ # fPAR, Tmin, VPD, PAR
            hdf['MODIS/MOD15A2HGF_fPAR_interp'][:],
            hdf['MERRA2/Tmin'][:][...,None]  - 273.15,
            MOD17.vpd(qv10m, ps, tday),
            MOD17.par(hdf['MERRA2/SWGNT'][:][...,None]),
        ]
        # Convert fPAR from (%) to [0,1]
        drivers[0] /= 100
    # Create a vectorized BPLUT; there are 5 GPP parameters
    params_vector = params_dict.copy()
    for p_name in MOD17.required_parameters[0:5]:
        params_vector[p_name] = params_dict[p_name][pft_map]\
            .reshape((1,N_SITES,N_SUBGRID))
    model = MOD17(params_vector)
    gpp0 = model.daily_gpp(*drivers)
    gpp = np.nanmean(gpp0, axis = -1)
    with h5py.File(OUTPUT_HDF5, 'a') as hdf:
        if 'MOD17_GPP' in hdf.keys():
            del hdf['MOD17_GPP']
        dataset = hdf.create_dataset('MOD17_GPP', gpp.shape, np.float32, gpp)
        dataset.attrs['units'] = 'g C m-2 d-1'
        dataset.attrs['description'] = 'MOD17 GPP predictions based on Collection 6.1 BPLUT, but with SWGNT instead of SWGDN'


if __name__ == '__main__':
    main()
