# Validating MOD17 predictions for an intermediate re-calibration, to help
#   decide on the new parameterization

library(dplyr, warn.conflicts = F)
library(tidyr, warn.conflicts = F)
library(stringr)
library(hdf5r)
library(ggplot2)
library(RColorBrewer)
library(reticulate)

MOD17.HDF <- '/anx_lagr4/MODIS_VIIRS/validation/MODIS_MOD17A2HGF_at_FLUXNET_sites_v1.h5'
MOD17.BASELINE.HDF <- '/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/20220425_MOD17_GPP_Collection6-1_run.h5'
FLUXNET.HDF <- '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v2.h5'
PFT.TABLE <- data.frame(code = c(1:10, 12),
  abbrv = c('ENF', 'EBF', 'DNF', 'DBF', 'MF', 'CSH', 'OSH', 'WSV', 'SAV', 'GRS', 'CRO'))
PFT.TABLE$abbrv <- with(PFT.TABLE, ordered(abbrv, levels = abbrv))
BLACKLIST <- c("CN-Do1", "CN-Do2", "CN-Do3", "IT-Vig", "NL-Hor", "US-ORv", "US-WPT")

# Getting the site list
h5.mod17 <- H5File$new(MOD17.HDF, 'r')
SITE.IDS <- h5.mod17[['site_id']][1:356]
h5.mod17$close()

# Read in FLUXNET data ####################################

require(reticulate)
if (Sys.info()['nodename'] == 'Gullveig') {
  use_python('/usr/local/python-env/mod17/bin/python3.8')
} else {
  use_python('/usr/local/pythonenv/mod17/bin/python3.8', required = TRUE)
}
mod17 <- import('mod17')

# Open the MOD17 dataset, get start and end dates
h5.flux <- H5File$new(FLUXNET.HDF, 'r')
site.list <- h5.flux[['FLUXNET/site_id']][1:356]

# Use reticulate to access the dominant-PFT function
pft.map <- mod17$utils$pft_dominant(t(h5.flux[['state/PFT']][1:9,1:356]), site.list)

# Read the valid data mask
valid.mask <- h5.flux[['FLUXNET/validation_mask']][1:356,1:6575,1:13]
valid.mask.sum <- apply(valid.mask, c(1,2), sum)

# Mask out the calibration data
tmp.gpp <- h5.flux[['FLUXNET/GPP']][1:356,1:6575]
df.gpp0 <- as.data.frame(tmp.gpp)
tmp.gpp[valid.mask.sum == 0] = NA
df.gpp <- as.data.frame(tmp.gpp)
names(df.gpp) <- seq.Date(
  from = as.Date('2000-01-01'), to = as.Date('2017-12-31'), by = '1 day')
names(df.gpp0) <- names(df.gpp)
df.gpp$site.id <- site.list
df.gpp0$site.id <- site.list
df.gpp <- df.gpp %>%
  gather(key = date, value = GPP, -site.id)
df.gpp0 <- df.gpp0 %>%
  gather(key = date, value = GPP, -site.id)

# Read in the MCD12Q1 data
df.pft <- data.frame(PFT = pft.map, site.id = site.list)
# Classify these two sites as DNF instead, as we don't have any others
df.pft <- df.pft %>%
  mutate(PFT = ifelse(site.id %in% c('CA-SF2', 'CA-SF3'), 3, PFT))

# Read the PAR data for filtering GPP
# Convert down-welling short-wave radiation [W m-2] to daily PAR [MJ m-2] by
#   multiplying by (3600 seconds * 24 hours), dividing by 1e6 J, then taking 45%
df.PAR <- as.data.frame(
  h5.flux[['MERRA2/SWGDN']][1:356,1:6575] * 0.45 * 0.0036 * 24)
names(df.PAR) <- seq.Date(
  from = as.Date('2000-01-01'), to = as.Date('2017-12-31'), by = '1 day')
df.PAR$site.id <- h5.flux[['FLUXNET/site_id']][1:356]
df.PAR <- df.PAR %>%
  gather(key = date, value = PAR, -site.id)
# Join together
df.gpp <- df.gpp %>%
  left_join(df.PAR, by = c('site.id', 'date')) %>%
  left_join(df.pft, by = c('site.id'))
df.gpp0 <- df.gpp0 %>%
  left_join(df.PAR, by = c('site.id', 'date')) %>%
  left_join(df.pft, by = c('site.id'))
h5.flux$close()

# Loading MOD17 baseline predictions ######################

# Open the MOD17 dataset, get start and end dates
h5.baseline <- H5File$new(MOD17.BASELINE.HDF, 'r')
# Read in the GPP data
d.mat <- h5.baseline[['MOD17_GPP']][1:356,1:6575]
h5.baseline$close()
# Calculate mean over 3-by-3 subgrid
# d.mat <- as.data.frame(apply(d.mat, c(2, 3), mean, na.rm = T))
dates <- with(filter(df.gpp, site.id == 'AR-SLu'), date)
df.baseline <- as.data.frame(d.mat)
names(df.baseline) <- dates
df.baseline$site.id <- SITE.IDS
df.baseline <- df.baseline %>%
  gather(key = date, value = value, -site.id) %>%
  mutate(date = as.Date(date, '%Y-%m-%d'))
h5.baseline$close()


# Loading MOD17 (re-calibrated) predictions ###############

base.dir <- '/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/'
predictions <- c(
  '20220421_MOD17_GPP_recal_tmin0-fixed_run.h5',
  '20220421_MOD17_GPP_recal_tmin0-tmin1-fixed_run.h5',
  '20220421_MOD17_GPP_recal_tmin0-vpd0-fixed_run.h5',
  '20220421_MOD17_GPP_recal_tmin0-tmin1-vpd0-fixed_run.h5')

i <- 1
for (filename in predictions) {
  h5.mod17 <- H5File$new(paste0(base.dir, filename), 'r')
  # Read in the GPP data
  d.mat <- h5.mod17[['MOD17_GPP']][1:356,1:6575]
  h5.mod17$close()
  dates <- with(filter(df.gpp, site.id == 'AR-SLu'), date)
  tmp <- as.data.frame(d.mat)
  names(tmp) <- dates
  tmp$site.id <- SITE.IDS
  tmp <- tmp %>%
    gather(key = date, value = value, -site.id) %>%
    mutate(date = as.Date(date, '%Y-%m-%d')) %>%
    mutate(Model = str_extract(filename, 'tmin0.*-fixed'))
  if (i == 1) {
    df.pred <- tmp
  } else {
    df.pred <- bind_rows(df.pred, tmp)
  }
  i <- i + 1
}


# Goodness of fit: Statistics #############################

require(broom)
require(purrr, warn.conflicts = F)
df.stats <- df.gpp %>%
  filter(!(site.id %in% BLACKLIST)) %>%
  mutate(date = as.Date(date)) %>%
  filter(!is.na(GPP)) %>%
  left_join(bind_rows(df.pred, mutate(df.baseline, Model = 'Collection 6.1')),
    by = c('site.id', 'date')) %>%
  group_by(site.id) %>%
  filter(GPP >= 0) %>%
  # Filter out GPP at low levels of PAR
  filter(PAR > 0.1)

tmp.anomalies <- df.stats %>%
  mutate(ordinal = julian(date) - julian(as.Date('2000-01-01'))) %>%
  nest(data = -c(Model, PFT)) %>%
  mutate(fit.pred = map(data, ~ lm(value ~ ordinal, data = .x)),
    fit.obs = map(data, ~ lm(GPP ~ ordinal, data = .x)),
    anomalies.pred = map(fit.pred, augment),
    anomalies.obs = map(fit.obs, augment))

df.anomalies <- bind_cols(select(df.stats, Model, site.id, date, PFT, GPP, value), 
  tmp.anomalies %>%
    unnest(anomalies.pred) %>% 
    select(predicted.anomaly = .resid),
  tmp.anomalies %>%
    unnest(anomalies.obs) %>% 
    select(observed.anomaly = .resid))

df.anomalies %>%
  group_by(Model) %>%
  summarize(RMSE = sqrt(mean((GPP - value)^2, na.rm = T)),
    nRMSE = round(100 * (RMSE / (max(GPP, na.rm = T) - min(GPP, na.rm = T))), 1),
    Corr = cor(GPP, value, use = 'pairwise.complete.obs'),
    ubRMSE = sqrt(mean((observed.anomaly - predicted.anomaly)^2, na.rm = T)),
    Anomaly.Corr = cor(observed.anomaly, predicted.anomaly, use = 'pairwise.complete.obs'),
    Number = length(unique(site.id))) %>%
  select(Model, Number, RMSE, nRMSE, ubRMSE, Corr, Anomaly.Corr) %>%
knitr::kable(format = 'markdown', 
  col.names = c('Model', 'Num. Sites', 'RMSE', 'RMSE (%)', 'ubRMSE', '*r*', 'Anomaly *r*'),
  digits = 2)


