library(dplyr, warn.conflicts = F)

PFT.TABLE <- data.frame(code = c(1:10, 12),
  abbrv = c('ENF', 'EBF', 'DNF', 'DBF', 'MF', 'CSH', 'OSH', 'WSV', 'SAV', 'GRS', 'CRO'))
PFT.TABLE$abbrv <- with(PFT.TABLE, ordered(abbrv, levels = abbrv))

# Generally, ferns and herbacious annuals are not classified; also,
#   genera that are too diverse (or "comsopolitan") are not classified
GENERA.NOT.CLASSED <- c('Unidentified', 'Leaves', 'Stigmaphyllon', 'Saxifraga', 'Bonamia', 
  'Lupinus', 'Trichilia', 'Rinorea', 'Hironima', 'Santiria', 'Silene', 'Thelypteris',
  'Harrimanella', 'Potentilla', 'Antennaria', 'Ranunculus', 'Persicaria', 'Epilobium', 
  'Beckwithia', 'Trifolium', 'Galium', 'Rumex', 'Veronica', 'Achillea', 'Elaphoglossum',
  'Arabidopsis', 'Impatiens', 'Chenopodium', 'Astragalus', 'Sonchus', 'Aradopsis',
  'Capsella', 'Neslia', 'Petasites', 'Faramea', 'Viola', 'Centaurea', 'Erigeron',
  'Helianthemum', 'Taraxacum', 'Geranium', 'Solidago', 'Eriophorum', 'Symphyotrichum',
  'Lathyrus', 'Euphorbia', 'Campanula', 'Cirsium', 'Pilosella', 'Pedicularis', 
  'Atriplex', 'Lycium', 'Herniaria', 'Lespedeza', 'Sambucus', 'Mentha',
  'Lotus', 'Thalictrum', 'Hypericum', 'Anthurium', 'Senecio', 'Eupatorium', 
  'Pimpinella', 'Asclepias', 'Luzula', 'Sandwithia', 'Leucanthemum', 'Ambrosia',
  'Oenothera', 'Tabernaemontana', 'Polygonum', 'Erisma', 'Agathophora', 'Penstemon',
  'Linaria', 'Crepis', 'Urtica', 'Adenocalymma', 'Gentiana', 'Valeriana', 'Stellaria',
  'Anemone', 'Leontodon', 'Oxytropis', 'Myosotis', 'Anthriscus', 'Sedum',
  'Heisteria', 'Vochysia', 'Thymus', 'Christensonella', 'Cerastium', 'Filipendula',
  'Lysimachia', 'Laetia', 'Atractylis', 'Marila', 'Maianthemum', 'Eriogonum',
  'Erodium', 'Parthenocissus', 'Ulex', 'Geum', 'Equisetum', 'Hieracium', 'Iris',
  'Primula', 'Danaea', 'Limonium', 'Bellis', 'Wilkesia', 'Oxalis', 'Croton',
  'Asplenium', 'Sanguisorba', 'Alchemilla', 'Bidens', 'Verbascum', 'Smilax',
  'Lepidium', 'Cardamine', 'Amphicarpaea', 'Amphicarpa', 'Blechnum', 'Ambelania',
  'Prunella', 'Coreopsis', 'Thlaspi', 'Stachys', 'Dialium', 'Arenaria',
  'Tragopogon', 'Pallenis', 'Caesalpinia', 'Trophis', 'Peucedanum', 'Tanaecium',
  'Inula', 'Koelpinia', 'Campyloneurum', 'Rhinanthus', 'Hymenoxys', 'Goupia',
  'Spergularia', 'Heracleum', 'Andira', 'Acalypha', 'Sextonia', 'Notoceras',
  'Euclea', 'Dactylorhiza', 'Scorzoneroides', 'Convolvulus', 'Cassipourea',
  'Schefflera', 'Lamium', 'Fridericia', 'Simaba', 'Polygala', 'Parkia',
  'Duguetia', 'Bupleurum', 'Ouratea', 'Suaeda', 'Pseudopiptadenia', 'Aristolochia',
  'Prinostemma', 'Clitoria', 'Alseis', 'Bignonia', 'Zornia', 'Amphilophium',
  'Pittoniotis')

# Sources:
#   https://www.conifers.org/zz/gymnosperms.php
#   https://budburst.org/plants
#   https://www.tree-guide.com/
#   https://www.fao.org/fileadmin/templates/ess/documents/world_census_of_agriculture/appendix4_r7.pdf
#
# Genera with mostly "Semi-deciduous" species or that are not frequently
#   described as deciduous were classified as Evergreen
# Avicennia includes both shrubs and mangroves but only mangroves are present here
# GRS includes graminoids and sedges
PFT.pipeline <- function (df) {
  df %>%
  # Pseudolmedia is mis-spelled in some entries
  mutate(Genus = ifelse(Genus == 'Pseudomeldia', 'Pseudolmedia', Genus)) %>%
  mutate(Genus = ifelse(Genus == 'Aremisia', 'Artemesia', Genus)) %>%
  mutate(PFT1 = case_when(
    Genus %in% c('Prunus', 'Luehea', 'Xylopia', 'Sterculia', 'Sclerolobium', 
        'Tachigali', 'Desmostachys', 'Coelocaryon', 'Desbordesia', 'Pouteria',
        'Protium', 'Eschweilera', 'Guatteria', 'Iryanthera', 'Eperua', 'Couepia',
        'Coffea', 'Casearia', 'Brosimum', 'Manilkara', 'Nectandra', 'Nothofagus',
        'Cupania', 'Gustavia', 'Zanthoxylum', 'Hirtella', 'Symphonia', 'Rhamnus',
        'Chrysochlamys', 'Tapirira', 'Scaphyglottis', 'Pogonophora', 'Vitex',
        'Pachira', 'Pseudolmedia', 'Handroanthus', 'Erythroxylum', 'Neea',
        'Poecilanthe', 'Poraqueiba', 'Symplocos', 'Conceveiba', 'Citrus',
        'Ceiba', 'Litsea', 'Pistacia', 'Platypodium', 'Chomelia', 'Doliocarpus',
        'Lannea'
      ) ~ 'Broadleaf',
    Genus %in% c('Abies', 'Acmopyle', 'Araucaria', 'Athrotaxis', 'Austrocedrus', 
        'Callitris', 'Calocedrus', 'Cathaya', 'Cedrus', 'Chamaecyparis', 
        'Cryptomeria', 'Cunninghamia', 'Cupressus', 'Cycas', 'Tsuga',
        'Dacrycarpus', 'Dacrydium', 'Diselma', 'Fitzroya', 'Widdringtonia',
        'Juniperus', 'Keteleeria', 'Lagarostrobos', 'Libocedrus', 'Manoao', 
        'Microbiota', 'Nothotsuga', 'Papuacedrus', 'Pherosphaera', 'Picea', 
        'Pinus', 'Platycladus', 'Prumnopitys', 'Pseudotsuga', 'Saxegothaea', 
        'Sciadopitys', 'Sequoia', 'Sequoiadendron', 'Taiwania', 'Taxus', 
        'Tetraclinis', 'Thuja', 'Thujopsis', 'Torreya'
      ) ~ 'ENF',
    Genus %in% c('Eucalyptus', 'Euc', 'Cassia', 'Stelechocarpus', 'Astronium', 'Orbigynia',
        'Ocotea', 'Bertholletia', 'Guarea', 'Cecropia', 'Naucleopsis', 'Sorocea',
        'Trymatococcus', 'Virola', 'Theobroma', 'Terminalia', 'Klainedoxa',
        'Musanga', 'Trema', 'Anacardium', 'Ficus', 'Chrysophyllum', 'Bowenia', 
        'Ceratozamia', 'Chigua', 'Dioon', 'Encephalartos', 'Lepidozamia', 
        'Macrozamia', 'Microcycas', 'Stangeria', 'Zamia', 'Ginkgo', 'Welwitschia',
        'Gnetum', 'Agathis', 'Wollemia', 'Afrocarpus', 'Nageia', 'Podocarpus',
        'Acacia', 'Avicennia', 'Pipturus', 'Pithecellobium', 'Inga', 'Swartzia',
        'Micropholis', 'Garcinia', 'Pourouma', 'Oxandra', 'Macaranga', 'Dicorynia',
        'Bocoa', 'Sloanea', 'Unonopsis', 'Tovomita', 'Quararibea', 'Oenocarpus',
        'Dendropanax', 'Tetragastris', 'Drypetes', 'Licaria', 'Talisia', 'Maytenus',
        'Chimarrhis', 'Lacmellea', 'Tapura', 'Mouriri', 'Vouacapoua', 'Perebea',
        'Castanopsis', 'Astrocaryum', 'Pradosia', 'Minquartia', 'Siparuna',
        'Couratari', 'Vismia', 'Dendrobangia', 'Copaifera', 'Thyrsodium',
        'Socratea', 'Simarouba', 'Dacryodes', 'Annona', 'Persea', 'Cinnamomum',
        'Beilschmiedia', 'Maquira', 'Cariniana', 'Rhodostemonodaphne', 'Lacistema',
        'Philodendron', 'Mytilaria', 'Gallesia', 'Angophora', 'Corymbia'
      ) ~ 'EBF',
    Genus %in% c('Larix', 'Pseudolarix', 'Metasequoia', 'Glyptostrobus', 'Taxodium'
      ) ~ 'DNF',
    Genus %in% c('Acer', 'Quercus', 'Populus', 'Betula', 'Fagus', 'Juglans',
        'Carya', 'Cornus', 'Fraxinus', 'Liriodendron', 'Magnolia', 'Nyssa', 'Ulmus',
        'Oxydendron', 'Platanus', 'Robinia', 'Tilia', 'Distemonanthus', 'Sorbus',
        'Pterocarpus', 'Lophira', 'Triplochiton', 'Liquidambar', 'Castilla',
        'Spondias', 'Erythrophleum', 'Aesculus', 'Ailanthus', 'Alnus', 'Carapa',
        'Carpinus', 'Castanea', 'Catalpa', 'Celtis', 'Cercidiphyllum', 'Qualea',
        'Cladastris', 'Corylus', 'Gleditsia', 'Gymnocladus', 'Hemiptelea',
        'Maclura', 'Malus', 'Paulownia', 'Pterocarya', 'Staphylea', 'Dipterocarpus',
        'Styphnolobium', 'Lecythis', 'Ostrya', 'Aspidosperma', 'Jacaranda',
        'Laburnum', 'Vitellaria', 'Anogeissus', 'Cochlospermum'
      ) ~ 'DBF',
    Genus %in% c('Poa', 'Dactylis', 'Achnatherum', 'Andropogon', 'Bouteloua',
        'Bromus', 'Calamagrostis', 'Carex', 'Chasmanthium', 'Danthonia',
        'Deschampsia', 'Distichlis', 'Eragrostis', 'Hesperostipa', 'Hierochloe',
        'Juncus', 'Koeleria', 'Miscanthus', 'Nassella', 'Panicum', 'Pascopyrum',
        'Schizachyrium', 'Schoenoplectus', 'Scirpus', 'Spartina', 'Apera',
        'Sporobolus', 'Zizania', 'Stipa', 'Melilotus', 'Festuca', 'Elymus',
        'Anthoxanthum', 'Agrostis', 'Schismus', 'Phleum', 'Lolium', 'Arrhenatherum',
        'Helictotrichon', 'Alopecurus', 'Muhlenbergia', 'Holcus', 'Sorghastrum',
        'Dupontia', 'Cynosurus', 'Cleistogenes', 'Trisetum', 'Stipagrostis',
        'Arctophila', 'Aegilops', 'Leymus', 'Kobresia', 'Briza', 'Agropyron',
        'Chondrosum', 'Sesleria', 'Brachypodium', 'Aristida', 'Phragmites',
        'Glyceria', 'Eleocharis', 'Rytidosperma', 'Syzygium', 'Alloteropsis',
      	'Cenchrus', 'Digitaria', 'Oryzopsis', 'Piptatherum', 'Polypogon',
        'Schedonorous', 'Schedonorus', 'Avena'
      ) ~ 'GRS',
    Genus %in% c('Cassiope', 'Diapensia', 'Empetrum', 'Salix', 'Rhododendron', 
        'Licania', 'Brenandendron', 'Zuelania', 'Combretum', 'Serjania',
        'Ephedra', 'Falcatifolium', 'Halocarpus', 'Lepidothamnus', 'Coussarea',
        'Microcachrys', 'Parasitaxus', 'Arctostaphylos', 'Argusia', 'Hedera',
        'Artemisia', 'Baccharis', 'Berberis', 'Byrsonima', 'Cercocarpus',
        'Coccoloba', 'Encelia', 'Euonymus', 'Dodonaea', 'Gaultheria', 'Dalbergia',
        'Heteromeles', 'Ilex', 'Kalmia', 'Morella', 'Myoporum', 'Argyroxiphium',
        'Osteomeles', 'Phoradendron', 'Rhus', 'Sida', 'Vaccinium', 'Psychotria',
        'Wikstroemia', 'Crataegus', 'Styrax', 'Calophyllum', 'Caragana',
        'Dubautia', 'Miconia', 'Viburnum', 'Rubus', 'Hebe', 'Cordia',
        'Eugenia', 'Rosa', 'Diospyros', 'Ardisia', 'Salsola', 'Sapium', 'Haloxylon',
        'Apeiba', 'Ribes', 'Lyonia', 'Psidium', 'Clusia', 'Buxus', 'Ormosia',
        'Myrcia', 'Elaeagnus', 'Arbutus', 'Urera', 'Buddleja', 'Albizia',
        'Gymnosporia', 'Daphne', 'Ligustrum', 'Calluna', 'Gaylussacia', 'Larrea',
        'Prosopis', 'Securidaca', 'Comarum', 'Adelphia'
      ) ~ 'Shrub',
    Genus %in% c('Helianthus', 'Pisum', 'Solanum', 'Triticum', 'Vicia', 'Sorghum',
        'Medicago', 'Vitis', 'Oryza', 'Plantago', 'Brassica', 'Allium', 'Piper',
        'Fragaria', 'Eruca', 'Secale', 'Lens', 'Hordeum', 'Phaseolus', 'Fagopyrum',
        'Daucus', 'Manihot', 'Ceratonia', 'Ricinus', 'Apium', 'Capsicum', 'Cichorium',
        'Cicer', 'Zea', 'Vigna', 'Cucumis', 'Xanthasoma', 'Linum', 'Musa', 
        'Humulus', 'Lactuca', 'Pennisetum', 'Agave', 'Cucurbita', 'Glycine', 
        'Saccharum', 'Camellia', 'Cotton'
      ) ~ 'CRO'
  )) %>%
  mutate(PFT2 = case_when( # NOTE: Order matters here! More specific terms should appear first
    str_detect(Habitat, 'evergreen lowland tropical forest') ~ 'EBF',
    str_detect(Habitat, 'pinelands') ~ 'ENF',
    str_detect(Habitat, 'meadow') ~ 'GRS',
    str_detect(Habitat, 'eucalypt') ~ 'EBF',
    str_detect(Habitat, 'larch') ~ 'DNF',
    str_detect(Habitat, 'agroforestry') ~ 'CRO',
    str_detect(Habitat, 'heath') ~ 'GRS',
    str_detect(Habitat, 'steppe') ~ 'GRS',
    str_detect(Habitat, 'savanna woodland') ~ 'SAV',
    str_detect(Habitat, 'savanna') ~ 'SAV',
    str_detect(Habitat, 'shrub') ~ 'Shrub',
    str_detect(Habitat, 'grassland') ~ 'GRS',
    str_detect(Habitat, 'pasture') ~ 'GRS',
    str_detect(Habitat, 'rainforest') ~ 'EBF',
    str_detect(Habitat, 'evergreen.*forest') ~ 'EBF',
    str_detect(Habitat, 'deciduous.*forest') ~ 'DBF',
    str_detect(Habitat, 'forest') ~ 'MF',
    str_detect(Habitat, 'scrub') ~ 'Shrub',
    str_detect(Habitat, 'woodlands') ~ 'MF',
    is.na(Habitat) ~ '')) %>%
  mutate(PFT3 = case_when(
    str_detect(VegType, 'Alpine tundra and steppe') ~ 'GRS',
    str_detect(VegType, 'Subtropical forest complex') ~ 'MF',
    str_detect(VegType, 'Boreal and subalpine forest and shrubland') ~ 'Shrub',
    str_detect(VegType, 'Subtropical deciduous and mixed forest') ~ 'MF',
    str_detect(VegType, 'Cold-resistant crops') ~ 'CRO',
    str_detect(VegType, 'Cold-resistant crops') ~ 'CRO',
    str_detect(VegType, 'Temperate desert') ~ 'OSH',
    str_detect(VegType, 'Temperate crops') ~ 'CRO',
    str_detect(VegType, 'Temperate steppe') ~ 'GRS',
    str_detect(VegType, 'Temperate deciduous forest complex') ~ 'DBF',
    str_detect(VegType, 'Cold-resistant crops with deciduous orchards') ~ 'CRO',
    str_detect(VegType, 'Temperate needleleaf forest') ~ 'ENF',
    str_detect(VegType, 'Tropical monsoon forest complex') ~ 'EBF',
    str_detect(VegType, 'Tropical rainforest complex') ~ 'EBF',
    str_detect(VegType, 'Temperate woodland and dry grassland') ~ 'WSV',
    is.na(VegType) ~ '')) %>%
  mutate(PFT1 = ifelse(is.na(PFT1), '', PFT1),
    PFT2 = ifelse(is.na(PFT2), '', PFT2),
    PFT3 = ifelse(is.na(PFT3), '', PFT3))
}

MCD12.pipeline <- function (df) {
  df %>%
    rename(OID = ObservationID) %>%
    mutate(ObservationID = str_split(OID, ',')) %>%
    unnest(ObservationID) %>%
    select(ObservationID, Longitude, Latitude, code = PFT_MCD12Q1) %>%
    mutate(ObservationID = as.numeric(ObservationID)) %>%
    left_join(PFT.TABLE, by = 'code') %>%
    mutate(abbrv = as.character(abbrv)) %>%
    rename(PFT.mapped = abbrv) %>%
    select(-code) %>%
    group_by(ObservationID, Latitude, Longitude) %>%
    summarize(PFT.mapped = first(PFT.mapped), .groups = 'drop') %>%
    ungroup() %>%
    select(-Latitude, -Longitude)
}