# Process the TRY database export(s); reads in the subset text file, joins
#   vegetation type, habitat, and location information, then exports by
#   unique ObservationID, Latitude, and Longitude combination for further
#   processing

library(dplyr, warn.conflicts = F)
library(tidyr)
library(stringr)
library(ggplot2)
options(stringsAsFactors = F)

PFT.TABLE <- data.frame(code = c(1:10, 12),
  abbrv = c('ENF', 'EBF', 'DNF', 'DBF', 'MF', 'CSH', 'OSH', 'WSV', 'SAV', 'GRS', 'CRO'))
PFT.TABLE$abbrv <- with(PFT.TABLE, ordered(abbrv, levels = abbrv))

# Specific Leaf Area (SLA) ###########################

# For main dataset:
# cat ~/Downloads/20988_07052022180913/20988.txt | awk -F "\t" '{ if ($11 == "TraitName" || $11 == "Leaf area per leaf dry mass (specific leaf area, SLA or 1/LMA): petiole excluded" || $11 == "Leaf carbon (C) content per leaf dry mass" || $11 == "Plant biomass and allometry: Leaf dry mass per plant") print }' > /anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_SLA.txt

# For ancillary data:
# cat ~/Downloads/20988_07052022180913/20988.txt | awk -F "\t" '{ if ($11 == "TraitName" || $13 == "Climate_PFT_Biome equivalent_Poulter" || $13 == "Clustered vegetation type" || $13 == "Habitat / plot description" || $13 == "Latitude" || $13 == "Longitude") print }' > /anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_SLA_attrs.txt

df.attrs <- read.csv('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_SLA_attrs.txt', 
    sep = '\t', fileEncoding = 'latin1') %>%
  select(ObservationID, DataName, OrigValueStr, StdValue) %>%
  group_by(ObservationID, DataName) %>%
  mutate(StdValue = as.character(StdValue)) %>%
  summarize(OrigValueStr = first(OrigValueStr),
    StdValue = first(StdValue)) %>%
  # Unify the different "value" sources, preferring StdValue
  mutate(value = ifelse(is.na(StdValue) | StdValue == "", OrigValueStr, StdValue)) %>%
  select(ObservationID, DataName, value) %>%
  spread(key = DataName, value = value) %>%
  rename(Biome = `Climate_PFT_Biome equivalent_Poulter`, 
    Habitat = `Habitat / plot description`,
    VegType = `Clustered vegetation type`) %>%
  mutate(Latitude = as.numeric(Latitude), 
    Longitude = as.numeric(Longitude),
    Habitat = str_to_lower(Habitat))

df.sla <- read.csv('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_SLA.txt', sep = '\t', fileEncoding = 'latin1') %>%
  # For some reason, str_split won't work unless it is within groups
  group_by(AccSpeciesName) %>%
  mutate(Genus = str_to_title(str_split(AccSpeciesName, ' ', simplify = T)[1])) %>%
  ungroup() %>%
  left_join(df.attrs, by = 'ObservationID') %>%
  select(ObservationID, AccSpeciesName, TraitName, DataName, 
    OrigValueStr, OrigUnitStr, StdValue, UnitName, Biome, VegType, Habitat, 
    Latitude, Longitude, Reference, Genus)

df.sla %>%
  select(ObservationID, Latitude, Longitude) %>%
  filter(!is.na(Latitude) & !is.na(Longitude)) %>%
  group_by(Latitude, Longitude) %>%
  summarize(ObservationID = str_flatten(ObservationID, collapse = ',')) %>%
write.csv(row.names = F, file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/TRY/TRY_SLA_PFT.csv')

# `froot_leaf_ratio` #################################

# cat ~/Downloads/20988_07052022180913/20988.txt | awk -F "\t" '{ if ($11 == "TraitName" || $11 == "Leaf carbon (C) content per leaf dry mass" || $11 == "Fine root carbon (C) content per fine root dry mass") print }'

df.froot.leaf.ratio <- read.csv(unz('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_froot_leaf_ratio.zip', 'TRY_froot_leaf_ratio.txt'), 
    sep = '\t', fileEncoding = 'latin1') %>%
  # For some reason, str_split won't work unless it is within groups
  group_by(AccSpeciesName) %>%
  mutate(Genus = str_to_title(str_split(AccSpeciesName, ' ', simplify = T)[1])) %>%
  ungroup() %>%
  left_join(df.attrs, by = 'ObservationID') %>%
  select(ObservationID, AccSpeciesName, TraitName, DataName, 
    OrigValueStr, OrigUnitStr, StdValue, UnitName, Biome, VegType, Habitat, 
    Latitude, Longitude, Reference, Genus)

df.froot.leaf.ratio %>%
  select(ObservationID, Latitude, Longitude) %>%
  filter(!is.na(Latitude) & !is.na(Longitude)) %>%
  group_by(Latitude, Longitude) %>%
  summarize(ObservationID = str_flatten(ObservationID, collapse = ',')) %>%
write.csv(row.names = F, file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/TRY/TRY_froot_leaf_ratio_PFT.csv')


# `leaf_mr_base` #####################################

df.leaf.mr.base <- read.csv(unz('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_leaf_mr_base.zip', 'TRY_leaf_mr_base.txt'), 
    sep = '\t', fileEncoding = 'latin1') %>%
  # For some reason, str_split won't work unless it is within groups
  group_by(AccSpeciesName) %>%
  mutate(Genus = str_to_title(str_split(AccSpeciesName, ' ', simplify = T)[1])) %>%
  ungroup() %>%
  left_join(df.attrs, by = 'ObservationID') %>%
  select(ObservationID, AccSpeciesName, TraitName, DataName, 
    OrigValueStr, OrigUnitStr, StdValue, UnitName, Biome, VegType, Habitat, 
    Latitude, Longitude, Reference, Genus)

df.leaf.mr.base %>%
  select(ObservationID, Latitude, Longitude) %>%
  filter(!is.na(Latitude) & !is.na(Longitude)) %>%
  group_by(Latitude, Longitude) %>%
  summarize(ObservationID = str_flatten(ObservationID, collapse = ',')) %>%
write.csv(row.names = F, file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/TRY/TRY_leaf_mr_base_PFT.csv')


# `froot_mr_base` ####################################

# cat ~/Downloads/20988_07052022180913/20988.txt | awk -F "\t" '{ if ($11 == "TraitName" || $11 == "Plant biomass and allometry: Fine root dry mass per plant" || $11 == "Fine root carbon (C) content per fine root dry mass" || $11 == "Fine root respiration rate per fine root dry mass") print }' > /anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_froot_mr_base.txt

df.froot.mr.base <- read.csv('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_froot_mr_base.txt', sep = '\t', fileEncoding = 'latin1') %>%
  # For some reason, str_split won't work unless it is within groups
  group_by(AccSpeciesName) %>%
  mutate(Genus = str_to_title(str_split(AccSpeciesName, ' ', simplify = T)[1])) %>%
  ungroup() %>%
  left_join(df.attrs, by = 'ObservationID') %>%
  select(ObservationID, AccSpeciesName, TraitName, DataName, 
    OrigValueStr, OrigUnitStr, StdValue, UnitName, Biome, VegType, Habitat, 
    Latitude, Longitude, Reference, Genus)

df.froot.mr.base %>%
  select(ObservationID, Latitude, Longitude) %>%
  filter(!is.na(Latitude) & !is.na(Longitude)) %>%
  group_by(Latitude, Longitude) %>%
  summarize(ObservationID = str_flatten(ObservationID, collapse = ',')) %>%
write.csv(row.names = F, file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/TRY/TRY_froot_mr_base_PFT.csv')


# `livewood_leaf_ratio` #################################

# at ~/Downloads/21115_20052022145340/21115.txt | awk -F "\t" '{ if ($11 == "TraitName" || $11 == "Leaf area index (LAI) of a single plant") print '} > TRY_LAI.txt

df.lai <- read.csv(unz('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_LAI.zip', 'TRY_LAI.txt'), 
    sep = '\t', fileEncoding = 'latin1') %>%
  # For some reason, str_split won't work unless it is within groups
  group_by(AccSpeciesName) %>%
  mutate(Genus = str_to_title(str_split(AccSpeciesName, ' ', simplify = T)[1])) %>%
  ungroup() %>%
  left_join(df.attrs, by = 'ObservationID') %>%
  select(ObservationID, AccSpeciesName, TraitName, DataName, 
    OrigValueStr, OrigUnitStr, StdValue, UnitName, Biome, VegType, Habitat, 
    Latitude, Longitude, Reference, Genus)

df.lai %>%
  select(ObservationID, Latitude, Longitude) %>%
  filter(!is.na(Latitude) & !is.na(Longitude)) %>%
  group_by(Latitude, Longitude) %>%
  summarize(ObservationID = str_flatten(ObservationID, collapse = ',')) %>%
write.csv(row.names = F, file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/TRY/TRY_LAI_PFT.csv')
