'''
Extracts the MODIS MCD12Q1 land cover class for each TRY database sample
that lacks PFT information.
'''

import csv
import glob
import os
import re
import datetime
import numpy as np
from osgeo import gdal
from tqdm import tqdm
from mod17.srs import TILE_LINES, modis_tile_from_wgs84, modis_row_col_from_wgs84, modis_row_col_to_wgs84

BASE_DIR = '/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/TRY/'
CSV_LIST = [ # (input file, output file)
    ('TRY_LAI_PFT.csv', 'TRY_LAI_PFT_from_MCD12Q1.csv'),
    ('TRY_SLA_PFT.csv', 'TRY_SLA_PFT_from_MCD12Q1.csv'),
    ('TRY_froot_leaf_ratio_PFT.csv', 'TRY_froot_leaf_ratio_PFT_from_MCD12Q1.csv'),
    ('TRY_froot_mr_base_PFT.csv', 'TRY_froot_mr_base_PFT_from_MCD12Q1.csv'),
    ('TRY_leaf_mr_base_PFT.csv', 'TRY_leaf_mr_base_PFT_from_MCD12Q1.csv'),
    ('TRY_livewood_mr_base_PFT.csv', 'TRY_livewood_mr_base_PFT_from_MCD12Q1.csv'),
]
MODIS_LC_DIR = '/anx_lagr4/MODIS_VIIRS/MCD12Q1'
MODIS_LC_RX = re.compile(r'^MCD12Q1\.A2015001\.h(?P<hh>\d{2})v(?P<vv>\d{2})\.006.*\.hdf$')

def main(infile, outfile):
    'Extracts the MODIS MCD12Q1 land cover type for each TRY sample'
    with open(infile, 'r') as file:
        reader = csv.reader(file)
        all_sites = []
        all_coords = []
        for line in reader:
            if reader.line_num == 1:
                assert line == ['Latitude', 'Longitude', 'ObservationID']
                continue
            try:
                all_sites.append(line[-1])
                all_coords.append((float(line[1]), float(line[0])))
            except ValueError:
                continue
    coords = np.stack(all_coords).swapaxes(0, 1)
    grid = modis_row_col_from_wgs84(coords)
    row, col = grid.astype(np.int16)
    # Get a matrix of (H, V) or tile number elements with the same shape
    tile = modis_tile_from_wgs84(coords)
    tile_h, tile_v = tile
    # Now, to extract the land cover
    file_list = glob.glob(os.path.join(MODIS_LC_DIR, '*.hdf'))
    assert len(file_list) == 315
    # Get a container for the PFT
    results = np.full(tile_h.shape, np.nan)
    for filename in tqdm(file_list):
        hh, vv = map(int, MODIS_LC_RX.match(os.path.basename(filename)).groups())
        tile_mask = np.logical_and(tile_h == hh, tile_v == vv)
        if not tile_mask.any():
            continue # Skip tiles that contain no FLUXNET sites
        ds = gdal.Open(f'HDF4_EOS:EOS_GRID:"{filename}":MCD12Q1:LC_Type2')
        arr = ds.ReadAsArray()
        midx = np.ravel_multi_index(
            (row[tile_mask].ravel(), col[tile_mask].ravel()), arr.shape)
        pft_result = arr.ravel()[midx]
        # NOTE: "Cropland/Natural Vegetation Mosaics" is re-classified as
        #   "Croplands"
        results[tile_mask] = np.where(pft_result == 14, 12, pft_result)
    with open(outfile, 'w') as file:
        writer = csv.writer(file)
        writer.writerow(('Longitude', 'Latitude', 'ObservationID', 'PFT_MCD12Q1'))
        for i in range(0, results.size):
            writer.writerow((*all_coords[i], all_sites[i], results[i]))


if __name__ == '__main__':
    for infile, outfile in CSV_LIST:
        infile = os.path.join(BASE_DIR, infile)
        outfile = os.path.join(BASE_DIR, outfile)
        main(infile, outfile)
