# Process the TRY database export(s) to determine priors for NPP calibration

# Could improve known Genera by using:
#   https://eol.org/pages/2895126
#   https://plants.usda.gov/home

# TODOs
# [x] The "mr_base" parameters are defined at 20 degrees C; are temperature
#   data available for the traits? Yes, but rarely.
# [x] Double-check Likelihood ratio test
# [x] Double-check `df.froot.leaf.ratio` for Leaf dry mass per plant UNITS

library(dplyr, warn.conflicts = F)
library(tidyr)
library(stringr)
library(ggplot2)
library(latex2exp)
library(bbmle, warn.conflicts = F)
options(stringsAsFactors = F)
Sys.setlocale("LC_ALL", "C")

CO2.MOLAR.MASS <- 44.009 # grams per mol
C.MOLAR.MASS <- 12.0107 # grams per mol

discrete.mode <- function (x) {
  u <- unique(x)
  tab <- tabulate(match(x, u))
  u[tab == max(tab)]
}

PFT.TABLE <- data.frame(code = c(1:10, 12),
  abbrv = c('ENF', 'EBF', 'DNF', 'DBF', 'MF', 'CSH', 'OSH', 'WSV', 'SAV', 'GRS', 'CRO'))
PFT.TABLE$abbrv <- with(PFT.TABLE, ordered(abbrv, levels = abbrv))

source('~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/workflows/Y2021_MOD16_MOD17_calibration/20220505_0001_MOD17_NPP_calibration_priors_TRY_database_plant_genera.R')

# For ancillary data:
# cat ~/Downloads/20988_07052022180913/20988.txt | awk -F "\t" '{ if ($11 == "TraitName" || $13 == "Climate_PFT_Biome equivalent_Poulter" || $13 == "Clustered vegetation type" || $13 == "Habitat / plot description" || $13 == "Latitude" || $13 == "Longitude") print }' > /anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_SLA_attrs.txt

# Must use: Sys.setlocale("LC_ALL", "C")
df.attrs <- read.csv(unz('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_SLA_attrs.zip', 'TRY_SLA_attrs.txt'),
    sep = '\t', fileEncoding = 'latin1') %>%
  select(ObservationID, DataName, OrigValueStr, StdValue) %>%
  group_by(ObservationID, DataName) %>%
  mutate(StdValue = as.character(StdValue)) %>%
  summarize(OrigValueStr = first(OrigValueStr),
    StdValue = first(StdValue)) %>%
  # Unify the different "value" sources, preferring StdValue
  mutate(value = ifelse(is.na(StdValue) | StdValue == "", OrigValueStr, StdValue)) %>%
  select(ObservationID, DataName, value) %>%
  spread(key = DataName, value = value) %>%
  rename(Biome = `Climate_PFT_Biome equivalent_Poulter`,
    Habitat = `Habitat / plot description`,
    VegType = `Clustered vegetation type`) %>%
  mutate(Latitude = as.numeric(Latitude),
    Longitude = as.numeric(Longitude),
    Habitat = str_to_lower(Habitat))


# Q10 of leaf respiration ############################

df.try <- read.csv(unz('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_database_export_20987_Q10.zip', '20987.txt'),
    sep = '\t') %>%
  # For some reason, str_split won't work unless it is within groups
  group_by(AccSpeciesName) %>%
  mutate(Genus = str_to_title(str_split(AccSpeciesName, ' ', simplify = T)[1]))

tmp <- df.try %>%
  filter(TraitName == 'Leaf respiration rate in the dark temperature dependence') %>%
  filter(str_detect(DataName, 'Q10.*')) %>%
  group_by(ObservationID, AccSpeciesName) %>%
  select(ObservationID, AccSpeciesName, DataName, OriglName, OrigValueStr, OrigUnitStr, StdValue, UnitName, Genus, Reference) %>%
  mutate(value = as.numeric(OrigValueStr), std.value = as.numeric(StdValue)) %>%
  mutate(Habitat = NA, VegType = NA) %>%
  PFT.pipeline %>%
  # NOTE: Values from one reference were probably entered incorrectly; the
  #   publication was checked and it does report Q10 values in the range [2,4]
  mutate(value = ifelse(value > 150, value / 100, value)) %>%
  rename(PFT = PFT1) %>%
  select(-PFT2, -PFT3)

df.q10 <- bind_rows(filter(tmp, PFT %in% c('ENF', 'DNF', 'EBF', 'DBF', 'GRS', 'CRO')),
  mutate(filter(tmp, PFT %in% c('ENF', 'DNF', 'EBF', 'DBF')), PFT = 'MF'),
  mutate(filter(tmp, PFT %in% c('Broadleaf')), PFT = 'DBF'),
  mutate(filter(tmp, PFT %in% c('Broadleaf')), PFT = 'EBF'),
  mutate(filter(tmp, PFT == 'Shrub'), PFT = 'CSH'),
  mutate(filter(tmp, PFT == 'Shrub'), PFT = 'OSH'),
  mutate(tmp, PFT = 'WSV'),
  mutate(tmp, PFT = 'SAV')) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv))

ggplot(df.q10, mapping = aes(x = PFT, y = value)) +
  geom_boxplot() +
  labs(y = 'Reported Q10 for Leaf Dark Respiration') +
  coord_flip() +
  theme_minimal()
ggsave(width = 4.5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_prior_Q10_leaf_respiration.png')


# Q10 of stem respiration ############################

tmp <- df.try %>%
  filter(TraitName == 'Stem respiration temperature dependence') %>%
  filter(str_detect(DataName, '.*Q10.*')) %>%
  group_by(ObservationID, AccSpeciesName) %>%
  select(ObservationID, AccSpeciesName, DataName, OriglName, OrigValueStr, OrigUnitStr, StdValue, UnitName, Genus, Reference) %>%
  mutate(value = as.numeric(OrigValueStr), std.value = as.numeric(StdValue)) %>%
  mutate(Habitat = NA, VegType = NA) %>%
  PFT.pipeline %>%
  rename(PFT = PFT1) %>%
  select(-PFT2, -PFT3)

# NOTE: Stem Q10 should be zero for GRS and CRO
df.q10.stem <- bind_rows(filter(tmp, PFT %in% c('ENF', 'EBF', 'DBF')),
  mutate(filter(tmp, PFT %in% c('ENF', 'EBF', 'DBF')), PFT = 'MF'),
  mutate(filter(tmp, PFT == 'ENF'), PFT = 'DNF'),
  mutate(tmp, PFT = 'CSH'),
  mutate(tmp, PFT = 'OSH'),
  mutate(tmp, PFT = 'WSV'),
  mutate(tmp, PFT = 'SAV'),
  mutate(tmp, PFT = 'GRS'),
  mutate(tmp, PFT = 'CRO')) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv))

ggplot(df.q10.stem, mapping = aes(x = PFT, y = value)) +
  geom_boxplot() +
  labs(y = 'Reported Q10 for Stem Respiration') +
  coord_flip() +
  theme_minimal()
ggsave(width = 4.5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_prior_Q10_stem_respiration.png')

# As with Q10 for leaves, values are quasi-normally distributed
# ggplot(df.q10.stem, mapping = aes(x = value)) +
#   geom_histogram() +
#   facet_wrap(~ PFT) +
#   theme_linedraw()

df.q10.stem %>%
  group_by(PFT) %>%
  summarize(Mean = mean(value, na.rm = T),
    Std.Dev = sd(value, na.rm = T))

# NOTE: MLE is not used here because the distribution is apparently normal
#   and we want to reproduce the observed moments
with(filter(df.q10.stem, PFT == 'WSV'),
  plot(density(value), lwd = 2, bty = 'n',
    xlim = c(0, 5), ylim = c(0, 4.0), xlab = 'Q10 Stem Respiration',
    ylab = 'Density', main = ''))
with(filter(df.q10.stem, PFT == 'ENF'),
  lines(density(value), lwd = 2, col = 'red'))
plot.function(function (x) { dnorm(x, mean = 1.84, sd = 0.22) },
  from = 0, to = 5, type = 'l', bty = 'n', lwd = 2, lty = 'dashed', add = TRUE,
  xlab = 'Q10 Stem Respiration', ylab = 'Density', ylim = c(0, 3.2))
plot.function(function (x) { dnorm(x, mean = 1.89, sd = 0.27) },
  from = 0, to = 5, type = 'l', lwd = 2, lty = 'dashed', add = TRUE, col = 'red')
# plot.function(function (x) { dnorm(x, mean = 1.81, sd = 0.12) },
#   from = 0, to = 5, type = 'l', lwd = 2, add = TRUE, col = 'blue')
legend('topright',
  legend = c(latex2exp::TeX('Needleleaf $(\\mu=1.89,\\sigma=0.27)$'),
    latex2exp::TeX('Others $(\\mu=1.84, \\sigma=0.22)$')),
  col = c('red', 'black'), lwd = 2)


# Q10 of root respiration ######################################

# NOTE: From the literature, not TRY

# plot.function(function (x) { dnorm(x, mean = 1.6, sd = 1.6) },
#   from = 0, to = 5, type = 'l', bty = 'n', lwd = 2,
#   xlab = 'Q10 Fine Root Respiration', ylab = 'Density')


# Load ancillary temperature data ##############################

# For ancillary temperature data:
# cat ~/Downloads/20988_07052022180913/20988.txt | awk -F "\t" '{ if ($13 == "DataName" || $13 == "Temperature during measurement" || $13 == "Temperature during measurement: maximum" || $13 == "Temperature during respiration measurements") print }' > ~/Downloads/TRY_SLA_temperature_attrs.txt
df.attrs.temp <- read.csv(unz('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_SLA_temperature_attrs.zip', 'TRY_SLA_temperature_attrs.txt'),
    sep = '\t', fileEncoding = 'latin1') %>%
  select(ObservationID, DataName, OrigValueStr, StdValue, OrigUnitStr) %>%
  group_by(ObservationID, DataName) %>%
  mutate(StdValue = as.character(StdValue)) %>%
  summarize(OrigValueStr = first(OrigValueStr),
    StdValue = first(StdValue),
    OrigUnitStr = first(OrigUnitStr)) %>%
  # Unify the different "value" sources, preferring StdValue
  mutate(value = ifelse(is.na(StdValue) | StdValue == "", OrigValueStr, StdValue)) %>%
  select(ObservationID, DataName, value) %>%
  spread(key = DataName, value = value) %>%
  rename(Temperature2 = `Temperature during measurement`,
    Temperature = `Temperature during respiration measurements`,
    Temperature3 = `Temperature during measurement: maximum`) %>%
  mutate(across(starts_with('Temperature'), as.numeric)) %>%
  mutate(Temperature = ifelse(is.na(Temperature),
    ifelse(is.na(Temperature2), Temperature3, Temperature2), Temperature)) %>%
  select(-Temperature2, -Temperature3)

# my.hist <- hist(df.attrs.temp$Temperature, breaks = 20)
# my.hist$counts <- cumsum(my.hist$counts)
# plot(my.hist, xlab = 'Temperature (deg C)', main = 'Recorded Temperature during Trait Measurement')

# Specific Leaf Area (SLA) ###########################

# For SLA dataset:
# cat ~/Downloads/20988_07052022180913/20988.txt | awk -F "\t" '{ if ($11 == "TraitName" || $11 == "Leaf area per leaf dry mass (specific leaf area, SLA or 1/LMA): petiole excluded") print }' > /anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_SLA.txt

df.sla <- read.csv(unz('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_SLA.zip', 'TRY_SLA.txt'),
    sep = '\t', fileEncoding = 'latin1') %>%
  # For some reason, str_split won't work unless it is within groups
  group_by(AccSpeciesName) %>%
  mutate(Genus = str_to_title(str_split(AccSpeciesName, ' ', simplify = T)[1])) %>%
  ungroup() %>%
  left_join(df.attrs, by = 'ObservationID') %>%
  select(ObservationID, AccSpeciesName, TraitName, DataName,
    OrigValueStr, OrigUnitStr, StdValue, UnitName, Biome, VegType, Habitat,
    Latitude, Longitude, Reference, Genus) %>%
  PFT.pipeline %>%
  mutate(PFT = ifelse(PFT1 == '',
    ifelse(PFT2 == '',
      ifelse(PFT3 == '', NA, PFT3), PFT2), PFT1)) %>%
  select(-PFT1, -PFT2, -PFT3) %>%
  # NOTE: Giving up on trying to derive SLA in C terms (see version 0402)
  filter(TraitName == 'Leaf area per leaf dry mass (specific leaf area, SLA or 1/LMA): petiole excluded')

df.sla.pft <- read.csv('~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/TRY/TRY_SLA_PFT_from_MCD12Q1.csv') %>%
  MCD12.pipeline

# Join the MODIS MCD12Q1 PFT data for those Genera not already assigned PFTs
df.sla.joined <- df.sla %>%
  filter(!is.na(StdValue)) %>%
  left_join(df.sla.pft, by = 'ObservationID') %>%
  mutate(PFT = ifelse(is.na(PFT), PFT.mapped, PFT))

# Consolidate functional groups into PFTs, then aggregate traits by species, taking
#   the majority PFT class
df.sla.by.species <- bind_rows(
  filter(df.sla.joined, PFT %in% c('ENF', 'EBF', 'DBF', 'MF', 'SAV', 'WSV', 'GRS', 'CRO')),
  mutate(filter(df.sla.joined, PFT %in% c('ENF', 'DNF')), PFT = 'DNF'),
  mutate(filter(df.sla.joined, PFT %in% c('Shrub', 'CSH')), PFT = 'CSH'),
  mutate(filter(df.sla.joined, PFT %in% c('Shrub', 'OSH')), PFT = 'OSH')) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv)) %>%
  # Remove varieties and trailing whitespace
  mutate(AccSpeciesName = str_remove(str_remove(AccSpeciesName, 'var\\..*'), ' $')) %>%
  group_by(AccSpeciesName, Genus, TraitName) %>%
  summarize(StdValue = median(StdValue, na.rm = T),
    PFT = discrete.mode(PFT)) %>%
  filter(StdValue < 100)

# Expected differences in 1/LMA given Leaf economics spectrum (see Diaz et al. 2016, Nature)
df.sla.by.species %>%
ggplot(mapping = aes(x = PFT, y = StdValue)) +
  geom_boxplot(outlier.shape = 1) +
  geom_point(aes(y = y, color = 'prev'), shape = 19, size = 2,
    data = data.frame(y = c(15,26.9,16.9,24.7,22.6,9.4,12,28.8,28.9,38,38), PFT = PFT.TABLE$abbrv)) +
  coord_flip() +
  scale_color_manual(name = NULL, values = c('prev' = 'red'),
    labels = c('prev' = 'Collection 6.1')) +
  labs(y = latex2exp::TeX('1/LMA ($mm^2\\,mg^{-1}$)')) +
  theme_linedraw() +
  theme(legend.position = c(0.84,0.14),
        legend.background = element_rect(color = 'darkgray', fill = 'white'),
        legend.margin = margin(-0.1, 0.2, 0.1, 0.1, 'cm'))
ggsave(width = 4.5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_LMA-inverse.png')

# Finding initial values for MLE
init.sla <- df.sla.summary %>%
  group_by(PFT) %>%
  summarize(mu = log(mean(StdValue)),
    sigma = log(sd(StdValue)))

# Maximum-likelihood fit of Log-normal distribution for pooled species
require(bbmle)
results <- matrix(nrow = 11, ncol = 5)
i <- 1
for (pft in levels(df.sla.by.species$PFT)) {
  nll.lnorm <- function (mu, sigma) {
    -sum(dlnorm(x = filter(df.sla.by.species, PFT == pft)$StdValue,
      meanlog = mu, sdlog = sigma, log = T))
  }
  nll.norm <- function (mu, sigma) {
    -sum(dnorm(x = filter(df.sla.by.species, PFT == pft)$StdValue,
      mean = mu, sd = sigma, log = T))
  }
  fit1 <- mle2(nll.lnorm, start = list(mu = 2.86, sigma = 0.52))
  fit2 <- mle2(nll.norm, start = list(mu = 20, sigma = 10.8))
  lrt = 2 * (-nll.lnorm(coef(fit1)[1], coef(fit1)[2]) - -nll.lnorm(2.863, 0.524))
  p.value <- 1 - pchisq(lrt, 2)
  print(sprintf('LRT (%s): %.2f (p-value: %.3f)', pft, lrt, p.value))
  print(sprintf('AIC (LogN): %.1f', AIC(fit1)))
  print(sprintf('AIC (Norm): %.1f', AIC(fit2)))
  results[i,] <- c(p.value, AIC(fit1), AIC(fit2), coef(fit1))
  i <- i + 1
  rm(fit1, fit2, p.value, lrt)
}
# Log-normal AIC is always lower than Normal AIC

# KDE plot
with(filter(df.sla.summary, PFT == 'WSV'),
  plot(density(StdValue), lwd = 2, bty = 'n',
    xlim = c(0, 100), ylim = c(0, 0.12),
    xlab = latex2exp::TeX('1/LMA $(m^2\\,kg^{-1})$'),
    main = ''))
with(filter(df.sla.summary, PFT == 'ENF'),
  lines(density(StdValue), lwd = 2, col = 'blue'))
with(filter(df.sla.summary, PFT == 'CSH'),
  lines(density(StdValue), lwd = 2, col = 'red'))
with(filter(df.sla.summary, PFT == 'CRO'),
  lines(density(StdValue), lwd = 2, col = 'green'))
with(filter(df.sla.summary, PFT == 'WSV'), rug(StdValue))
plot.function(function (x) { dlnorm(x, meanlog = 2.86, sdlog = 0.52) },
  from = 0, to = 100, type = 'l', bty = 'n', lwd = 2, col = 'black',
  lty = 'dashed', add = T)
plot.function(function (x) { dlnorm(x, meanlog = 2.25, sdlog = 0.73) }, # ENF
  from = 0, to = 100, type = 'l', bty = 'n', lwd = 2, col = 'blue',
  lty = 'dashed', add = T)
plot.function(function (x) { dlnorm(x, meanlog = 2.64, sdlog = 0.55) }, # Shrub
  from = 0, to = 100, type = 'l', bty = 'n', lwd = 2, col = 'red',
  lty = 'dashed', add = T)
plot.function(function (x) { dlnorm(x, meanlog = 2.98, sdlog = 0.50) }, # CRO
  from = 0, to = 100, type = 'l', bty = 'n', lwd = 2, col = 'green',
  lty = 'dashed', add = T)
legend('topright',
  legend = c('KDE (Observed)', 'KDE (Observed, ENF)', 'KDE (Observed, Shrub)', 'KDE (Observed, CRO)',
    TeX('LogNormal$(\\mu=2.86,\\sigma=0.52)$'),
    TeX('LogNormal$(\\mu=2.25,\\sigma=0.73)$'),
    TeX('LogNormal$(\\mu=2.64,\\sigma=0.55)$'),
    TeX('LogNormal$(\\mu=2.98,\\sigma=0.50)$')),
  col = rep(c('black', 'blue', 'red', 'green'), 2), lwd = 2,
  lty = rep(c('solid', 'dashed'), each = 4))

tmp <- as.data.frame(results) %>%
  rename(p.value = V1, lognormal.AIC = V2, normal.AIC = V3, mu = V4, sigma = V5) %>%
  mutate(code = c(1:10, 12)) %>%
  left_join(PFT.TABLE, by = 'code')
i <- 1
for (pft in PFT.TABLE$abbrv) {
  plot.function(function (x) { with(filter(tmp, abbrv == pft), dlnorm(x, mu, sigma)) },
    from = 0, to = 75, main = 'Maximum likelihood estimate',
    xlab = TeX('1/LMA ($m^2\\,kg^{-1}$)'), ylab = 'Density', bty = 'n', lwd = 2,
    col = viridisLite::viridis(11)[i], add = (pft != 'ENF'))
  i <- i + 1
}


# `froot_leaf_ratio` ###########################################

# cat ~/Downloads/20988_07052022180913/20988.txt | awk -F "\t" '{ if ($11 == "TraitName" || $11 == "Leaf carbon (C) content per leaf dry mass" || $11 == "Fine root carbon (C) content per fine root dry mass") print }'

df.froot.leaf.ratio <- read.csv(unz('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_froot_leaf_ratio.zip', 'TRY_froot_leaf_ratio.txt'),
  sep = '\t', fileEncoding = 'latin1') %>%
  # For some reason, str_split won't work unless it is within groups
  group_by(AccSpeciesName) %>%
  mutate(Genus = str_to_title(str_split(AccSpeciesName, ' ', simplify = T)[1])) %>%
  ungroup() %>%
  left_join(df.attrs, by = 'ObservationID') %>%
  PFT.pipeline %>%
  mutate(PFT = ifelse(PFT1 == '',
    ifelse(PFT2 == '',
      ifelse(PFT3 == '', NA, PFT3), PFT2), PFT1)) %>%
  select(ObservationID, AccSpeciesName, TraitName, DataName,
    OrigValueStr, OrigUnitStr, StdValue, UnitName, PFT, Reference, Genus)

df.froot.leaf.ratio.pft <- read.csv('~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/TRY/TRY_froot_leaf_ratio_PFT_from_MCD12Q1.csv') %>%
  MCD12.pipeline

# Join the MODIS MCD12Q1 PFT data for those Genera not already assigned PFTs
df.froot.leaf.ratio.joined <- df.froot.leaf.ratio %>%
  filter(!is.na(StdValue)) %>%
  left_join(df.froot.leaf.ratio.pft, by = 'ObservationID') %>%
  mutate(PFT = ifelse(is.na(PFT), PFT.mapped, PFT))

# Consolidate functional groups into PFTs, then aggregate traits by species, taking
#   the majority PFT class
df.froot.leaf.ratio.by.species <- bind_rows(
  filter(df.froot.leaf.ratio.joined, PFT %in% c('ENF', 'EBF', 'DBF', 'GRS')),
  mutate(filter(df.froot.leaf.ratio.joined, PFT %in% c('ENF', 'DNF')),
    PFT = 'DNF'),
  mutate(filter(df.froot.leaf.ratio.joined, PFT %in% c('ENF', 'EBF', 'DNF', 'DBF', 'MF', 'Broadleaf')),
    PFT = 'MF'),
  mutate(filter(df.froot.leaf.ratio.joined, PFT %in% c('Shrub', 'CSH')), PFT = 'CSH'),
  mutate(filter(df.froot.leaf.ratio.joined, PFT %in% c('Shrub', 'OSH')), PFT = 'OSH'),
  mutate(df.froot.leaf.ratio.joined, PFT = 'SAV'),
  mutate(df.froot.leaf.ratio.joined, PFT = 'WSV'),
  mutate(df.froot.leaf.ratio.joined, PFT = 'CRO')) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv)) %>%
  # Remove varieties and trailing whitespace
  mutate(AccSpeciesName = str_remove(str_remove(AccSpeciesName, 'var\\..*'), ' $')) %>%
  group_by(AccSpeciesName, Genus, TraitName) %>%
  summarize(StdValue = median(StdValue, na.rm = T),
    PFT = discrete.mode(PFT))

df.froot.leaf.ratio.by.species %>%
  filter(TraitName == "Fine root carbon (C) content per fine root dry mass") %>%
ggplot(mapping = aes(x = PFT, y = StdValue)) +
  geom_boxplot() +
  coord_flip() +
  labs(y = 'Fine root carbon per fine root dry mass (mg/g)') +
  theme_linedraw()
ggsave(width = 4.5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_FineRootC.png')

df.froot.leaf.ratio.by.species %>%
  filter(TraitName == "Leaf carbon (C) content per leaf dry mass") %>%
ggplot(mapping = aes(x = PFT, y = StdValue)) +
  geom_boxplot() +
  coord_flip() +
  labs(y = 'Leaf carbon per leaf dry mass (mg/g)') +
  theme_linedraw()
ggsave(width = 4.5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_LeafC.png')

# Extract trait names
df.froot.leaf.ratio.summary <- df.froot.leaf.ratio.by.species %>%
  select(AccSpeciesName, TraitName, StdValue, Genus, PFT) %>%
  mutate(TraitName = str_extract(TraitName, '(Fine root carbon|Leaf carbon|Fine root dry mass|Leaf dry mass)')) %>%
  ungroup() %>%
  spread(key = TraitName, value = StdValue) %>%
  # Convert "X carbon (C) per X dry mass" quantities to "X carbon (C)"
  mutate(FineRootC = `Fine root carbon` * `Fine root dry mass`) %>%
  mutate(LeafC = `Leaf carbon` * `Leaf dry mass`) %>%
  # Calculate Fine root C : Leaf C ratio; this is done two different ways
  mutate(froot.leaf.ratio = `Fine root carbon` / `Leaf carbon`,
    froot.leaf.ratio2 = FineRootC / LeafC) %>%
  filter(!is.na(froot.leaf.ratio))

ggplot(df.froot.leaf.ratio.summary, mapping = aes(x = froot.leaf.ratio)) +
  geom_histogram(bins = 30) +
  facet_wrap(~ PFT) +
  theme_linedraw()

# Maximum-likelihood fit of Log-normal distribution for pooled species
require(bbmle)
results <- matrix(nrow = 11, ncol = 7)
i <- 1
for (pft in levels(df.froot.leaf.ratio.summary$PFT)) {
  nll.lnorm <- function (mu, sigma) {
    -sum(dlnorm(x = filter(df.froot.leaf.ratio.summary, PFT == pft)$froot.leaf.ratio,
      meanlog = mu, sdlog = sigma, log = T))
  }
  nll.norm <- function (mu, sigma) {
    -sum(dnorm(x = filter(df.froot.leaf.ratio.summary, PFT == pft)$froot.leaf.ratio,
      mean = mu, sd = sigma, log = T))
  }
  fit1 <- mle2(nll.lnorm, start = list(mu = -0.01, sigma = 0.22))
  fit2 <- mle2(nll.norm, start = list(mu = 1.04, sigma = 0.32))
  lrt = 2 * (-nll.lnorm(coef(fit1)[1], coef(fit1)[2]) - -nll.lnorm(-0.01, 0.22))
  p.value <- 1 - pchisq(lrt, 2)
  print(sprintf('LRT (%s): %.2f (p-value: %.3f)', pft, lrt, p.value))
  print(sprintf('AIC (LogN): %.1f', AIC(fit1)))
  print(sprintf('AIC (Norm): %.1f', AIC(fit2)))
  results[i,] <- c(p.value, AIC(fit1), AIC(fit2), coef(fit1), coef(fit2))
  i <- i + 1
  rm(fit1, fit2, p.value, lrt)
}
# Normal fits better in 6/11 cases, under AIC; LogNormal better in 5/11 cases,
#   but where LogNormal AIC is lower it is MUCH lower than Normal; can't say
#   the same about where Normal AIC is lower.
# LRT indicates that DBF (p-value < 0.001), MF (p-value < 0.001),
#   CSH (p-value = 0.036), OSH (p-value = 0.036), and GRS (p-value < 0.001)
#   have different prior distributions from the pooled distribution

round(ifelse(results[,1] < 0.05, results[,4], results[8,4]), 3)
round(ifelse(results[,1] < 0.05, results[,5], results[8,5]), 3)

nll.lnorm <- function (mu, sigma) {
  -sum(dlnorm(x = filter(df.froot.leaf.ratio.summary, PFT == 'WSV')$froot.leaf.ratio,
    meanlog = mu, sdlog = sigma, log = T))
}
df.sweep <- expand.grid(mu = seq(-1, 1, 0.01), sigma = seq(0.05, 1, 0.05)) %>%
  group_by(mu, sigma) %>%
  mutate(loglik = -nll.lnorm(mu, sigma))

ggplot(df.sweep, mapping = aes(x = mu, y = sigma)) +
  geom_tile(aes(fill = loglik)) +
  geom_contour(aes(z = loglik), binwidth = 500, color = 'black') +
  geom_point(aes(x = mu, y = sigma), shape = 10, size = 5, color = 'black',
    data = data.frame(mu = 0.01, sigma = 0.22, loglik = NA)) +
  geom_vline(aes(xintercept = mu), color = 'black', linetype = 'dashed',
    data = data.frame(mu = 0.01)) +
  scale_fill_viridis_c() +
  scale_x_continuous(expand = c(0, 0)) +
  scale_y_continuous(expand = c(0, 0)) +
  labs(x = 'Log10(Mean)', y = 'Log10(Std. Deviation)',
    fill = 'Log-\nLikelihood') +
  theme_minimal()
ggsave(width = 5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_froot_leaf_ratio_MLE.png')

plot(density(df.froot.leaf.ratio.summary$froot.leaf.ratio),
  type = 'l', lwd = 2, xlim = c(0, 3.5), ylim = c(0, 5), bty = 'n',
  xlab = 'Fine Root-Leaf C Ratio', ylab = 'Density', main = '')
plot.function(function (x) { dlnorm(x, meanlog = 0.01, sdlog = 0.22) },
  from = 0, to = 6, type = 'l', lty = 'dashed', bty = 'n', lwd = 2, col = 'red', add = T)
plot.function(function (x) { dnorm(x, mean = 1.04, sd = 0.32) },
  from = 0, to = 6, type = 'l', lty = 'dashed', bty = 'n', lwd = 2, col = 'blue', add = T)
plot.function(function (x) { dlnorm(x, meanlog = 0.07, sdlog = 0.33) },
  from = 0, to = 6, type = 'l', lty = 'dashed', bty = 'n', lwd = 2, col = 'green', add = T)
legend('topright',
  legend = c('KDE (Observed)',
    latex2exp::TeX('Normal$(\\mu=1.04, \\sigma=0.32)$'),
    latex2exp::TeX('LogNormal$(\\mu=0.01,\\sigma=0.22)$'),
    latex2exp::TeX('LogNormal$(\\mu=0.07, \\sigma=0.33)$ (GRS)')),
  col = c('black', 'blue', 'red', 'green'), lty = c('solid', 'dashed', 'dashed', 'dashed'), lwd = 2)

ggplot(df.froot.leaf.ratio.summary) +
  geom_histogram(aes(x = froot.leaf.ratio), bins = 30, fill = 'lightblue') +
  geom_function(fun = function (x) { 10 * dnorm(x, mean = 1.01, sd = 0.27) },
    size = 0.8) +
  geom_text(mapping = aes(x = x, y = y), label = latex2exp::TeX('Normal$(1.01, 0.27)$'),
    data = data.frame(x = 2, y = 18.5)) +
  scale_x_continuous(expand = c(0, 0), limits = c(0, 6),
    breaks = seq(0, 6, 1)) +
  scale_y_continuous(expand = c(0, 0), limits = c(0, 20),
    breaks = seq(10, 100, 10)) +
  labs(x = 'Fine Root-Leaf Mass Carbon Ratio', y = latex2exp::TeX('Count or (Density$\\times 20$)')) +
  theme_minimal()
ggsave(width = 4, height = 4, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_froot_leaf_ratio_Solution.png')


# `froot_leaf_ratio` (scaled) ########################

# Here, we removed the "per unit dry mass" normalization before calculating the
#   Fine root C : Leaf C ratio
# There are only 14 species total, so no need to do this separately for each PFT
require(bbmle)
nll.lnorm <- function (mu, sigma) {
  -sum(dlnorm(x = filter(df.froot.leaf.ratio.summary,
      !is.na(froot.leaf.ratio2), PFT == 'WSV')$froot.leaf.ratio2,
    meanlog = mu, sdlog = sigma, log = T))
}
fit1 <- mle2(nll.lnorm, start = list(mu = -0.01, sigma = 0.22), optimizer = 'nlminb')

df.sweep <- expand.grid(mu = seq(-3, 1, 0.01), sigma = seq(1, 4, 0.05)) %>%
  group_by(mu, sigma) %>%
  mutate(loglik = -nll.lnorm(mu, sigma))

ggplot(df.sweep, mapping = aes(x = mu, y = sigma)) +
  geom_tile(aes(fill = loglik)) +
  geom_contour(aes(z = loglik), binwidth = 5, color = 'black') +
  geom_point(aes(x = mu, y = sigma), shape = 10, size = 5, color = 'black',
    data = data.frame(mu = -1.66, sigma = 2.55, loglik = NA)) +
  geom_vline(aes(xintercept = mu), color = 'black', linetype = 'dashed',
    data = data.frame(mu = 0.51)) +
  scale_fill_viridis_c() +
  scale_x_continuous(expand = c(0, 0)) +
  scale_y_continuous(expand = c(0, 0)) +
  labs(x = 'Log10(Mean)', y = 'Log10(Std. Deviation)',
    fill = 'Log-\nLikelihood') +
  theme_minimal()
ggsave(width = 5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_froot_leaf_ratio_SCALED_MLE.png')

plot(density(df.froot.leaf.ratio.summary$froot.leaf.ratio2, na.rm = T),
  type = 'l', lwd = 2, xlim = c(0, 5), ylim = c(0, 1.5), bty = 'n',
  xlab = 'Fine Root-Leaf C Ratio', ylab = 'Density', main = '')
plot.function(function (x) { dlnorm(x, meanlog = -1.66, sdlog = 2.55) },
  from = 0, to = 6, type = 'l', bty = 'n', lwd = 2, col = 'red', add = T)
plot.function(function (x) { dlnorm(x, meanlog = 0.51, sdlog = 2.55) },
  from = 0, to = 6, type = 'l', bty = 'n', lwd = 2, col = 'blue', add = T)
plot.function(function (x) { dnorm(x, mean = 1.34, sd = 3.1) },
  from = 0, to = 6, type = 'l', bty = 'n', lwd = 2, col = 'green', add = T)

# plot(NA, NA, xlim = c(0, 5), ylim = c(0, 0.8), bty = 'n')
# for (i in 1:11) {
#   froot.leaf.ratio <- c(1.2,1.1,1.7,1.1,1.1,1,1.3,1.8,1.8,2.6,2)
#   plot.function(function (x) { dnorm(x, mean = froot.leaf.ratio[i], sd = 0.5) },
#     from = 0, to = 5, add = T)
# }


# `leaf_mr_base` #####################################

df.leaf.mr.base <- read.csv(unz('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_leaf_mr_base.zip', 'TRY_leaf_mr_base.txt'),
    sep = '\t', fileEncoding = 'latin1') %>%
  # For some reason, str_split won't work unless it is within groups
  group_by(AccSpeciesName) %>%
  mutate(Genus = str_to_title(str_split(AccSpeciesName, ' ', simplify = T)[1])) %>%
  ungroup() %>%
  left_join(df.attrs, by = 'ObservationID') %>%
  # left_join(df.attrs.temp, by = 'ObservationID') %>%
  PFT.pipeline %>%
  mutate(PFT = ifelse(PFT1 == '',
    ifelse(PFT2 == '',
      ifelse(PFT3 == '', NA, PFT3), PFT2), PFT1)) %>%
  select(ObservationID, AccSpeciesName, TraitName, DataName,
    OrigValueStr, OrigUnitStr, StdValue, UnitName, PFT, Reference, Genus)

# There are only 16 data points on the LeafR(Light):LeafR(Dark) ratio...
#   looks like the ratio generally falls within a broad range: [0.2, 0.6]
with(filter(df.leaf.mr.base,
    TraitName == 'Leaf respiration rate in light per leaf respiration rate in the dark'),
  sort(as.numeric(OrigValueStr)))
with(filter(df.leaf.mr.base,
    TraitName == 'Leaf respiration rate in light per leaf respiration rate in the dark'),
  median(as.numeric(OrigValueStr))) # Median value: 45%

df.leaf.mr.base.pft <- read.csv('~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/TRY/TRY_leaf_mr_base_PFT_from_MCD12Q1.csv') %>%
  MCD12.pipeline

# Join the MODIS MCD12Q1 PFT data for those Genera not already assigned PFTs
df.leaf.mr.base.joined <- df.leaf.mr.base %>%
  filter(!is.na(StdValue)) %>%
  left_join(df.leaf.mr.base.pft, by = 'ObservationID') %>%
  mutate(PFT = ifelse(is.na(PFT), PFT.mapped, PFT))

# Consolidate functional groups into PFTs, then aggregate traits by species, taking
#   the majority PFT class
df.leaf.mr.base.by.species <- bind_rows(
  filter(df.leaf.mr.base.joined, PFT %in% c('ENF', 'EBF', 'DNF', 'DBF', 'GRS', 'CRO')),
  mutate(filter(df.leaf.mr.base.joined, PFT %in% c('ENF', 'EBF', 'DNF', 'DBF', 'MF', 'Broadleaf')),
    PFT = 'MF'),
  mutate(filter(df.leaf.mr.base.joined, PFT %in% c('Shrub', 'CSH')),
    PFT = 'CSH'),
  mutate(filter(df.leaf.mr.base.joined, PFT %in% c('Shrub', 'OSH')),
    PFT = 'OSH'),
  mutate(df.leaf.mr.base.joined, PFT = 'WSV'),
  mutate(df.leaf.mr.base.joined, PFT = 'SAV')) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv)) %>%
  # Remove varieties and trailing whitespace
  mutate(AccSpeciesName = str_remove(str_remove(AccSpeciesName, 'var\\..*'), ' $')) %>%
  group_by(AccSpeciesName, Genus, TraitName) %>%
  summarize(StdValue = median(StdValue, na.rm = T),
    PFT = discrete.mode(PFT))

df.leaf.mr.base.by.species %>%
  filter(TraitName == 'Leaf respiration rate in the dark per leaf dry mass') %>%
ggplot(mapping = aes(x = PFT, y = StdValue)) +
  geom_boxplot() +
  coord_flip() +
  labs(y = latex2exp::TeX('Leaf dark respiration rate $(umol\\,C\\,g^{-1}\\,s^{-1})$')) +
  theme_linedraw()
ggsave(width = 4.5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_LeafDarkRespirationRate.png')

# Extract trait names
df.leaf.mr.base.summary <- df.leaf.mr.base.by.species %>%
  select(AccSpeciesName, TraitName, StdValue, Genus, PFT) %>%
  mutate(TraitName = str_extract(TraitName, '(Leaf respiration rate|Leaf carbon)')) %>%
  ungroup()

df.leaf.mr.base.final <- df.leaf.mr.base.summary %>%
  filter(TraitName == 'Leaf respiration rate') %>%
  rename(LeafR.umol.g.s = StdValue) %>%
  select(-TraitName) %>%
  left_join(filter(df.leaf.mr.base.summary, TraitName == 'Leaf carbon'),
    by = c('AccSpeciesName', 'Genus', 'PFT')) %>%
  rename(LeafC.mg.g = StdValue) %>%
  select(-TraitName) %>%
  # Convert leaf respiration rate from (umol C) to (kg C) %>%
  mutate(LeafR.kg.g.s = ((LeafR.umol.g.s / 1e6) * C.MOLAR.MASS) / 1e3) %>%
  # Convert from a seconds-rate to a daily rate
  mutate(LeafR.kg.g.day = LeafR.kg.g.s * 60 * 60 * 24) %>%
  # Convert from a Dry mass ratio to a Carbon content ratio
  mutate(LeafR.kg.mg.day = LeafR.kg.g.day / LeafC.mg.g) %>%
  # Finally, convert C (denominator) units
  mutate(LeafR.kg.kg.day = LeafR.kg.mg.day * 1e6) %>%
  select(AccSpeciesName, Genus, PFT, LeafR.kg.kg.day) %>%
  filter(!is.na(LeafR.kg.kg.day))

# Going to combine Evergreen canopies; combine Deciduous canopies
df.leaf.mr.base.recombined <- bind_rows(
  mutate(filter(df.leaf.mr.base.final, PFT %in% c('ENF', 'EBF')), PFT = 'ENF'),
  mutate(filter(df.leaf.mr.base.final, PFT %in% c('ENF', 'EBF')), PFT = 'EBF'),
  mutate(filter(df.leaf.mr.base.final, PFT %in% c('DNF', 'DBF')), PFT = 'DNF'),
  mutate(filter(df.leaf.mr.base.final, PFT %in% c('DNF', 'DBF')), PFT = 'DBF'),
  filter(df.leaf.mr.base.final, !(PFT %in% c('ENF', 'DNF', 'EBF', 'DBF')))) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv))

ggplot(df.leaf.mr.base.recombined,
  mapping = aes(x = PFT, y = 0.45 * LeafR.kg.kg.day)) +
  geom_boxplot() +
  coord_flip() +
  labs(y = latex2exp::TeX('45% of Leaf dark respiration per unit leaf C ($kg\\,C\\,\\[kg\\,C\\]^{-1}\\,d^{-1}$)')) +
  theme_linedraw() +
  theme(axis.title.x = element_text(size = 10))
ggsave(width = 4.5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_leaf_mr_base.png')

require(bbmle)
results <- matrix(nrow = 11, ncol = 5)
i <- 1
for (pft in levels(df.leaf.mr.base.recombined$PFT)) {
  nll.lnorm <- function (mu, sigma) {
    -sum(dlnorm(x = 0.45 * filter(df.leaf.mr.base.recombined, PFT == pft)$LeafR.kg.kg.day,
      meanlog = mu, sdlog = sigma, log = T))
  }
  nll.norm <- function (mu, sigma) {
    -sum(dnorm(x = 0.45 * filter(df.leaf.mr.base.recombined, PFT == pft)$LeafR.kg.kg.day,
      mean = mu, sd = sigma, log = T))
  }
  fit1 <- mle2(nll.lnorm, start = list(mu = -4.36, sigma = 0.71))
  fit2 <- mle2(nll.norm, start = list(mu = 0.03, sigma = 0.03))
  lrt = 2 * (-nll.lnorm(coef(fit1)[1], coef(fit1)[2]) - -nll.lnorm(-4.36, 0.71))
  p.value <- 1 - pchisq(lrt, 2)
  print(sprintf('LRT (%s): %.2f (p-value: %.3f)', pft, lrt, p.value))
  print(sprintf('AIC (LogN): %.1f', AIC(fit1)))
  print(sprintf('AIC (Norm): %.1f', AIC(fit2)))
  results[i,] <- c(p.value, AIC(fit1), AIC(fit2), coef(fit1))
  i <- i + 1
  rm(fit1, fit2, p.value, lrt)
}
# LRT indicates that ENF (p-value < 0.001), EBF (p-value < 0.001),
#   DNF (p-value = 0.011), DBF (p-value = 0.011), MF (p-value < 0.001),
#   GRS (p-value < 0.001), and CRO (p-value = 0.002) are significantly different
#   at alpha = 0.05

tmp <- as.data.frame(results) %>%
  rename(p.value = V1, lognormal.AIC = V2, normal.AIC = V3, mu = V4, sigma = V5) %>%
  mutate(code = c(1:10, 12)) %>%
  left_join(PFT.TABLE, by = 'code')
i <- 1
for (pft in PFT.TABLE$abbrv) {
  plot.function(function (x) { with(filter(tmp, abbrv == pft), dlnorm(x, mu, sigma)) },
    from = 0, to = 0.1, main = 'Maximum likelihood estimate',
    xlab = TeX('45% of Leaf dark respiration per unit leaf C ($kg\\,C\\,\\[kg\\,C\\]^{-1}\\,d^{-1}$)'),
    ylab = 'Density', bty = 'n', lwd = 2,
    col = viridisLite::viridis(11)[i], add = (pft != 'ENF'))
  i <- i + 1
}

nll.lnorm <- function (mu, sigma) {
  # NOTE: Assuming MR in the light is 45% of Dark Respiration
  -sum(dlnorm(x = 0.45 * filter(df.leaf.mr.base.recombined, PFT == 'WSV')$LeafR.kg.kg.day,
    meanlog = mu, sdlog = sigma, log = TRUE))
}
df.sweep <- expand.grid(mu = seq(-5, -0.1, 0.1), sigma = seq(0.5, 4, 0.05)) %>%
  group_by(mu, sigma) %>%
  mutate(loglik = -nll.lnorm(mu, sigma))

ggplot(df.sweep, mapping = aes(x = mu, y = sigma)) +
  geom_tile(aes(fill = loglik)) +
  geom_contour(aes(z = loglik), binwidth = 1e3, color = 'black') +
  geom_point(aes(x = mu, y = sigma), shape = 10, size = 5, color = 'black',
    data = data.frame(mu = -4.36, sigma = 0.71, loglik = NA)) +
  geom_vline(aes(xintercept = mu), color = 'black', linetype = 'dashed',
    data = data.frame(mu = -3.3)) +
  scale_fill_viridis_c() +
  scale_x_continuous(expand = c(0, 0)) +
  scale_y_continuous(expand = c(0, 0)) +
  labs(x = 'Log10(Mean)', y = 'Log10(Std. Deviation)',
    fill = 'Log-\nLikelihood') +
  theme_minimal()
ggsave(width = 5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_leaf_mr_base_MLE.png')

# KDE plot
with(filter(df.leaf.mr.base.final, PFT == 'WSV'),
  plot(density(0.45 * LeafR.kg.kg.day), lwd = 2, bty = 'n',
    xlim = c(0, 0.1), ylim = c(0, 60),
    xlab = latex2exp::TeX('45% of Leaf Dark Respiration Rate $(kg\\,C\\,\\[kg\\,C\\]^{-1}\\,d^{-1})$'),
    main = ''))
plot.function(function (x) { dlnorm(x, meanlog = -4.36, sdlog = 0.71) },
  from = 0, to = 0.1, type = 'l', bty = 'n', lwd = 2, col = 'red', add = T)
plot.function(function (x) { dlnorm(x, meanlog = -3.77, sdlog = 0.85) },
  from = 0, to = 0.1, type = 'l', bty = 'n', lwd = 2, col = 'blue', add = T)
legend('topright',
  legend = c('KDE (Observed)',
    latex2exp::TeX('LogNormal$(\\mu=-4.36,\\sigma=0.71)$'),
    latex2exp::TeX('LogNormal$(\\mu=-3.77,\\sigma=0.85)$ (CRO)')),
  col = c('black', 'red', 'blue'), lwd = 2)

# Create PDFs for each of the fit distributions
test.data <- expand.grid(PFT = c('ENF', 'DBF', 'MF', 'WSV', 'GRS', 'CRO'),
    x = seq(0, 0.2, 0.01)) %>%
  left_join(data.frame(meanlog = mulogs, sdlog = sigmalogs,
      PFT = c('ENF', 'DBF', 'MF', 'WSV', 'GRS', 'CRO')),
    by = 'PFT') %>%
  group_by(PFT) %>%
  mutate(prob = dlnorm(0.45 * x, meanlog = meanlog, sdlog = sdlog))

# WSV represents the "population" distribution; only CRO is significantly
#   different from the MLE estimate for the full dataset
df.leaf.mr.base.recombined %>%
  filter(PFT %in% c('ENF', 'DBF', 'MF', 'WSV', 'GRS', 'CRO')) %>%
ggplot() +
  geom_histogram(aes(x = 0.45 * LeafR.kg.kg.day), bins = 20, fill = 'lightblue') +
  geom_line(aes(x = x, y = prob), size = 0.8,
    data = test.data) +
  scale_x_continuous(expand = c(0, 0), limits = c(0, 0.2),
    breaks = seq(0, 0.2, 0.05)) +
  scale_y_continuous(expand = c(0, 0)) +
  facet_wrap(~ PFT, scales = 'free_y') +
  labs(x = latex2exp::TeX('45% of Leaf Dark Respiration Rate $(kg\\,C\\,\\[kg\\,C\\]^{-1}\\,d^{-1})$'),
    y = 'Count or Density') +
  theme_linedraw() +
  theme(strip.text = element_text(color = 'black', face = 'bold'),
    strip.background = element_blank())
ggsave(width = 6, height = 5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_leaf_mr_base_Solution.png')


# `froot_mr_base` ####################################

# cat ~/Downloads/20988_07052022180913/20988.txt | awk -F "\t" '{ if ($11 == "TraitName" || $11 == "Plant biomass and allometry: Fine root dry mass per plant" || $11 == "Fine root carbon (C) content per fine root dry mass" || $11 == "Fine root respiration rate per fine root dry mass") print }' > /anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_froot_mr_base.txt

df.froot.mr.base <- read.csv(unz('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_froot_mr_base.zip', 'TRY_froot_mr_base.txt'),
    sep = '\t', fileEncoding = 'latin1') %>%
  # For some reason, str_split won't work unless it is within groups
  group_by(AccSpeciesName) %>%
  mutate(Genus = str_to_title(str_split(AccSpeciesName, ' ', simplify = T)[1])) %>%
  ungroup() %>%
  left_join(df.attrs, by = 'ObservationID') %>%
  PFT.pipeline %>%
  mutate(PFT = ifelse(PFT1 == '',
    ifelse(PFT2 == '',
      ifelse(PFT3 == '', NA, PFT3), PFT2), PFT1)) %>%
  select(ObservationID, AccSpeciesName, TraitName, DataName,
    OrigValueStr, OrigUnitStr, StdValue, UnitName, PFT, Reference, Genus)

df.froot.mr.base.pft <- read.csv('~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/TRY/TRY_froot_mr_base_PFT_from_MCD12Q1.csv') %>%
  MCD12.pipeline

# Join the MODIS MCD12Q1 PFT data for those Genera not already assigned PFTs
df.froot.mr.base.joined <- df.froot.mr.base %>%
  # For some reason, "Fine root respiration rate per fine root dry mass"
  #   doesn't have a StdValue
  mutate(StdValue = ifelse(TraitName != 'Fine root respiration rate per fine root dry mass', StdValue,
      # Convert (nmol CO2) to (g C) then to (ug C)
    ifelse(OrigUnitStr == 'nmol_O2/g/s', NA,
      ifelse(OrigUnitStr == 'nmol_CO2/g/s', (as.numeric(OrigValueStr) / 1e9) * C.MOLAR.MASS * 1e6, as.numeric(OrigValueStr))))) %>%
  # We're now working with ug C g-1 s-1
  filter(!is.na(StdValue)) %>%
  left_join(df.froot.mr.base.pft, by = 'ObservationID') %>%
  mutate(PFT = ifelse(is.na(PFT), PFT.mapped, PFT))

# Consolidate functional groups into PFTs, then aggregate traits by species, taking
#   the majority PFT class
df.froot.mr.base.by.species <- bind_rows(
  filter(df.froot.mr.base.joined, PFT %in% c('ENF', 'EBF', 'DNF', 'DBF', 'GRS', 'CRO')),
  mutate(filter(df.froot.mr.base.joined, PFT %in% c('ENF', 'EBF', 'DNF', 'DBF', 'MF', 'Broadleaf')),
    PFT = 'MF'),
  mutate(filter(df.froot.mr.base.joined, PFT %in% c('Shrub', 'CSH')),
    PFT = 'CSH'),
  mutate(filter(df.froot.mr.base.joined, PFT %in% c('Shrub', 'OSH')),
    PFT = 'OSH'),
  mutate(df.froot.mr.base.joined, PFT = 'WSV'),
  mutate(df.froot.mr.base.joined, PFT = 'SAV')) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv)) %>%
  # Remove varieties and trailing whitespace
  mutate(AccSpeciesName = str_remove(str_remove(AccSpeciesName, 'var\\..*'), ' $')) %>%
  group_by(AccSpeciesName, Genus, TraitName) %>%
  summarize(StdValue = median(StdValue, na.rm = T),
    PFT = discrete.mode(PFT))

df.froot.mr.base.by.species %>%
  filter(TraitName == "Fine root respiration rate per fine root dry mass") %>%
ggplot(mapping = aes(x = PFT, y = StdValue)) +
  geom_boxplot() +
  coord_flip() +
  labs(y = latex2exp::TeX('Fine root respiration rate per fine root dry mass $(ug\\,C\\,g^{-1}\\,s^{-1})$')) +
  theme_linedraw()
ggsave(width = 4.5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_FineRootRperFineRootDryMass.png')

# Extract trait names
df.froot.mr.base.summary <- df.froot.mr.base.by.species %>%
  select(AccSpeciesName, TraitName, StdValue, Genus, PFT) %>%
  mutate(TraitName = str_extract(TraitName, '(Fine root respiration|Fine root carbon|Plant biomass)')) %>%
  mutate(TraitName = ifelse(TraitName == 'Plant biomass', 'Fine root dry mass', TraitName)) %>%
  ungroup()

# Can check that the result is the same if we multiply each variable by the
#   dry mass-per-plant, first
# df.froot.mr.base.summary %>%
#   filter(TraitName == 'Fine root respiration') %>%
#   select(-TraitName) %>%
#   rename(FineRootRPerDryMass.ug.g.s = StdValue) %>%
#   left_join(filter(df.froot.mr.base.summary, TraitName == 'Fine root dry mass'),
#     by = c('AccSpeciesName', 'Genus', 'PFT')) %>%
#   select(-TraitName) %>%
#   rename(FineRootDryMass.g = StdValue) %>%
#     left_join(filter(df.froot.mr.base.summary, TraitName == 'Fine root carbon'),
#     by = c('AccSpeciesName', 'Genus', 'PFT')) %>%
#   select(-TraitName) %>%
#   rename(FineRootC.mg.g = StdValue) %>%
#   # Knock out denominator; Convert fine root C to C content (and from mg to kg)
#   mutate(FineRootCContent.kg = (FineRootC.mg.g * FineRootDryMass.g) / 1e6) %>%
#   # Knock out denominator; Respiration now in (ug C s-1)
#   mutate(FineRootR.ug.s = FineRootRPerDryMass.ug.g.s * FineRootDryMass.g) %>%
#   # Convert from (ug C s-1) to (ug C day-1) %>%
#   mutate(FineRootR.ug.day = FineRootR.ug.s * 60 * 60 * 24) %>%
#   # Convert from (ug C day-1) to (kg C day-1) %>%
#   mutate(FineRootR.kg.day = FineRootR.ug.day / 1e9) %>%
#   # Finally, denominate respiration in root C terms
#   mutate(FineRootRPerC.kg.kg.day = FineRootR.kg.day / FineRootCContent.kg) %>%
#   summary

df.froot.mr.base.final <- df.froot.mr.base.summary %>%
  filter(TraitName == 'Fine root dry mass') %>%
  select(-TraitName) %>%
  rename(FineRootDryMass.g = StdValue) %>%
  left_join(filter(df.froot.mr.base.summary, TraitName == 'Fine root respiration'),
    by = c('AccSpeciesName', 'Genus', 'PFT')) %>%
  select(-TraitName) %>%
  rename(FineRootRPerDryMass.ug.g.s = StdValue) %>%
    left_join(filter(df.froot.mr.base.summary, TraitName == 'Fine root carbon'),
    by = c('AccSpeciesName', 'Genus', 'PFT')) %>%
  select(-TraitName) %>%
  rename(FineRootC.mg.g = StdValue) %>%
  # Convert Root-respiration per Dry mass to Root-respiration per Carbon mass
  mutate(FineRootRPerC.ug.mg.s = FineRootRPerDryMass.ug.g.s / FineRootC.mg.g) %>%
  # Convert [ug C (mg C)-1 s-1] to [ug C (mg C)-1 day-1]
  mutate(FineRootRPerC.ug.mg.day = FineRootRPerC.ug.mg.s * 60 * 60 * 24) %>%
  # Convert [ug C (mg C)-1 day-1] to [kg C (kg C)-1 day-1]
  mutate(FineRootRPerC.kg.kg.day = FineRootRPerC.ug.mg.day * (1e6 / 1e9)) %>%
  select(AccSpeciesName, Genus, PFT, FineRootRPerC.kg.kg.day) %>%
  filter(!is.na(FineRootRPerC.kg.kg.day))

# Only 23 species... Most PFTs not represented well
# with(df.froot.mr.base.final, table(PFT))

# KDE plot
with(filter(df.froot.mr.base.final, PFT == 'WSV'),
  plot(density(FineRootRPerC.kg.kg.day), lwd = 2, bty = 'n',
    xlim = c(0, 0.1), ylim = c(0, 60),
    xlab = latex2exp::TeX('Fine Root Respiration per Fine Root Carbon $(kg\\,C\\,\\[kg\\,C\\]^{-1}\\,d^{-1})$'),
    main = ''))
plot.function(function (x) { dlnorm(x, meanlog = -4.21, sdlog = 0.84) },
  from = 0, to = 0.1, type = 'l', bty = 'n', lwd = 2, col = 'red', add = T)
legend('topright',
  legend = c('KDE (Observed)',
    latex2exp::TeX('LogNormal$(\\mu=-4.21,\\sigma=0.84)$')),
  col = c('black', 'red'), lwd = 2)

# Maximum-likelihood fit of Log-normal distribution
require(bbmle)
nll.lnorm <- function (mu, sigma) {
  -sum(dlnorm(x = filter(df.froot.mr.base.final, PFT == 'WSV')$FineRootRPerC.kg.kg.day,
    meanlog = mu, sdlog = sigma, log = TRUE))
}
fit1 <- mle2(nll.lnorm, start = list(mu = -3.8, sigma = 1))
summary(fit1)

df.sweep <- expand.grid(mu = seq(-5, -0.1, 0.1), sigma = seq(0.5, 4, 0.05)) %>%
  group_by(mu, sigma) %>%
  mutate(loglik = -nll.lnorm(mu, sigma))

ggplot(df.sweep, mapping = aes(x = mu, y = sigma)) +
  geom_tile(aes(fill = loglik)) +
  geom_contour(aes(z = loglik), binwidth = 10, color = 'black') +
  geom_point(aes(x = mu, y = sigma), shape = 10, size = 5, color = 'black',
    data = data.frame(mu = -4.2, sigma = 0.84, loglik = NA)) +
  geom_vline(aes(xintercept = mu), color = 'black', linetype = 'dashed',
    data = data.frame(mu = -3.8)) +
  scale_fill_viridis_c() +
  scale_x_continuous(expand = c(0, 0)) +
  scale_y_continuous(expand = c(0, 0)) +
  labs(x = 'Log10(Mean)', y = 'Log10(Std. Deviation)',
    fill = 'Log-\nLikelihood') +
  theme_minimal()
ggsave(width = 5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_froot_mr_base_MLE.png')

ggplot(df.froot.mr.base.final) +
  geom_histogram(aes(x = FineRootRPerC.kg.kg.day), bins = 20, fill = 'lightblue') +
  geom_function(fun = function (x) { dlnorm(x, meanlog = -4.2, sdlog = 0.84) },
    size = 0.8) +
  geom_text(mapping = aes(x = x, y = y), label = latex2exp::TeX('LogNormal$(-4.20, 0.84)$'),
    data = data.frame(x = 0.04, y = 42.5)) +
  scale_x_continuous(expand = c(0, 0), limits = c(0, 0.1),
    breaks = seq(0, 0.08, 0.02)) +
  scale_y_continuous(expand = c(0, 0), limits = c(0, 50),
    breaks = seq(10, 100, 10)) +
  labs(x = latex2exp::TeX('Fine Root Respiration per Fine Root Carbon $(kg\\,C\\,\\[kg\\,C\\]^{-1}\\,d^{-1})$'),
    y = 'Count or Density') +
  theme_minimal() +
  theme(axis.title.x = element_text(size = 9))
ggsave(width = 4, height = 4, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_froot_mr_base_Solution.png')


# `livewood_mr_base` ###########################################

# cat ~/Downloads/21115_20052022145340/21115.txt | awk -F "\t" '{ if ($11 == "TraitName" || $11 == "Plant biomass and allometry: Stem dry mass per plant" || $11 == "Stem carbon (C) content per stem dry mass" || $11 == "Stem respiration rate per stem dry mass") print '} > TRY_stem_respiration.txt

df.livewood.mr.base <- read.csv(unz('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_stem_respiration.zip', 'TRY_stem_respiration.txt'),
    sep = '\t', fileEncoding = 'latin1') %>%
  # For some reason, str_split won't work unless it is within groups
  group_by(AccSpeciesName) %>%
  mutate(Genus = str_to_title(str_split(AccSpeciesName, ' ', simplify = T)[1])) %>%
  ungroup() %>%
  left_join(df.attrs, by = 'ObservationID') %>%
  PFT.pipeline %>%
  mutate(PFT = ifelse(PFT1 == '',
    ifelse(PFT2 == '',
      ifelse(PFT3 == '', NA, PFT3), PFT2), PFT1)) %>%
  select(ObservationID, AccSpeciesName, TraitName, DataName,
    OrigValueStr, OrigUnitStr, StdValue, UnitName, PFT, Reference, Genus)

df.livewood.mr.base.pft <- read.csv('~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/TRY/TRY_livewood_mr_base_PFT_from_MCD12Q1.csv') %>%
  MCD12.pipeline

# Join the MODIS MCD12Q1 PFT data for those Genera not already assigned PFTs
df.livewood.mr.base.joined <- df.livewood.mr.base %>%
  # For some reason, "Stem respiration rate per stem dry mass"
  #   doesn't have a StdValue
  mutate(StdValue = ifelse(TraitName != 'Stem respiration rate per stem dry mass', StdValue,
    # Convert (nmol CO2 g-1) to (g C g-1) then to (mg C g-1)
    ifelse(OrigUnitStr == 'nmol g-1 s-1', (as.numeric(OrigValueStr) / 1e9) * C.MOLAR.MASS * 1e3,
      # Convert (umol CO2 kg-1) to (g C kg-1), which is equivalent to (mg C g-1)
      ifelse(OrigUnitStr == 'umol kg-1 DW s-1', (as.numeric(OrigValueStr) / 1e6) * C.MOLAR.MASS, as.numeric(OrigValueStr))))) %>%
  # We're now working with g C mg-1 s-1
  filter(!is.na(StdValue)) %>%
  left_join(df.livewood.mr.base.pft, by = 'ObservationID') %>%
  mutate(PFT = ifelse(is.na(PFT), PFT.mapped, PFT))

# Consolidate functional groups into PFTs, then aggregate traits by species, taking
#   the majority PFT class
df.livewood.mr.base.by.species <- bind_rows(
  filter(df.livewood.mr.base.joined, PFT %in% c('ENF', 'EBF', 'DNF', 'DBF', 'GRS', 'CRO')),
  mutate(filter(df.livewood.mr.base.joined, PFT %in% c('ENF', 'EBF', 'DNF', 'DBF', 'Broadleaf')),
    PFT = 'MF'),
  mutate(filter(df.livewood.mr.base.joined, PFT %in% c('Shrub', 'CSH')),
    PFT = 'CSH'),
  mutate(filter(df.livewood.mr.base.joined, PFT %in% c('Shrub', 'OSH')),
    PFT = 'OSH'),
  mutate(df.livewood.mr.base.joined, PFT = 'WSV'),
  mutate(df.livewood.mr.base.joined, PFT = 'SAV')) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv)) %>%
  # Remove varieties and trailing whitespace
  mutate(AccSpeciesName = str_remove(str_remove(AccSpeciesName, 'var\\..*'), ' $')) %>%
  group_by(AccSpeciesName, Genus, TraitName) %>%
  summarize(StdValue = median(StdValue, na.rm = T),
    PFT = discrete.mode(PFT))

df.livewood.mr.base.by.species %>%
  filter(TraitName == "Stem respiration rate per stem dry mass") %>%
ggplot(mapping = aes(x = PFT, y = StdValue)) +
  geom_boxplot() +
  coord_flip() +
  labs(y = latex2exp::TeX('Stem respiration rate per fine root dry mass $(g\\,C\\,mg^{-1}\\,s^{-1})$')) +
  theme_linedraw()
ggsave(width = 4.5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_StemRperStemDryMass.png')

# Extract trait names
df.livewood.mr.base.summary <- df.livewood.mr.base.by.species %>%
  select(AccSpeciesName, TraitName, StdValue, Genus, PFT) %>%
  mutate(TraitName = str_extract(TraitName, '(Stem respiration|Stem carbon|Plant biomass)')) %>%
  mutate(TraitName = ifelse(TraitName == 'Plant biomass', 'Stem dry mass', TraitName)) %>%
  ungroup() %>%
  spread(key = TraitName, value = StdValue) %>%
  # Convert Stem respiration rate from Dry mass denomination to Carbon denomination
  mutate(StemR.mgC.mgC.s = `Stem respiration` / `Stem carbon`) %>%
  # Convert Stem respiration from per-second to per-day
  mutate(StemR.kgC.kgC.day = StemR.mgC.mgC.s * 60 * 60 * 24) %>%
  filter(!is.na(StemR.kgC.kgC.day))

# Only 20 unique species, about half ENF and half DBF,
#   so we're going to need to use the population distribution
# with(df.livewood.mr.base.summary, length(unique(AccSpeciesName)))

# KDE plot
with(filter(df.livewood.mr.base.summary, PFT == 'WSV'),
  plot(density(StemR.kgC.kgC.day), lwd = 2, bty = 'n',
    xlim = c(0, 0.06), ylim = c(0, 60),
    xlab = latex2exp::TeX('Stem Respiration per Stem Carbon $(kg\\,C\\,\\[kg\\,C\\]^{-1}\\,d^{-1})$'),
    main = ''))
with(filter(df.livewood.mr.base.summary, PFT == 'WSV'), rug(StemR.kgC.kgC.day))
plot.function(function (x) { dlnorm(x, meanlog = -5.27, sdlog = 1.54) },
  from = 0, to = 0.1, type = 'l', bty = 'n', lwd = 2, col = 'red', add = T)
legend('topright',
  legend = c('KDE (Observed)',
    latex2exp::TeX('LogNormal$(\\mu=-5.27,\\sigma=1.54)$')),
  col = c('black', 'red'), lwd = 2)

# Maximum-likelihood fit of Log-normal distribution
require(bbmle)
nll.lnorm <- function (mu, sigma) {
  -sum(dlnorm(x = filter(df.livewood.mr.base.summary, PFT == 'WSV')$StemR.kgC.kgC.day,
    meanlog = mu, sdlog = sigma, log = TRUE))
}
fit1 <- mle2(nll.lnorm, start = list(mu = -4.3, sigma = 1.6))
summary(fit1)

df.sweep <- expand.grid(mu = seq(-6, -0.1, 0.1), sigma = seq(0.5, 4, 0.05)) %>%
  group_by(mu, sigma) %>%
  mutate(loglik = -nll.lnorm(mu, sigma))

ggplot(df.sweep, mapping = aes(x = mu, y = sigma)) +
  geom_tile(aes(fill = loglik)) +
  geom_contour(aes(z = loglik), binwidth = 50, color = 'black') +
  geom_point(aes(x = mu, y = sigma), shape = 10, size = 5, color = 'black',
    data = data.frame(mu = -5.27, sigma = 1.54, loglik = NA)) +
  geom_vline(aes(xintercept = mu), color = 'black', linetype = 'dashed',
    data = data.frame(mu = -3.8)) +
  scale_fill_viridis_c() +
  scale_x_continuous(expand = c(0, 0)) +
  scale_y_continuous(expand = c(0, 0)) +
  labs(x = 'Log10(Mean)', y = 'Log10(Std. Deviation)',
    fill = 'Log-\nLikelihood') +
  theme_minimal()
ggsave(width = 5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_livewood_mr_base_MLE.png')
