# Process the TRY database export(s), looking at additional traits related to
#   Specific Leaf Area (SLA)

library(dplyr, warn.conflicts = F)
library(tidyr)
library(stringr)
library(ggplot2)
library(latex2exp)
library(bbmle, warn.conflicts = F)
options(stringsAsFactors = F)
Sys.setlocale("LC_ALL", "C")

CO2.MOLAR.MASS <- 44.009 # grams per mol
C.MOLAR.MASS <- 12.0107 # grams per mol

discrete.mode <- function (x) {
  u <- unique(x)
  tab <- tabulate(match(x, u))
  u[tab == max(tab)]
}

PFT.TABLE <- data.frame(code = c(1:10, 12),
  abbrv = c('ENF', 'EBF', 'DNF', 'DBF', 'MF', 'CSH', 'OSH', 'WSV', 'SAV', 'GRS', 'CRO'))
PFT.TABLE$abbrv <- with(PFT.TABLE, ordered(abbrv, levels = abbrv))

source('~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/workflows/Y2021_MOD16_MOD17_calibration/20220505_0001_MOD17_NPP_calibration_priors_TRY_database_plant_genera.R')


# Revisiting SLA ###############################################################

df.sla.attrs <- read.csv(unz('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_SLA2.zip', 'TRY_SLA2.txt'),
    sep = '\t', fileEncoding = 'latin1', quote = "") %>%
  select(ObservationID, DataName, OrigValueStr, StdValue) %>%
  # This may not be necessary, as there are no duplicated Observation IDs
  group_by(ObservationID, DataName) %>%
  mutate(StdValue = as.character(StdValue)) %>%
  summarize(OrigValueStr = first(OrigValueStr),
    StdValue = first(StdValue)) %>%
  # Unify the different "value" sources, preferring StdValue
  mutate(value = ifelse(is.na(StdValue) | StdValue == "", OrigValueStr, StdValue)) %>%
  select(ObservationID, DataName, value) %>%
  spread(key = DataName, value = value) %>%
  # Prepare the data to assist in classifying PFT
  rename(Habitat = `Habitat / plot description`,
    VegType = `Clustered vegetation type`) %>%
  mutate(Biome = '',
    Latitude = as.numeric(Latitude), 
    Longitude = as.numeric(Longitude),
    Habitat = str_to_lower(Habitat))

df.sla <- read.csv(unz('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_SLA2.zip', 'TRY_SLA2.txt'),
    sep = '\t', fileEncoding = 'latin1', quote = "") %>%
  # For some reason, str_split won't work unless it is within groups
  group_by(AccSpeciesName) %>%
  mutate(Genus = str_to_title(str_split(AccSpeciesName, ' ', simplify = T)[1])) %>%
  left_join(df.sla.attrs, by = 'ObservationID') %>%
  PFT.pipeline() %>%
  ungroup() %>%
  PFT.pipeline %>%
  mutate(PFT = ifelse(PFT1 == '', 
    ifelse(PFT2 == '',
      ifelse(PFT3 == '', NA, PFT3), PFT2), PFT1)) %>%
  select(-PFT1, -PFT2, -PFT3)

# Then go back and run 20220505_0301
# df.sla %>%
#   select(ObservationID, Latitude, Longitude) %>%
#   filter(!is.na(Latitude) & !is.na(Longitude)) %>%
#   group_by(Latitude, Longitude) %>%
#   summarize(ObservationID = str_flatten(ObservationID, collapse = ',')) %>%
#   filter(nchar(ObservationID) < 10000) %>%
# write.csv(row.names = F, file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/TRY/TRY_SLA2_PFT.csv')

df.sla.pft <- read.csv('~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/TRY/TRY_SLA2_PFT_from_MCD12Q1.csv') %>%
  MCD12.pipeline
 
# Join the MODIS MCD12Q1 PFT data for those Genera not already assigned PFTs
df.sla.joined <- df.sla %>%
  filter(!is.na(StdValue)) %>%
  left_join(df.sla.pft, by = 'ObservationID') %>%
  mutate(PFT = ifelse(is.na(PFT), PFT.mapped, PFT)) %>%
  filter(TraitName != '') %>%
  # "Leaf carbon..." is in units of [g m-2]; convert to [kg m-2] then to [m2 kg-1]
  mutate(StdValue = ifelse(TraitName == 'Leaf carbon (C) content per leaf area', 
    ifelse(StdValue > 0, 1 / (StdValue / 1e3), 0), StdValue))

with(filter(df.sla.joined, TraitName == 'Leaf carbon (C) content per leaf area'), table(PFT))

df.sla.by.species <- df.sla.joined %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv)) %>%
  # Remove varieties and trailing whitespace
  mutate(AccSpeciesName = str_remove(str_remove(AccSpeciesName, 'var\\..*'), ' $')) %>%
  group_by(AccSpeciesName, Genus, TraitName) %>%
  summarize(StdValue = median(StdValue, na.rm = T),
    PFT = discrete.mode(PFT))

# Duplicate OSH for this dataset, because CSH has no entries
df.sla.by.species2 <- bind_rows(
  filter(df.sla.by.species, TraitName == 'Leaf carbon (C) content per leaf area'),
  mutate(filter(df.sla.by.species, TraitName == 'Leaf carbon (C) content per leaf area', PFT == 'OSH'),
    PFT = 'CSH')) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv))

df.sla.by.species2 %>%
ggplot(mapping = aes(x = StdValue)) +
  geom_histogram(binwidth = 20) +
  geom_rug(sides = 't') +
  facet_wrap(~ PFT) +
  scale_x_continuous(expand = c(0, 0)) +
  scale_y_log10(expand = c(0, 0)) +
  labs(x = TeX('Specific Leaf Area ($m^2\\,kg\\,C^{-1}$)'), y = 'Species Count') +
  theme_linedraw() +
  theme(strip.background = element_blank(),
    strip.text = element_text(face = 'bold', color = 'black'))
ggsave(width = 6, height = 4.5, dpi = 300,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_SLA_in_C_terms_by_PFT_histograms_PRINT.png')

df.sla.by.species %>%
  filter(TraitName != 'Leaf carbon (C) content per leaf area') %>%
  filter(StdValue < 200) %>%
ggplot(mapping = aes(x = PFT, y = StdValue)) +
  geom_boxplot()

bind_rows(
  df.sla.by.species %>%
    filter(!is.na(PFT)) %>%
    mutate(TraitName = ifelse(TraitName == 'Leaf carbon (C) content per leaf area', 'Carbon terms', 'Dry mass terms')) %>%
    group_by(PFT, TraitName) %>%
    summarize(SLA = median(StdValue, na.rm = T)) %>%
    spread(key = TraitName, value = SLA),
  df.sla.by.species %>%
    mutate(TraitName = ifelse(TraitName == 'Leaf carbon (C) content per leaf area', 'Carbon terms', 'Dry mass terms')) %>%
    group_by(TraitName) %>%
    summarize(SLA = median(StdValue, na.rm = T)) %>%
    spread(key = TraitName, value = SLA) %>%
    mutate(PFT = 'Overall') %>%
    select(PFT, `Carbon terms`, `Dry mass terms`)) %>%
  left_join(data.frame(PFT = c(as.character(PFT.TABLE$abbrv), 'Overall'),
      `Collection 6.1` = c(15, 26.9, 16.9, 24.7, 22.6, 9.4, 12, 28.8, 28.9, 38, 38, 24.7)),
    by = 'PFT') %>%
  rename(`Collection 6.1` = Collection.6.1) %>%
  select(PFT, `Collection 6.1`, everything()) %>%
knitr::kable(format = 'markdown', digits = 1)

# Maximum likelihood estimation
require(bbmle)
results <- matrix(nrow = 11, ncol = 3)
i <- 1
for (pft in levels(df.sla.by.species2$PFT)) {
  nll.lnorm <- function (mu, sigma) {
    -sum(dlnorm(x = filter(df.sla.by.species2, PFT == pft)$StdValue, 
      meanlog = mu, sdlog = sigma, log = T))
  }
  fit1 <- mle2(nll.lnorm, start = list(mu = 3.43, sigma = 0.58), optimizer = 'nlminb')
  lrt = 2 * (-nll.lnorm(coef(fit1)[1], coef(fit1)[2]) - -nll.lnorm(3.43, 0.58))
  p.value <- 1 - pchisq(lrt, 2)
  print(sprintf('LRT (%s): %.2f (p-value: %.3f)', pft, lrt, p.value))
  results[i,] <- c(p.value, coef(fit1))
  i <- i + 1
  rm(fit1, p.value, lrt)
}

# Shrubs (OSH and CSH) are the only PFTs that don't have any prior observations
#   direct from the TRY database (because I didn't look up their Genus, given
#   very few overall observations); the observations were instead matched to PFTs
#   based on the MCD12Q1 map
with(filter(df.sla.by.species, str_starts(TraitName, 'Leaf carbon'), PFT == 'OSH'),
  table(Genus))

View(filter(df.sla.by.species, str_starts(TraitName, 'Leaf carbon'), PFT == 'OSH'))

# Remove some species added from the MCD12 map that should not be in this class
tmp <- df.sla.by.species %>%
  filter(AccSpeciesName != 'Lepidium latifolium') %>%
  filter(AccSpeciesName != 'Perennial grasses') %>%
  filter(str_starts(TraitName, 'Leaf carbon'), PFT == 'OSH') %>%
  filter(!(Genus %in% c('Apocynum', 'Abronia', 'Aremisia', 'Artemesia', 'Datura', 
    'Equisetum', 'Erodium', 'Euthamia', 'Hypolytrum', 'Macharanthera', 'Oenothera',
    'Oryzopsis', 'Piptatherum', 'Polypogon', 'Schedonorous')))

nll.lnorm <- function (mu, sigma) {
  -sum(dlnorm(x = tmp$StdValue, meanlog = mu, sdlog = sigma, log = T))
}
fit2 <- mle2(nll.lnorm, start = list(mu = 3.43, sigma = 0.58), optimizer = 'nlminb')
# exp(3.21) ~= 24.8
# exp(0.31)
# Median from raw data: 25.4